<?php
/** 
 * Plugin 'ds_db' for the 'ameos_formidable' extension.
 *
 * @author	Jerome Schneider <typo3dev@ameos.com>
 */

class tx_dsdb extends formidable_maindatasource {

	var $oDb = FALSE;

	function &_fetchData($aConfig = array(), $aFilters = array()) {

		$this->initDb();

		$iNumRows = 0;
		$aResults = array();
		
		$aFilters = $this->beforeSqlFilter($aConfig, $aFilters);

		if(($sSql = $this->_getSql($aConfig, $aFilters)) !== FALSE) {
			$this->oForm->_debug(
				$sSql,
				"DATASOURCE:DB [" . $this->aElement["name"] . "]"
			);

			$sSql = $this->beforeSqlExec($sSql, $aConfig, $aFilters);

			$rSql = $this->oForm->_watchOutDB(
				$this->oDb->sql_query($sSql),
				$sSql
			);

			if($rSql) {
				
				$iNumRows = $this->_getTotalNumberOfRows();

				while(($aRs =& $this->oDb->sql_fetch_assoc($rSql)) !== FALSE) {
					$aResults[] = $aRs;
					unset($aRs);
				}
			}
		}

		return array(
			"numrows" => $iNumRows,
			"results" => &$aResults,
		);
	}

	function initDb() {
		if(($aLink = $this->_navConf("/link")) !== FALSE) {
			$this->oDb = t3lib_div::makeInstance("t3lib_db");

			if($this->oForm->isUserObj(($sHost = $aLink["host"]))) {
				$sHost = $this->oForm->_callUserObj($sHost);
			}

			if($this->oForm->isUserObj(($sUser = $aLink["user"]))) {
				$sUser = $this->oForm->_callUserObj($sUser);
			}

			if($this->oForm->isUserObj(($sPassword = $aLink["password"]))) {
				$sPassword = $this->oForm->_callUserObj($sPassword);
			}

			if($this->oForm->isUserObj(($sDbName = $aLink["dbname"]))) {
				$sDbName = $this->oForm->_callUserObj($sDbName);
			}

			$this->oDb->sql_pconnect($sHost, $sUser, $sPassword);
			$this->oDb->sql_select_db($sDbName);
		} else {
			$this->oDb =& $GLOBALS["TYPO3_DB"];
		}
	}

	function beforeSqlExec($sSql, $aConfig, $aFilters) {
		if(($mUserobj = $this->_navConf("/beforesqlexec")) !== FALSE) {
			if($this->oForm->isUserObj($mUserobj)) {
				$sSql = $this->oForm->_callUserObj($mUserobj, array(
					"sql" => $sSql,
					"config" => $aConfig,
					"filters" => $aFilters
				));
			}
		}

		return $sSql;
	}

	function beforeSqlFilter($aConfig, $aFilters) {
		if(($mUserobj = $this->_navConf("/beforesqlfilter")) !== FALSE) {
			if($this->oForm->isUserObj($mUserobj)) {
				$aFilters = $this->oForm->_callUserObj($mUserobj, array(
					"config" => $aConfig,
					"filters" => $aFilters
				));
			}
		}

		return $aFilters;
	}

	function _getSql($aConfig = array(), $aFilters = array()) {
		
		$sSqlBase = "";
		$sSqlFilters = "";
		$sSqlOrderBy = "";
		$sSqlLimit = "";
		$sSqlGroupBy = "";

		if($this->_isFalse("/sql")) {
			return FALSE;
		}

		$sSqlBase = $this->_navConf("/sql");
		
		if($this->oForm->isUserObj($sSqlBase)) {
			$sSqlBase = $this->oForm->_callUserObj($sSqlBase);
		}

		$sSqlBase = trim($sSqlBase);

		if(t3lib_div::isFirstPartOfStr(strtoupper($sSqlBase), "SELECT")) {
			// modify the SQL query to include SQL_CALC_FOUND_ROWS
			$sSqlBase = "SELECT SQL_CALC_FOUND_ROWS " . substr($sSqlBase, strlen("SELECT"));
		} else {
			$this->oForm->mayday("DATASOURCE DB \"" . $this->aElement["name"] . "\" - requires /sql to start with SELECT. Check your XML conf.");
		}

		if(strpos(strtoupper($sSqlBase), "WHERE") === FALSE) {
			$sSqlBase .= " WHERE TRUE ";
		}

		if(!empty($aFilters)) {
			$sSqlFilters = " AND (" . implode(" AND ", $aFilters) . ")";
		}

		if(($sSqlGroupBy = stristr($sSqlBase, "GROUP BY")) !== FALSE) {
			$sSqlBase = str_replace($sSqlGroupBy, "", $sSqlBase);
		} else {
			$sSqlGroupBy = "";
		}

		if(array_key_exists("sortcolumn", $aConfig) && trim($aConfig["sortcolumn"]) != "") {
			
			$sSqlOrderBy = " ORDER BY  " . $aConfig["sortcolumn"] . " ";

			if(array_key_exists("sortdirection", $aConfig) && trim($aConfig["sortdirection"]) != "") {
				if(strtoupper($aConfig["sortdirection"]) === "ASC" || strtoupper($aConfig["sortdirection"]) === "DESC") {
					$sSqlOrderBy .= " " . strtoupper($aConfig["sortdirection"]);
				}
			}
		}

		$aLimit = $this->_getRecordWindow(
			$aConfig["page"],
			$aConfig["perpage"]
		);

		$sSqlLimit = $aLimit["sql"];

		return $sSqlBase . " " . $sSqlFilters . " " . $sSqlGroupBy . " " . $sSqlOrderBy . " " . $sSqlLimit;
	}

	function _getTotalNumberOfRows() {
		return $this->oForm->_navConf(
			"/nbrows",
			$this->oDb->sql_fetch_assoc(
				$this->oForm->_watchOutDB(
					$this->oDb->sql_query(
						"SELECT FOUND_ROWS() as nbrows"
					)
				)
			)
		);
	}
}


	if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/ds_db/api/class.tx_dsdb.php"])	{
		include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/ds_db/api/class.tx_dsdb.php"]);
	}
?>
