<?php
/** 
 * Plugin 'rdt_upload' for the 'ameos_formidable' extension.
 *
 * @author	Jerome Schneider <typo3dev@ameos.com>
 */

require_once(PATH_t3lib."class.t3lib_basicfilefunc.php");

class tx_rdtupload extends formidable_mainrenderlet {
	
	var $aUploaded = FALSE;	// array if file has just been uploaded

	function checkPoint($aPoints) {
		if(in_array("after-init-datahandler", $aPoints)) {
			$this->manageFile();
		}
	}
	
	function _render() {
		$sValue = $this->oForm->oDataHandler->getThisFormData($this->_getName());
		$sLabel = $this->oForm->_getLLLabel($this->aElement["label"]);
		$sInput = '<input type="file" name="' . $this->_getElementHtmlName() . '" id="' . $this->_getElementHtmlId() . '" ' . $this->_getAddInputParams() . ' />';
		$sInput .= '<input type="hidden" name="' . $this->_getElementHtmlName($this->_getName() . "_backup") . '" value="' . $sValue . '" />';

		$sLis = "<li>" . implode("</li><li>", t3lib_div::trimExplode(",", htmlspecialchars($sValue))) . "</li>";

		$sValuePreview = (trim($sValue) !== "") ? $sValue . "<br />" : "";

		$aRes = array(
			"__compiled" =>  $this->_displayLabel($sLabel) . $sValuePreview . $sInput,
			"input" => $sInput,
			"filelist.csv" => $sValue,
			"filelist.ol" => "<ol>" . $sLis . "</ol>",
			"filelist.ul" => "<ul>" . $sLis . "</ul>",
			"value" => $sValue,
			"value.preview" => $sValuePreview,
		);

		if(!$this->isMultiple()) {
			if(trim($sValue) != "") {
				$aRes["file.webpath"] = $this->oForm->toWebPath($this->getServerPath());
			} else {
				$aRes["file.webpath"] = "";
			}
		}

		return $aRes;
	}

	function getServerPath($sFileName = FALSE) {
		if(($sTargetFile = $this->getTargetFile()) !== FALSE) {
			return $sTargetFile;
		} elseif($sFileName !== FALSE) {
			return $this->getTargetDir() . $sFileName;
		}

		return $this->getTargetDir() . $this->oForm->oDataHandler->getThisFormData($this->_getName());
	}

	function getFullServerPath($sFileName = FALSE) {
		// dummy method for compat with renderlet:FILE and validator:FILE
		return $this->getServerPath($sFileName);
	}

	function manageFile() {

		/*
			ALGORITHM of file management

			0: PAGE DISPLAY:
				1: file has been uploaded
					1.1: moving file to targetdir and setValue(newfilename)
						1.1.1: multiple == TRUE
							1.1.1.1: data *are not* stored in database (creation mode)
								1.1.1.1.1: setValue to backupdata . newfilename
							1.1.1.2: data *are* stored in database
								1.1.1.2.1: setValue to storeddata . newfilename
				2: file has not been uploaded
					2.1: data *are not* stored in database, as it's a creation mode not fully completed yet
						2.1.1: formdata is a string
							2.1.1.1: formdata != backupdata ( value has been set by some server event with setValue )
								2.1.1.1.1: no need to setValue as it already contains what we need
							2.2.1.2: formdata == backupdata
								2.2.1.2.1: setValue to backupdata
						2.1.2: formdata is an array, and error!=0 (form submitted but no file given)
							2.1.2.1: setValue to backupdata
					2.2: data *are* stored in database
							2.2.1: formdata is a string
								2.2.1.1: formdata != storeddata ( value has been set by some server event with setValue )
									2.2.1.1.1: no need to setValue as it already contains what we need
								2.2.1.2: formdata == storeddata
									2.2.1.2.1: setValue to storeddata
							2.2.2: formdata is an array, and error!=0 ( form submitted but no file given)
								2.2.2.1: setValue to storeddata
		*/

		$aData = $this->oForm->oDataHandler->_getThisFormData($this->_getName());

		if(is_array($aData) && $aData["error"] == 0) {
			// a file has just been uploaded

			$oFileTool = t3lib_div::makeInstance("t3lib_basicFileFunctions");

			if(($sTargetFile = $this->getTargetFile()) !== FALSE) {
				$sTargetDir = t3lib_div::dirname($sTargetFile);
				$sName = basename($sTargetFile);
				$sTarget = $sTargetFile;
			} else {
				$sTargetDir = $this->getTargetDir();

				$sName = basename($aData["name"]);
				if($this->oForm->_defaultTrue("/data/cleanfilename", $this->aElement)) {
					$sName = strtolower(
						$oFileTool->cleanFileName($sName)
					);
				}
				
				$sTarget = $sTargetDir . $sName;
				if(!$this->oForm->_defaultFalse("/data/overwrite", $this->aElement)) {
					// rename the file if same name already exists
					
					$sExt = ((strpos($sName,'.') === FALSE) ? '' : '.' . substr(strrchr($sName, "."), 1));

					for($i=1; file_exists($sTarget); $i++) {
						$sTarget = $sTargetDir . substr($sName, 0, strlen($sName)-strlen($sExt)).'['.$i.']'.$sExt;
					}

					$sName = basename($sTarget);
				}
			}

			if(move_uploaded_file($aData["tmp_name"], $sTarget)) {
				// success
				$this->aUploaded = array(
					"dir" => $sTargetDir,
					"name" => $sName,
					"path" => $sTarget,
					"infos" => $aData,
				);

				if($this->isMultiple()) {
					// csv string of file names
					
					if($this->oForm->oDataHandler->_edition() === FALSE) {
						$aPost = $this->oForm->oDataHandler->_P();
						$sCurrent = $aPost[$this->_getName() . "_backup"];
					} else {
						$sCurrent = trim($this->oForm->oDataHandler->_getStoredData($this->_getName()));
					}
					
					if($sCurrent !== "") {
						
						$aCurrent = t3lib_div::trimExplode(",", $sCurrent);
						if(!in_array($sName, $aCurrent)) {
							$aCurrent[] = $sName;
						}
						
						// adding filename to list
						$this->setValue(implode(",", $aCurrent));
					} else {

						// first value in multiple list
						$this->setValue($sName);
					}
				} else {

					// replacing value in list
					$this->setValue($sName);
				}
			}
		} else {

			$aStoredData = $this->oForm->oDataHandler->_getStoredData();

			if((!array_key_exists($this->_getName(), $aStoredData)) || ($this->oForm->oDataHandler->_edition() === FALSE)) {
				$aPost = $this->oForm->oDataHandler->_P();
				$sBackupData = $aPost[$this->_getName() . "_backup"];

				if(is_string($aData)) {
					if(trim($aData) !== $sBackupData) {
						// value has been set by some process (probably a server event) with setValue()
						/* nothing to do, so */
					} else {
						// persisting existing value
						$this->setValue($sBackupData);
					}
				} else {
					// it's an array, and error!=0
						// persisting existing value
					$this->setValue($sBackupData);
				}
			} else {
				
				$sStoredData = $aStoredData[$this->_getName()];

				if(is_string($aData)) {
					// $aData is a string

					if(trim($aData) !== $sStoredData) {
						// value has been set by some process (probably a server event) with setValue()
						/* nothing to do, so */
					} else {
						// persisting existing value
						$this->setValue($sStoredData);
					}
				} else {
					// it's an array, and error!=0
						// persisting existing value
					$this->setValue($sStoredData);
				}
			}
		}
	}

	function getTargetDir() {

		$oFileTool = t3lib_div::makeInstance("t3lib_basicFileFunctions");

		if($this->oForm->isUserObj(($sTargetDir = $this->_navConf("/data/targetdir/")))) {
			$sTargetDir = $this->oForm->_callUserObj($sTargetDir);
		}

		return t3lib_div::fixWindowsFilePath(
			$oFileTool->slashPath(
				$oFileTool->rmDoubleSlash(
					$sTargetDir
				)
			)
		);
	}

	function getTargetFile() {
		if(($mTargetFile = $this->_navConf("/data/targetfile")) !== FALSE) {
			if($this->oForm->isUserObj($mTargetFile)) {
				$mTargetFile = $this->oForm->_callUserObj($mTargetFile);
			}

			if(is_string($mTargetFile) && trim($mTargetFile) !== "") {
				return $this->oForm->toServerPath(trim($mTargetFile));
			}
		}

		return FALSE;
	}

	function isMultiple() {
		return $this->oForm->_defaultFalse("/data/multiple", $this->aElement);
	}

	function deleteFile($sFile) {
		$aValues = t3lib_div::trimExplode(",", $this->oForm->oDataHandler->getThisFormData($this->_getName()));
		unset($aValues[array_search($sFile, $aValues)]);
		@unlink($this->getFullServerPath($sFile));
		$this->setValue(implode(",", $aValues));
	}
}


	if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/rdt_upload/api/class.tx_rdtupload.php"])	{
		include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/rdt_upload/api/class.tx_rdtupload.php"]);
	}
?>