<?php
/** 
 * Plugin 'va_file' for the 'ameos_formidable' extension.
 *
 * @author	Luc Muller <typo3dev@ameos.com>
 */


class tx_vafile extends formidable_mainvalidator {
	
	var $oFileFunc = null; //object for basics file function
	
	function _doTheMagic($aInfos, $aConf, $aElement) {
		
		$sElementName = $this->oForm->_navConf("/name/", $aElement);
		$bError = FALSE;
		
		if($this->oForm->oDataHandler->_isSubmitted()) {

			$aFormData = $this->oForm->oDataHandler->_getFlatFormData();
			$sFileName = $aFormData[$sElementName];

			$this->oFileFunc = t3lib_div::makeInstance("t3lib_basicFileFunctions");

			if($aElement["type"] === "FILE") {	// renderlet:FILE
				$sFileName = strtolower($this->oFileFunc->cleanFileName($sFileName));
			} elseif($aElement["type"] === "UPLOAD") {
				// managing multiple, if set

				if($sFileName !== "") {
					if($this->oForm->aORenderlets[$sElementName]->isMultiple()) {
						$aFileList = t3lib_div::trimExplode(",", $sFileName);
						$sFileName = array_pop($aFileList);	// last one, and remove from list; will be added later if valid
					}
				}
			}

			if($sFileName !== "") {

				if(($mValue = $this->oForm->_navConf("/extension", $aConf)) !== FALSE) {

					$aValues = t3lib_div::trimExplode(",",$mValue["value"]);

					if(!$this->_checkFileExt($sFileName, $aValues, $sElementName)) {
						$message = $this->oForm->_getLLLabel($this->oForm->_navConf("/message", $mValue));
						$this->oForm->_declareValidationError($sElementName, "FILE:extension", $message);

						$bError = TRUE;
					}
				}
				
				if(($mValue = $this->oForm->_navConf("/filesize", $aConf)) !== FALSE) {
					
					if(!$this->_checkFileSize($sFileName, $mValue["value"], $sElementName)) {
						$message = $this->oForm->_getLLLabel($this->oForm->_navConf("/message", $mValue));
						$this->oForm->_declareValidationError($sElementName, "FILE:filesize", $message);

						$bError = TRUE;
					}
				}

				if(($mValue = $this->oForm->_navConf("/filesizekb", $aConf)) !== FALSE) {
					
					if(!$this->_checkFileSizeKb($sFileName, $mValue["value"], $sElementName)) {
						$message = $this->oForm->_getLLLabel($this->oForm->_navConf("/message", $mValue));
						$this->oForm->_declareValidationError($sElementName, "FILE:filesizekb", $message);

						$bError = TRUE;
					}
				}

				if(($mValue = $this->oForm->_navConf("/filesizemb", $aConf)) !== FALSE) {
					
					if(!$this->_checkFileSizeMb($sFileName, $mValue["value"], $sElementName)) {
						$message = $this->oForm->_getLLLabel($this->oForm->_navConf("/message", $mValue));
						$this->oForm->_declareValidationError($sElementName, "FILE:filesizemb", $message);

						$bError = TRUE;
					}
				}

				if($bError === TRUE && $aElement["type"] === "UPLOAD") {
					if($this->oForm->aORenderlets[$sElementName]->isMultiple()) {
						// current filenamehas been remove from aFileList by previous array_pop
						$this->oForm->aORenderlets[$sElementName]->setValue(implode(", ", $aFileList));
					} else {
						$this->oForm->aORenderlets[$sElementName]->setValue("");
					}
				}
			}
		}
	}
	
	function _checkFileExt($sFileName,$aValues, $sName) {
		
		if($sFileName !== "") {
			
			$aFileName = explode(".",$sFileName);
			$sExt = array_pop($aFileName);
			$sExt = strtolower(str_replace(".", "", $sExt));
			
			foreach($aValues as $key=>$val) {
				
				$val = strtolower(str_replace(".", "", $val));
				
				if($val === $sExt) {
					return TRUE;
				}
			}

			// no match, unlink
			$this->_unlink(
				$this->_getFullServerPath($sName, $sFileName)
			);
			
			return FALSE;
		}

		return TRUE;
	}

	function _checkFileSizeKb($sFileName, $iMaxFileSize, $sName) {
		return $this->_checkFileSize($sFileName, $iMaxFileSize, $sName, "kilobyte");
	}

	function _checkFileSizeMb($sFileName, $iMaxFileSize, $sName) {
		return $this->_checkFileSize($sFileName, $iMaxFileSize, $sName, "megabyte");
	}

	function _checkFileSize($sFileName, $iMaxFileSize, $sName, $sType = "byte") {

		if(!empty($sFileName)) {

			$sFullPath = $this->_getFullServerPath($sName, $sFileName);
			require_once(PATH_t3lib . "class.t3lib_basicfilefunc.php");
			$aInfos = t3lib_basicFileFunctions::getTotalFileInfo($sFullPath);

			switch($sType) {
				case "kilobyte": {
					$iMaxFileSize = $iMaxFileSize * 1024;
					break;
				}
				case "megabyte": {
					$iMaxFileSize = $iMaxFileSize * 1024 * 1024;
					break;
				}
			}

			if(intval($aInfos["size"]) <= intval($iMaxFileSize)) {
				return TRUE;
			} else {
				$this->_unlink($sFullPath);
				return FALSE;
			}
		}

		return TRUE;
	}
   
	function _getFullServerPath($sName, $sFileName) {
		return $this->oForm->aORenderlets[$sName]->getFullServerPath($sFileName);
	}
	
	function _unlink($sFullPath) {
		@unlink($sFullPath);
	}
}


	if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/va_file/api/class.tx_vafile.php"])	{
		include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/base/va_file/api/class.tx_vafile.php"]);
	}
?>
