<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2007 Jerome Schneider (typo3dev@ameos.com)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Formidable API
 *
 * @author	Jerome Schneider <typo3dev@ameos.com>
 */

define("AMEOSFORMIDABLE_EVENT_SUBMIT_FULL",		1);
define("AMEOSFORMIDABLE_EVENT_SUBMIT_REFRESH",	-1);
define("AMEOSFORMIDABLE_EVENT_SUBMIT_TEST",		-2);
define("AMEOSFORMIDABLE_EVENT_SUBMIT_DRAFT",	-3);
define("AMEOSFORMIDABLE_EVENT_SUBMIT_CLEAR",	-4);
define("AMEOSFORMIDABLE_EVENT_SUBMIT_SEARCH",	-5);

define("AMEOSFORMIDABLE_LEXER_VOID",			"AMEOSFORMIDABLE_LEXER_VOID");
define("AMEOSFORMIDABLE_LEXER_FAILED",			"AMEOSFORMIDABLE_LEXER_FAILED");
define("AMEOSFORMIDABLE_LEXER_BREAKED",			"AMEOSFORMIDABLE_LEXER_BREAK");

require_once(PATH_t3lib . "class.t3lib_parsehtml.php");
require_once(PATH_t3lib . "class.t3lib_parsehtml_proc.php");
require_once(PATH_formidable . "api/class.mainobject.php");
require_once(PATH_formidable . "api/class.maindatasource.php");
require_once(PATH_formidable . "api/class.mainvalidator.php");
require_once(PATH_formidable . "api/class.maindatahandler.php");
require_once(PATH_formidable . "api/class.mainrenderer.php");
require_once(PATH_formidable . "api/class.mainrenderlet.php");
require_once(PATH_formidable . "api/class.mainactionlet.php");
require_once(PATH_formidable . "api/class.jslayer.php");
require_once(PATH_formidable . "api/class.json.php");


class tx_ameosformidable {

	var $bInited				= FALSE;
	var $bInitFromTs			= FALSE;
	var $_xmlPath				= null;
	var $_xmlData				= null;
	var $_aConf					= null;

	var $sExtPath				= null;	// server abs path to formidable
	var $sExtRelPath			= null;	// server path to formidable, relative to server root
	var $sExtWebPath			= null;	// web path to formidable

	var $_aValidators			= null;
	var $_aDataSources			= null;
	var $_aDataHandlers			= null;
	var $_aRenderers			= null;
	var $_aRenderlets			= null;
	var $_aActionlets			= null;

	var $oRenderer				= null;
	var $oDataHandler			= null;
	var $aORenderlets			= array();
	var $aODataSources			= array();

	var $oSandBox				= null;		// stores sandbox for xml-level user-defined "macros"
	var $oJs					= null;
	var $oJson					= null;		// library for manipulation of "JavaScript Object Notation (JSON)" documents
	var $aInitTasksUnobtrusive	= array();
	var $aInitTasks				= array();
	var $aInitTasksOutsideLoad	= array();	// tinyMCE cannot be init'd within Event.observe(window, "load", function() {})
	var $aInitTasksAjax			= array();

	var $_aValidationErrors			= array();
	var $_aValidationErrorsInfos	= array();
	var $_aValidationErrorsTypes	= array();

	var $bDebug				= FALSE;
	var $bDebugIP			= FALSE;
	var $aDebug				= array();
	var $start_tstamp		= null;

	var $formid				= "";

	var $_oParent			= null;
	var $oParent			= null;		// alias for _oParent ...

	var $bRendered			= FALSE;
	var $aSteps				= FALSE;	// array of steps for multi-steps forms
	var $_aStep				= FALSE;	// current step extracted from session and stored for further user

	var $sApiVersion		= null;

	var $iForcedEntryId		= FALSE;
	var $_aSubXmlCache		= array();	// cache container for the _getXml method
	var $_aInjectedData		= array();	// contains data to inject in the form at init
	var $aLastTs			= array();
	var $cObj				= null;

	var $bStoreFormInSession	= FALSE;	// whether or not to keep FORM in session for further use (ex: processing ajax events)
	var $bStoreParentInSession	= FALSE;	// whether or not to keep parent in session, if form is stored (ie $bStoreFormInSession==TRUE)

	var $aServerEvents		= array();
	var $aAjaxEvents		= array();
	var $aTempDebug			= array();

	var $aCrossRequests		= array();
	var $aOnloadEvents		= array(	// stores events that have to be thrown at onload ( onDOMReady actually )
		"ajax"		=> array(),
		"client"	=> array()
	);

	var $aSkinManifests		= array();

	var $__aRunningObjects	= array();

	var $oHtml				= FALSE;
	var $aRdtEvents			= array();
	var $aPreRendered		= array();

	var $oMajixEvent		= FALSE;
	var $aUserObjParamsStack = array();
	var $aForcedUserObjParamsStack = array();
	var $aAvailableCheckPoints = array(
		"start",
		"before-compilation",	// kept for back-compat, but should not be here
		"after-compilation",
		"before-init",
		"before-init-renderer",
		"after-init-renderer",
		"before-init-renderlets",
		"after-init-renderlets",
		"before-init-datahandler",
		"after-init-datahandler",
		"after-init",
		"before-render",
		"after-validation",
		"after-render",
		"end",
	);

	var $aAddPostVars = FALSE;
	var $aRawPost = array();

	/*********************************
	 *
	 * FORMidable initialization
	 *
	 *********************************/

	/**
	 * Standard init function
	 * Initializes :
	 * - the reference to the parent Extension ( stored in $this->_oParent )
	 * - the XML conf
	 * - the internal collection of Validators
	 * - the internal collection of DataHandlers
	 * - the internal collection of Renderers
	 * - the internal collection of Renderlets
	 * - the Renderer as configured in the XML conf in the /formidable/control/renderer/ section
	 * - the DataHandler as configured in the XML conf in the /formidable/control/datahandler/ section
	 *
	 * 		//	CURRENT SERVER EVENT CHECKPOINTS ( means when to process the even; ex:  <onclick runat="server" when="after-compilation" /> )
	 * 		//	DEFAULT IS *after-init*
	 * 		//
	 * 		//		start
	 * 		//		before-compilation
	 * 		//		before-compilation
	 * 		//		after-compilation
	 * 		//		before-init
	 * 		//		before-init-renderer
	 * 		//		after-init-renderer
	 * 		//		before-init-renderlets
	 * 		//		after-init-renderlets
	 * 		//		before-init-datahandler
	 * 		//		after-init-datahandler
	 * 		//		after-init
	 * 		//		before-render
	 * 		//		after-render
	 * 		//		end
	 *
	 * @param	object		Parent extension using FORMidable
	 * @param	mixed		Absolute path to the XML configuration file
	 * @param	[type]		$iForcedEntryId: ...
	 * @return	void
	 */
	function init(&$oParent, $mXml, $iForcedEntryId = FALSE) {

		$this->start_tstamp	= t3lib_div::milliseconds();

		require_once(PATH_tslib . "class.tslib_content.php");
		require_once(PATH_t3lib . "class.t3lib_tstemplate.php");
		$this->cObj = t3lib_div::makeInstance('tslib_cObj');

		$this->makeHtmlParser();
		$this->_makeJsonObj();

		if($this->__getEnvExecMode() !== "FE") {	// virtualizing FE for BE and eID (ajax) modes
			$this->__virtualizeFE();
		}

	/***** BASE INIT *****
	*
	*/
		$this->sExtPath = PATH_formidable;
		$this->sExtRelPath = t3lib_extMgm::siteRelPath("ameos_formidable");
		$this->sExtWebPath = t3lib_div::getIndpEnv("TYPO3_SITE_URL") . t3lib_extMgm::siteRelPath("ameos_formidable");



		$this->sApiVersion = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["ext_emconf.php"]["version"];
		$this->sApiVersionInt = t3lib_div::int_from_ver($GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["ext_emconf.php"]["version"]);

		$this->conf =& $GLOBALS["TSFE"]->config["config"]["tx_ameosformidable."];

		$this->_oParent =& $oParent;
		$this->oParent =& $oParent;

		$this->aTempDebug = array();

		$this->_loadDeclaredDataSources();
		$this->_loadDeclaredValidators();
		$this->_loadDeclaredDataHandlers();
		$this->_loadDeclaredRenderers();
		$this->_loadDeclaredRenderlets();
		$this->_loadDeclaredActionlets();


	/***** XML INIT *****
	*
	*/
		/*if(!is_array($mXml)) {
			$this->_xmlPath = $mXml;
			$this->_loadXmlConf();
		} else {
			$this->_aConf = $mXml;
		}*/

		if($this->bInitFromTs === FALSE) {
			$this->_xmlPath = $this->toServerPath($mXml);
			$this->_loadXmlConf();
		} else {
			$this->_aConf = $mXml;
			$this->_aConf = $this->refineTS($this->_aConf);
		}


	/***** DEBUG INIT *****
	*
	*	After this point raw xml data is available ( means before precompilation )
	*	So it is now possible to get some basic config from the xml
	*
	*/

		if(t3lib_div::cmpIP(t3lib_div::getIndpEnv('REMOTE_ADDR'), $GLOBALS['TYPO3_CONF_VARS']['SYS']['devIPmask'])) {
			$this->bDebugIP = TRUE;
		}

		$this->iDebug = intval($this->_navConf("/meta/debug"));

		if($this->iDebug > 0) {
			$this->bDebug = TRUE;
		} else {
			$this->bDebug = $this->_isTrue("/meta/debug/");
			if($this->bDebug) {
				$this->iDebug = 2;	// LIGHT
			}
		}

		$GLOBALS["TYPO3_DB"]->store_lastBuiltQuery = TRUE;
		if($this->bDebug) {
			$GLOBALS["TYPO3_DB"]->debugOutput = TRUE;
		}

		$this->oJs = t3lib_div::makeInstance("formidable_jslayer");
		$this->oJs->_init($this);


	/***** INIT FORM SIGNATURE *****
	*
	*/
		$this->formid = $this->_navConf("/meta/form/formid");
		if($this->isUserObj($this->formid)) {
			$this->formid = $this->_callUserObj($this->formid);
		}

		//$this->uniqueid = $this->formid . "_" . t3lib_div::shortMd5(serialize($this->_aConf) . "cUid:" . $this->_oParent->cObj->data["uid"], 5);

		// CHECKING FORMID COLLISION IN PAGE
		if(!array_key_exists($this->formid, $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["context"]["forms"])) {
			$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["context"]["forms"][$this->formid] = array();
		} else {
			$this->mayday("Two (or more) Formidable are using the same formid '<b>" . $this->formid . "</b>' on this page - cannot continue");
		}

		$this->initAddPost();
	
	/***** INIT DEFAULT (TEMPORARY) DATAHANDLER AND RENDERER *****
	*
	*	These two instances are meant to be destroyed later in the init process
	*	Useful for giving access to objects at precompilation time
	*
	*/
		$this->oDataHandler = $this->_makeDataHandler(
			array(
				"type" => "STANDARD"
			)
		);

		$this->oRenderer = $this->_makeRenderer(
			array(
				"type" => "STANDARD"
			)
		);
		
	/***** INIT EDIT MODE ? *****
	*
	*/
		
		if($iForcedEntryId !== FALSE) {
			// uid "iForcedEntryId" was passed to init() method of formidable

			if(($iCurrentEntryId = $this->oDataHandler->_currentEntryId()) !== FALSE) {
				// there is already an uid asked for edition
					// it has been passed thru POST var myformid[AMEOSFORMIDABLE_ENTRYID]

				if($iForcedEntryId != $iCurrentEntryId) {
					// the old edited uid is different of the newly asked one
						// therefore we'll ask formidable to *force* edition of this iForcedEntryId
						// meaning that formidable should forget field-values passed by POST
						// and re-take the record from DB
					$this->forceEntryId($iForcedEntryId);
				} else {
					// the old edited uid is the same that the newly asked one
					// let formidable handle himself the uid passed thru POST var myformid[AMEOSFORMIDABLE_ENTRYID]
					$iForcedEntryId = FALSE;
				}
			} else {
				$this->forceEntryId($iForcedEntryId);
			}
		} elseif(($mUid = $this->_navConf("/control/datahandler/editentry")) !== FALSE) {

			if($this->isUserObj($mUid)) {
				$mUid = $this->_callUserObj($mUid);
			}

			if(($iCurrentEntryId = $this->oDataHandler->_currentEntryId()) !== FALSE) {
				if($mUid != $iCurrentEntryId) {
					$this->forceEntryId($mUid);
				}
			} else {
				$this->forceEntryId($mUid);
			}

		}

		if($this->iForcedEntryId === FALSE) {
			if(($iTempUid = $this->editionRequested()) !== FALSE) {
				$this->forceEntryId($iTempUid);
			} else {
				$this->forceEntryId($iForcedEntryId);
			}
		}



	/***** XML PRECOMPILATION *****
	*
	*	Applying modifiers on the xml structure
	*	Thus producing new parts of xml and deleting some
	*	To get the definitive XML
	*
	*/
		$this->_aConf = $this->_compileConf(
			$this->_aConf,
			$this->aTempDebug
		);



	/***** GRABBING SERVER EVENTS *****
	*
	*	Grabbing the server and ajax events
	*
	*/
		$this->_grabServerAndAjaxEvents(
			$this->_aConf["elements"]
		);

		$this->checkPoint(
			array(
				"start",
				"before-compilation"	// kept for back-compat, but should not be here
			)
		);



		$this->bReliableXML = TRUE;

		// RELIABLE XML DATA CANNOT BE ACCESSED BEFORE THIS POINT
		// AND THEREFORE NEITHER ALL OBJECTS CONFIGURED BY THIS XML
		// (END OF XML PRE-COMPILATION)

		$this->sDefaultLLLPrefix = $this->_navConf("/meta/defaultlll");
		if($this->isUserObj($this->sDefaultLLLPrefix)) {
			$this->sDefaultLLLPrefix = $this->_callUserObj(
				$this->sDefaultLLLPrefix
			);
		}

		if($this->bDebug) {

			$aTrace		= debug_backtrace();
			$aLocation	= array_shift($aTrace);

			$this->_debug(
				"User called FORMidable<br>"
			.	"<br>&#149; In :<br>"
			.	"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" . $aLocation["file"] . ":" . $aLocation["line"]
			.	"<br>&#149; At :<br>"
			.	"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" . $aLocation["class"] . $aLocation["type"] . $aLocation["function"]
			.	"<br>&#149; With args: <br>"
			.	"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" . $this->_viewMixed($aLocation["args"])
			.	(($this->iForcedEntryId !== FALSE) ? "<br>&#149; Edition of entry " . $this->iForcedEntryId . " requested" : ""),
				"INITIALIZATION OF FORMIDABLE"
			);

			if(!empty($this->aTempDebug["aIncHierarchy"])) {
				$this->_debug(
					$this->aTempDebug["aIncHierarchy"],
					"XML INCLUSION HIERARCHY",
					FALSE
				);
			} else {
				$this->_debug(
					null,
					"NO XML INCLUSION",
					FALSE
				);
			}
		}



		$this->checkPoint(
			array(
				"after-compilation",
				"before-init",
				"before-init-renderer"
			)
		);

		if(($sSandClass = $this->_includeSandBox()) !== FALSE) {
			$this->_createSandBox($sSandClass);
		}

		if(($aOnInit = $this->_navConf("/meta/oninit")) !== FALSE && $this->isUserObj($aOnInit)) {
			$this->_callUserObj($aOnInit);
		}

		$this->_initDataSources();
		$this->_initRenderer();

		$this->checkPoint(
			array(
				"after-init-renderer",
				"before-init-renderlets"
			)
		);

		$this->_initRenderlets();

		$this->checkPoint(
			array(
				"after-init-renderlets",
				"before-init-datahandler"
			)
		);
			
		$this->_initDataHandler(
			$this->iForcedEntryId
		);

		$this->checkPoint(
			array(
				"after-init-datahandler",
				"after-init",
			)
		);

		$this->bInited = TRUE;
	}

	function initAPI(&$oParent) {
		$this->_oParent =& $oParent;
	}

	/**
	 * Makes a persistent instance of an HTML parser
	 * Mainly used for template processing
	 *
	 * @return	void
	 */
	function makeHtmlParser() {
		if($this->oHtml === FALSE) {
			$this->oHtml = t3lib_div::makeInstance("t3lib_parsehtml");
		}
	}

	function useJs() {
		return $this->_defaultTrue("/meta/accessibility/usejs");
	}

	function initAddPost() {
		$aRawPost = $this->_getRawPost($sFormId = FALSE, $bCache = FALSE);
		if(array_key_exists("AMEOSFORMIDABLE_ADDPOSTVARS", $aRawPost) && trim($aRawPost["AMEOSFORMIDABLE_ADDPOSTVARS"]) !== "") {
			if(!is_array(($this->aAddPostVars = $this->oJson->decode($aRawPost["AMEOSFORMIDABLE_ADDPOSTVARS"])))) {
				$this->aAddPostVars = FALSE;
			}
		} else {
			$this->aAddPostVars = FALSE;
		}
	}

	function editionRequested() {
		if($this->aAddPostVars !== FALSE) {
			reset($this->aAddPostVars);
			while(list($sKey, ) = each($this->aAddPostVars)) {
				if(array_key_exists("action", $this->aAddPostVars[$sKey]) && $this->aAddPostVars[$sKey]["action"] === "requestEdition") {
					$sOurSafeLock = $this->_getSafeLock(
						"requestEdition" . ":" . $this->aAddPostVars[$sKey]["params"]["tablename"] . ":" . $this->aAddPostVars[$sKey]["params"]["recorduid"]
					);
					$sTheirSafeLock = $this->aAddPostVars[$sKey]["params"]["hash"];

					if($sOurSafeLock === $sTheirSafeLock) {
						return $this->aAddPostVars[$sKey]["params"]["recorduid"];
					}
				}
			}
		}

		return FALSE;
	}

	/**
	 * Takes an array of typoscript configuration, and adapt it to formidable syntax
	 *
	 * @param	array		$aConf: TS array for application
	 * @return	array		Refined array
	 */
	function refineTS($aConf) {
		$aTemp = array();

		// processing meta
		$aTemp["meta"] = $this->_removeDots($aConf["meta."]);

		// processing control
		$aTemp["control"] = array();
		reset($aConf["control."]);
		while(list($sKey, ) = each($aConf["control."])) {
			if(is_string($aConf["control."][$sKey])) {
				if($sKey === "datahandler") {
					$aTemp["control"]["datahandler"] = array(
						"type" => substr($aConf["control."][$sKey], strlen("datahandler:"))
					);

					if(array_key_exists($sKey . ".", $aConf["control."])) {
						$aTemp["control"]["datahandler"] = t3lib_div::array_merge_recursive_overrule(
							$aTemp["control"]["datahandler"],
							$this->_removeDots($aConf["control."][$sKey . "."])
						);
					}
				} elseif($sKey === "renderer") {
					$aTemp["control"]["renderer"] = array(
						"type" => substr($aConf["control."][$sKey], strlen("renderer:"))
					);

					if(array_key_exists($sKey . ".", $aConf["control."])) {
						$aTemp["control"]["renderer"] = t3lib_div::array_merge_recursive_overrule(
							$aTemp["control"]["renderer"],
							$this->_removeDots($aConf["control."][$sKey . "."])
						);
					}
				}
			} else {
				if($sKey === "actionlets.") {
					$aTemp["control"]["actionlets"] = array();

					reset($aConf["control."][$sKey]);
					while(list($sActKey, ) = each($aConf["control."][$sKey])) {
						if(is_string($aConf["control."][$sKey][$sActKey])) {
							$aTemp["control"]["actionlets"]["actionlet-" . $sActKey] = array(
								"type" => substr($aConf["control."][$sKey][$sActKey], strlen("actionlet:"))
							);

							if(array_key_exists($sActKey . ".", $aConf["control."][$sKey])) {
								$aTemp["control"]["actionlets"]["actionlet-" . $sActKey] = t3lib_div::array_merge_recursive_overrule(
									$aTemp["control"]["actionlets"]["actionlet-" . $sActKey],
									$this->_removeDots($aConf["control."][$sKey][$sActKey . "."])
								);
							}
						}
					}
				} elseif($sKey === "datasources.") {
					$aTemp["control"]["datasources"] = array();

					reset($aConf["control."][$sKey]);
					while(list($sActKey, ) = each($aConf["control."][$sKey])) {
						if(is_string($aConf["control."][$sKey][$sActKey])) {
							$aTemp["control"]["datasources"]["datasource-" . $sActKey] = array(
								"type" => substr($aConf["control."][$sKey][$sActKey], strlen("datasource:"))
							);

							if(array_key_exists($sActKey . ".", $aConf["control."][$sKey])) {
								$aTemp["control"]["datasources"]["datasource-" . $sActKey] = t3lib_div::array_merge_recursive_overrule(
									$aTemp["control"]["datasources"]["datasource-" . $sActKey],
									$this->_removeDots($aConf["control."][$sKey][$sActKey . "."])
								);
							}
						}
					}
				}
			}
		}

		// processing renderlets
		$aTemp["elements"] = array();
		reset($aConf["elements."]);
		while(list($sKey, ) = each($aConf["elements."])) {
			if(is_string($aConf["elements."][$sKey])) {

				$aType = explode(":", $aConf["elements."][$sKey]);

				if($aType[0] === "renderlet") {
					if(array_key_exists($sKey . ".", $aConf["elements."])) {

						$aTemp["elements"][$aType[0] . "-" . $sKey . "-" . rand()] = $this->refineTS_renderlet(
							$aConf["elements."][$sKey],
							$aConf["elements."][$sKey . "."]
						);

					} else {
						$aTemp["elements"][$aType[0] . "-" . $sKey . "-" . rand()] = array("type" => $aType[1]);
					}
				}
			}
		}

		return $aTemp;
	}

	/**
	 * Takes a typoscript conf for a renderlet and refines it to formidable-syntax
	 *
	 * @param	string		$sTen: TS name like: 10 = renderlet:TEXT
	 * @param	array		$aTenDot: TS value of 10. like: 10.value = Hello World !
	 * @return	array		refined conf
	 */
	function refineTS_renderlet($sTen, $aTenDot) {
		$aType = explode(":", $sTen);
		$aRdt = array(
			"type" => $aType[1],
		);

		if(array_key_exists("childs.", $aTenDot)) {
			$aRdt["childs"] = array();

			reset($aTenDot["childs."]);
			while(list($sKey, $sChild) = each($aTenDot["childs."])) {

				$aChild = array();
				if(is_string($sChild)) {
					$aChildType = explode(":", $sChild);
					if($aChildType[0] === "renderlet") {
						if(array_key_exists($sKey . ".", $aTenDot["childs."])) {
							$aChild = $this->refineTS_renderlet(
								$sChild,
								$aTenDot["childs."][$sKey . "."]
							);
						} else {
							$aChild = $this->refineTS_renderlet(
								$sChild,
								array()
							);
						}
					}

					$aRdt["childs"][$aChildType[0] . "-" . $sKey . "-" . rand()] = $aChild;
				}
			}

			unset($aTenDot["childs."]);
		}

		if(array_key_exists("validators.", $aTenDot)) {
			$aRdt["validators"] = array();
			reset($aTenDot["validators."]);
			while(list($sKey, $sValidator) = each($aTenDot["validators."])) {
				$aValidator = array();
				if(is_string($sValidator)) {
					$aValType = explode(":", $sValidator);
					if($aValType[0] === "validator") {

						$aValidator["type"] = $aValType[1];

						if(array_key_exists($sKey . ".", $aTenDot["validators."])) {
							$aValidator = t3lib_div::array_merge_recursive_overrule(
								$aValidator,
								$this->_removeDots($aTenDot["validators."][$sKey . "."])
							);
						}

						$aRdt["validators"]["validator-" . $sKey] = $aValidator;
					}
				}
			}

			unset($aTenDot["validators."]);
		}

		$aRdt = t3lib_div::array_merge_recursive_overrule(
			$aRdt,
			$this->_removeDots($aTenDot)
		);

		reset($aRdt);
		return $aRdt;
	}

	/**
	 * Dispatch calls to checkpoints defined in the whole code of formidable
	 * Similar to hooks
	 *
	 * @param	array		$aPoints: names of the checkpoints to consider
	 * @return	void
	 */
	function checkPoint($aPoints) {
		$this->_processServerEvents($aPoints);
		$this->_processRdtCheckPoints($aPoints);
		$this->_processMetaCheckPoints($aPoints);
	}

	/**
	 * Handles checkpoint-calls on renderlets
	 *
	 * @param	array		$aPoints: names of the checkpoints to consider
	 * @return	void
	 */
	function _processRdtCheckPoints(&$aPoints) {
		if(count($this->aORenderlets) > 0) {
			$aKeys = array_keys($this->aORenderlets);
			while(list(, $sKey) = each($aKeys)) {
				$this->aORenderlets[$sKey]->checkPoint($aPoints);
			}
		}
	}

	function _processMetaCheckPoints($aPoints) {
		
		$aMeta = $this->_navConf("/meta");
		
		$aKeys = array_keys($aMeta);
		reset($aKeys);
		while(list(, $sKey) = each($aKeys)) {
			$sWhen = "";

			if($sKey{0} == "o" && $sKey{1} == "n" && (substr($sKey, 0, 12) === "oncheckpoint")) {
				$sWhen = $this->_navConf("/meta/" . $sKey . "/when");
				if(in_array($sWhen, $aPoints)) {
					if($this->isUserObj($aMeta[$sKey])) {
						$this->_callUserObj($aMeta[$sKey]);
					}
				}
			}
		}
	}

	/**
	 * Includes the sandbox in php context
	 *
	 * @return	mixed		FALSE or name of the sandbox class
	 */
	function _includeSandBox() {

		if(($aBox = $this->_navConf("/control/sandbox")) !== FALSE) {

			if(array_key_exists("extends", $aBox)) {
				$sExtends = (string)$aBox["extends"];
			} else {
				$sExtends = "EXT:ameos_formidable/res/shared/php/class.defaultsandbox.php::formidable_defaultsandbox";
			}

			$aExtends = explode("::", $sExtends);
			if(sizeof($aExtends) == 2) {
				$sFile = t3lib_div::getFileAbsFileName($aExtends[0]);
				$sClass = $aExtends[1];

				if(file_exists($sFile) && is_readable($sFile)) {
					ob_start();
					require_once($sFile);
					ob_end_clean();		// output buffering for easing use of php class files that execute something outside the class definition ( like BE module's index.php !!)
				} else {
					$this->mayday("<b>The declared php-FILE for sandbox ('" . $sFile . "') doesn't exists</b>");
				}
			} else {
				
				// trying to auto-determine class-name
				
				$sFile = t3lib_div::getFileAbsFileName($aExtends[0]);
				if(file_exists($sFile) && is_readable($sFile)) {
					$aClassesBefore = get_declared_classes();

					ob_start();
					require_once($sFile);
					ob_end_clean();		// output buffering for easing use of php class files that execute something outside the class definition ( like BE module's index.php !!)
					$aClassesAfter = get_declared_classes();
					
					$aNewClasses = array_diff($aClassesAfter, $aClassesBefore);
					
					if(count($aNewClasses) !== 1) {
						$this->mayday("<b>Cannot automatically determine the classname to use for the sandbox in '" . $sFile . "'</b><br />Please add '::myClassName' after the file-path in the sandbox declaration");
					} else {
						$sClass = array_shift($aNewClasses);
					}
				} else {
					$this->mayday("<b>The declared php-FILE for sandbox ('" . $sFile . "') doesn't exists</b>");
				}

			}

			if(class_exists($sClass)) {

				if($this->isUserObj($aBox)) {

					if(($sPhp = $this->_navConf("/userobj/php", $aBox)) !== FALSE) {

						if(class_exists($sClass)) {
							$sExtends = " extends " . $sClass;
						}

						$sClassName = "formidablesandbox_" . md5($sPhp);	// these 2 lines
						if(!class_exists($sClassName)) {					// allows same sandbox twice or more on the same page

							$sSandClass = <<<SANDBOXCLASS

	class {$sClassName} extends {$sClass} {

		var \$oForm = null;

		{$sPhp}
	}

SANDBOXCLASS;

							$this->__sEvalTemp = array("code" => $sSandClass, "xml" => $aBox);
							set_error_handler(array(&$this, "__catchEvalException"));
							eval($sSandClass);
							unset($this->__sEvalTemp);
							restore_error_handler();
						}

						return $sClassName;
					}
				} else {
					return $sClass;
				}
			} else {
				$this->mayday("<b>The declared php-CLASS for sandbox ('" . $sClass . "') doesn't exists</b>");
			}
		}

		return FALSE;
	}

	/**
	 * Builds a persistent instance of the sandbox
	 *
	 * @param	string		$sClassName: Name of the sandbox class, as returned by _includeSandBox()
	 * @return	void
	 */
	function _createSandBox($sClassName) {
		$this->oSandBox = new $sClassName();
		$this->oSandBox->oForm =& $this;
		if(method_exists($this->oSandBox, "init")) {
			$this->oSandBox->init($this);	// changed: avoid call-time pass-by-reference
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$oParent: ...
	 * @param	[type]		$aSteps: ...
	 * @return	[type]		...
	 */
	function initSteps(&$oParent, $aSteps) {

		$this->aSteps = $aSteps;

		$aExtract = $this->__extractStep();

		$iStep = $this->_getStep();
		$aCurStep = $this->aSteps[$iStep];

		$sPath = $aCurStep["path"];

		if($aExtract === FALSE || ($iEntryId = $aExtract["AMEOSFORMIDABLE_STEP_UID"]) === FALSE) {
			$iEntryId = (array_key_exists("uid", $aCurStep) ? $aCurStep["uid"] : FALSE);
		}

		$this->init(
			$oParent,	// changed: avoid call-time pass-by-reference
			$sPath,
			$iEntryId
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _getStepperId() {

		if($this->aSteps !== FALSE) {
			return md5(serialize($this->aSteps));
		}

		return FALSE;
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _getStep() {

		$aStep = $this->__extractStep();
		if($aStep === FALSE) {
			return 0;
		}

		return $aStep["AMEOSFORMIDABLE_STEP"];
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function __extractStep() {

		$sStepperId = $this->_getStepperId();

		if($this->_aStep === FALSE) {
			if(		array_key_exists("ameos_formidable", $GLOBALS["_SESSION"])
				&&	array_key_exists("stepper", $GLOBALS["_SESSION"]["ameos_formidable"])
				&&	array_key_exists($sStepperId, $GLOBALS["_SESSION"]["ameos_formidable"]["stepper"])) {

				$this->_aStep = $GLOBALS["_SESSION"]["ameos_formidable"]["stepper"][$sStepperId];
				unset($GLOBALS["_SESSION"]["ameos_formidable"]["stepper"][$sStepperId]);
			} else {

				$aP = t3lib_div::_POST();

				if(array_key_exists("AMEOSFORMIDABLE_STEP", $aP) && array_key_exists("AMEOSFORMIDABLE_STEP_HASH", $aP)) {

					if($this->_getSafeLock($aP["AMEOSFORMIDABLE_STEP"]) === $aP["AMEOSFORMIDABLE_STEP_HASH"]) {
						$this->_aStep = array(
							"AMEOSFORMIDABLE_STEP" => $aP["AMEOSFORMIDABLE_STEP"],
							"AMEOSFORMIDABLE_STEP_UID" => FALSE,
						);
					}
				}
			}
		}

		return $this->_aStep;	// FALSE if none set
	}

	/**
	 * Util-method for use in __getEventsInConf
	 *
	 * @param	string		$sName: name of the current conf-key
	 * @return	boolean		TRUE if event (onXYZ), FALSE if not
	 */
	function __cbkFilterEvents($sName) {
		return $sName{0} === "o" && $sName{1} === "n";	// should start with "on", but speed check
	}

	/**
	 * Extracts all events defined in a conf array
	 *
	 * @param	array		$aConf: conf containing events to detect
	 * @return	array		events extracted
	 */
	function __getEventsInConf($aConf) {
		return array_merge(		// array_merge reindexes array
			array_filter(
				array_keys($aConf),
				array($this, "__cbkFilterEvents")
			)
		);
	}

	/**
	 * Extracts and registers server AND ajax events
	 *
	 * @param	array		$aData: internal use only (recursivity)
	 * @return	void
	 */
	function _grabServerAndAjaxEvents($aData = FALSE) {

		// recursive way of doing it

		$sRdtId = FALSE;
		if($aData === FALSE || !is_array($aData)) { $aData = array();}		// initializes local container
																			// is_array if /elements is empty

		reset($aData);
		while(list($sKey, $mVal) = each($aData)) {

			if(($sKey{0} === "r" && t3lib_div::isFirstPartOfStr($sKey, "renderlet")) || ($sKey{0} === "c" && t3lib_div::isFirstPartOfStr($sKey, "column"))) {	// column for ajaxlist
				if($mVal["name"] != "") {
					$sRdtName = $mVal["name"];
				} else {
					$sRdtName = $this->_getAnonymousName(
						$mVal
					);
				}
			}


			if(is_array($mVal)) {

				$aEvents = $this->__getEventsInConf($mVal);

				if(!empty($aEvents)) {
					reset($aEvents);
					while(list(, $sEvent) = each($aEvents)) {

						if(array_key_exists($sEvent, $mVal) && is_array(($xEvent = $this->_navConf($sEvent, $mVal))) && (array_key_exists("runat", $xEvent))) {

							$aEvent = array($sEvent => $mVal[$sEvent]);

							if($xEvent["runat"] == "server") {

								$sEventId = $this->_getServerEventId($sRdtName, $xEvent);	// before any modif to get the *real* eventid

								$aNeededParams = array();

								if(array_key_exists("params", $xEvent) && is_string($xEvent["params"])) {
									$aNeededParams = t3lib_div::trimExplode(",", $xEvent["params"]);
									$xEvent["params"] = $aNeededParams;
								}

								$sWhen = $this->_navConf("/when", $xEvent);
								if($sWhen === FALSE) {
									$sWhen = "end";
								}

								if(!in_array($sWhen, $this->aAvailableCheckPoints)) {
									$this->mayday("SERVER EVENT on <b>" . $sKey . " " . $sRdtName . "</b>: defined checkpoint (when='" . $sWhen . "') does not exists; Available checkpoints are: <br /><br />" . t3lib_div::view_array($this->aAvailableCheckPoints));
								}

								$this->aServerEvents[$sEventId] = array(
									"name" => $sRdtName,
									"eventid" => $sEventId,
									"trigger" => $sEvent,
									"when" => $sWhen,	// default when : end
									"event" => $xEvent,
									"params" => $aNeededParams,
									"raw" => array($sEvent => $xEvent),
								);
							} /*elseif($xEvent["runat"] == "ajax") {

								$sEventId = $this->_getAjaxEventId($sRdtName, $xEvent);

								if($this->_defaultFalse("/needparent", $xEvent) === TRUE) {
									$this->bStoreParentInSession = TRUE;
								}

								$this->aAjaxEvents[$sEventId] = array(
									"eventid" => $sEventId,
									"trigger" => $sEvent,
									"cache" => $this->_defaultTrue("/cache", $xEvent),
									"event" => $xEvent,
								);

								if($sEvent === "onload") {
									$this->aOnloadEvents["ajax"][$sEventId] = array(
										"name" => $sRdtName,
										"eventid" => $sEventId,
										"event" => $xEvent
									);
								}
							}*/
						}
					}
				}

				$this->_grabServerAndAjaxEvents($mVal);
			}
		}
	}

	/**
	 * Creates unique ID for a given server event
	 *
	 * @param	string		$sRdtName: name of the renderlet defining the event
	 * @param	array		$aEvent: conf array of the event
	 * @return	string		Server Event ID
	 */
	function _getServerEventId($sRdtName, $aEvent) {
		return $this->_getSafeLock(
			$sRdtName . serialize($aEvent)
		);
	}

	/**
	 * Creates unique ID for a given ajax event
	 *
	 * @param	string		$sRdtName: name of the renderlet defining the event
	 * @param	array		$aEvent: conf array of the event
	 * @return	string		Ajax Event ID
	 */
	function _getAjaxEventId($sRdtName, $aEvent) {
		return $this->_getServerEventId($sRdtName, $aEvent);	// same HashKey algorithm
	}

	/**
	 * Creates unique ID for a given client event
	 *
	 * @param	string		$sRdtName: name of the renderlet defining the event
	 * @param	array		$aEvent: conf array of the event
	 * @return	string		Client Event ID
	 */
	function _getClientEventId($sRdtName, $aEvent) {
		return $this->_getServerEventId($sRdtName, $aEvent);	// same HashKey algorithm
	}

	/**
	 * Executes triggered server events
	 * Called by checkPoint()
	 *
	 * @param	array		$aTriggers: array of checkpoints names to consider
	 * @return	void
	 */
	function _processServerEvents($aTriggers) {

		$aP = $this->_getRawPost();
		if(array_key_exists("AMEOSFORMIDABLE_SERVEREVENT", $aP) && (trim($aP["AMEOSFORMIDABLE_SERVEREVENT"]) !== "")) {
			if(array_key_exists($aP["AMEOSFORMIDABLE_SERVEREVENT"], $this->aServerEvents)) {
				$aEvent = $this->aServerEvents[$aP["AMEOSFORMIDABLE_SERVEREVENT"]];
				if(in_array($aEvent["when"], $aTriggers)) {

					$this->_callUserObj(
						$aEvent["event"],
						$this->_getServerEventParams()
					);

				}
			}
		} else {
			reset($this->aServerEvents);
			while(list($sKey, ) = each($this->aServerEvents)) {
				if(array_key_exists($this->aServerEvents[$sKey]["name"], $aP)) {
					if(array_key_exists($this->aServerEvents[$sKey]["name"], $this->aORenderlets) && $this->aORenderlets[$this->aServerEvents[$sKey]["name"]]->aObjectType["TYPE"] === "SUBMIT") {
						$aEvent = $this->aServerEvents[$sKey];
						if(in_array($aEvent["when"], $aTriggers)) {
							$this->_callUserObj(
								$aEvent["event"],
								array()
							);
						}
					}
				}
			}
		}
	}

	/**
	 * Get params for the triggered server event
	 *
	 * @return	array		Params
	 */
	function _getServerEventParams() {

		$aPost = $this->_getRawPost();

		if(
			array_key_exists("AMEOSFORMIDABLE_SERVEREVENT_PARAMS", $aPost) &&
			array_key_exists("AMEOSFORMIDABLE_SERVEREVENT_HASH", $aPost) &&
			$this->_getSafeLock($aPost["AMEOSFORMIDABLE_SERVEREVENT_PARAMS"]) == $aPost["AMEOSFORMIDABLE_SERVEREVENT_HASH"]) {

			return unserialize(base64_decode($aPost["AMEOSFORMIDABLE_SERVEREVENT_PARAMS"]));

		} else {
			return array();
		}
	}

	/**
	 * Returns RAW POST+FILES data
	 *
	 * @param	string		$sFormId: optional; if none given, current formid is used
	 * @return	array		POST+FILES data
	 */
	function _getRawPost($sFormId = FALSE, $bCache = TRUE) {

		if($sFormId === FALSE) {
			$sFormId = $this->formid;
		}
		
		if(!array_key_exists($sFormId, $this->aRawPost) || ($bCache === FALSE)) {
			$aPost = t3lib_div::_POST();
			$aPost	= is_array($aPost[$sFormId]) ? $aPost[$sFormId] : array();
			$aFiles	= is_array($GLOBALS["_FILES"][$sFormId]) ? $GLOBALS["_FILES"][$sFormId] : array();
			
			$aAddParams = array();

			if($this->aAddPostVars !== FALSE) {

				reset($this->aAddPostVars);
				while(list($sKey, ) = each($this->aAddPostVars)) {
					if(array_key_exists("action", $this->aAddPostVars[$sKey]) && $this->aAddPostVars[$sKey]["action"] === "formData") {

						reset($this->aAddPostVars[$sKey]["params"]);
						while(list($sParam, $sValue) = each($this->aAddPostVars[$sKey]["params"])) {

							$aAddParams =  t3lib_div::array_merge_recursive_overrule(
								$aAddParams,
								t3lib_div::explodeUrl2Array(
									$sParam . "=" . $sValue,
									TRUE	// multidim ?
								)
							);
						}
					}
				}
			}

			$aRes = t3lib_div::array_merge_recursive_overrule($aPost, $aFiles);
			$aRes = t3lib_div::array_merge_recursive_overrule($aRes, $aAddParams);
			reset($aRes);

			if($bCache === FALSE) {
				return $aRes;
			}

			$this->aRawPost[$sFormId] = $aRes;
		}

		return $this->aRawPost[$sFormId];
	}

	/**
	 * Loads declared libraries, like scriptaculous
	 * To declare: /meta/libs = scriptaculous, lib_x, lib_y
	 *
	 * @return	void
	 */
	function _includeLibraries() {
		$this->oRenderer->_includeLibraries();
		if(($sLibs = $this->_navConf("/meta/libs")) !== FALSE) {
			//debug($sLibs);
			$aLibs = t3lib_div::trimExplode(",", $sLibs);
			reset($aLibs);
			while(list(, $sLib) = each($aLibs)) {
				if($sLib === "scriptaculous") {
					$this->oJs->loadScriptaculous();
				} elseif($sLib === "dragdrop") {
					$this->oJs->loadScriptaculousDragDrop();
				}
			}
		}
	}

	/**
	 * Unsets renderlets having /process = FALSE
	 *
	 * @return	void
	 */
	function _filterUnProcessed() {

		$aTempObjects = $this->aORenderlets;

		reset($aTempObjects);
		while(list($sName,) = each($aTempObjects)) {
			if($this->aORenderlets[$sName]->shouldProcess() === FALSE) {
				$this->unsetRenderlet($sName);
			}
		}
	}

	/**
	 * Unsets the rdt corresponding to the given name
	 * Also unsets it's childs if any, and it's validators-errors if any
	 *
	 * @param	string		$sName: ...
	 * @return	void
	 */
	function unsetRenderlet($sName) {

		if(array_key_exists($sName, $this->aORenderlets)) {
			if($this->aORenderlets[$sName]->mayHaveChilds() && $this->aORenderlets[$sName]->hasChilds()) {
				reset($this->aORenderlets[$sName]->aChilds);
				while(list($sChildName, ) = each($this->aORenderlets[$sName]->aChilds)) {
					$this->unsetRenderlet($sChildName);
				}
			}
		}

		// unsetting events
			// onload events
		
		$aAjaxOnloadEventsKeys = array_keys($this->aOnloadEvents["ajax"]);
		while(list(, $sKey) = each($aAjaxOnloadEventsKeys)) {
			if($this->aOnloadEvents["ajax"][$sKey]["name"] === $sName) {
				unset($this->aOnloadEvents["ajax"][$sKey]);
			}
		}

		unset($this->_aValidationErrors[$sName]);	// removes potentialy thrown validation errors


		unset($this->aORenderlets[$sName]);
		unset($this->oDataHandler->__aFormData[$sName]);
		unset($this->oDataHandler->__aFormDataManaged[$sName]);

		// pre-setting it's render to void
		$this->aPreRendered[$sName] = "";
	}

	/**
	 * Initialize formidable with typoscript
	 *
	 * @param	object		&$oParent: ref to parent object (usualy plugin)
	 * @param	array		$aConf: typoscript array 
	 * @param	integer		$iForcedEntryId: optional; uid to edit, if any
	 * @return	void
	 */
	function initFromTs(&$oParent, $aConf, $iForcedEntryId = FALSE) {
		$this->bInitFromTs = TRUE;
		$this->init($oParent, $aConf, $iForcedEntryId);
	}

	/**
	 * Refine raw conf and:
	 *	-> inserts recursively all includexml declared
	 *	-> inserts recursively all includets declared
	 *	-> apply modifiers declared, if any
	 *	-> remove sections emptied by modifiers, if any
	 *	-> execute xmlbuilders declared, if any
	 *
	 * @param	array		$aConf: array of raw config to refine
	 * @param	[type]		$aTempDebug: internal use
	 * @return	array		refined array of conf
	 */
	function _compileConf($aConf, &$aTempDebug) {

		$aTempDebug["aIncHierarchy"] = array();

		$aConf = $this->_insertSubXml(
			$aConf,
			$aTempDebug["aIncHierarchy"]
		);

		$aConf = $this->_insertSubTS($aConf);
		$aConf = $this->_applyModifiers($aConf);
		$aConf = $this->_deleteEmpties($aConf);	// ????  surveiller

		$aConf = $this->_insertXmlBuilder($aConf);

		$this->_debug($aIncHierarchy, "FORMIDABLE CORE - INCLUSION HIERARCHY");
		return $aConf;
	}

	/**
	 * Executes and inserts conf generated by xmlbuilders, if any declared
	 *
	 * @param	array		$aConf: array of conf to process
	 * @param	array		$aTemp: optional; internal use
	 * @return	array		processed array of conf
	 */
	function _insertXmlBuilder($aConf, $aTemp = array()) {

		reset($aConf);
		while(list($key, $val) = each($aConf)) {

			if(is_array($val)) {

				if($key{0} === "x" && t3lib_div::isFirstPartOfStr($key, "xmlbuilder")) {

					$aTemp = $this->array_add(
						$this->_callUserObj($val),
						$aTemp
					);

				} else {
					$aTemp[$key] = $this->_insertXmlBuilder($val);
				}
			} else {
				$aTemp[$key] = $val;
			}
		}

		return $aTemp;
	}

	/**
	 * Insert conf referenced by includexml tags
	 *
	 * @param	array		$aConf: array of conf to process
	 * @param	array		$aDebug: internal use
	 * @param	string		$sParent: optional; parent xpath
	 * @return	array		processed conf array
	 */
	function _insertSubXml($aConf, &$aDebug, $sParent = FALSE) {

		$aTemp = array();

		if($sParent === FALSE) {
			$sParent = "";
		}

		if(!$aConf) { return array();}

		while(list($key, $val) = each($aConf)) {

			if(is_array($val)) {

				if($key{0} === "i" && t3lib_div::isFirstPartOfStr($key, "includexml")) {

					if(array_key_exists("path", $val)) {
						$sPath = $val["path"];
					} elseif(trim($val["__value"]) !== "") {
						$sPath = $val["__value"];
					} else {
						$sPath = $this->_xmlPath;
					}

					$bInclude = TRUE;

					if(array_key_exists("condition", $val)) {
						$bInclude = $this->_defaultTrue("/condition", $val);
					}

					if($bInclude) {

						$aDebug[] = array(
							$sParent . " 1- " . $sPath,
							"subxml" => array()
						);
						$iNewKey = count($aDebug) - 1;

						$aXml = $this->_getXml(
							//t3lib_div::getFileAbsFileName($sPath),
							$this->toServerPath($sPath),
							TRUE	// subXml, adds virtualroot for parsing
						);

						if(array_key_exists("dynaxml", $val)) {
							$aDynaXml = $val["dynaxml"];
							$aXml = $this->_substituteDynaXml($aXml, $aDynaXml);
						}

						if(array_key_exists("xpath", $val)) {
							$aXml = $this->xPath("XPATH:" . $val["xpath"], $aXml);
						}

						$aTemp = $this->array_add(
							$this->_insertSubXml(
								$aXml,
								$aDebug[$iNewKey]["subxml"],
								$sParent . "/" . $key
							),
							$aTemp
						);

						if(empty($aDebug[$iNewKey]["subxml"])) {
							unset($aDebug[$iNewKey]["subxml"]);
						}
					}
				} else {

					$aInsert = $this->_insertSubXml(
						$val,
						$aDebug,
						$sParent . "/" . $key
					);

					if(array_key_exists($key, $aTemp)) {

						// reindexing the xml array for correct merging
						$counter = 0;
						while(array_key_exists($key . "-" . $counter, $aTemp)) {
							$counter++;
						}

						$aTemp[$key . "-" . $counter] = $aInsert;
					} else {
						$aTemp[$key] = $aInsert;
					}
				}
			} else {

				if($key{0} === "i" && t3lib_div::isFirstPartOfStr($key, "includexml")) {

					$aDebug[] = array(
						$sParent => $val,
						"subxml" => array()
					);

					$iNewKey = count($aDebug) - 1;

					$aXml = $this->_getXml(
						t3lib_div::getFileAbsFileName($val),
						TRUE	// subXml, adds virtualroot for parsing
					);

					$aTemp = $this->array_add(
						$this->_insertSubXml(
							$aXml,
							$aDebug[$iNewKey]["subxml"],
							$sParent . "/" . $key
						),
						$aTemp
					);

					if(empty($aDebug[$iNewKey]["subxml"])) {
						unset($aDebug[$iNewKey]["subxml"]);
					}

				} else {
					$aTemp[$key] = $val;
				}
			}
		}

		return $aTemp;
	}

	/**
	 * Inserts conf declared by includets
	 *
	 * @param	array		$aConf: array of conf to process
	 * @param	array		$aTemp: optional; internal use
	 * @return	array		processed conf array
	 */
	function _insertSubTS($aConf, $aTemp = array()) {

		reset($aConf);
		while(list($key, $val) = each($aConf)) {

			if(is_array($val)) {

				if($key{0} === "i" && t3lib_div::isFirstPartOfStr($key, "includets")) {

					if(array_key_exists("path", $val)) {

						$aTs = $this->_getTS($val);

						$aTemp = $this->array_add(
							$this->_insertSubTS($aTs),
							$aTemp
						);

					} else {
						/* nothing ;) */
					}

				} else {
					$aTemp[$key] = $this->_insertSubTS($val);
				}
			}
			else {

				if($key{0} === "i" && t3lib_div::isFirstPartOfStr($key, "includets")) {

					$aTs = $this->_getTS($val);

					$aTemp = $this->array_add(
						$aTs,
						$aTemp
					);

				} else {
					$aTemp[$key] = $val;
				}
			}
		}

		return $aTemp;
	}

	/**
	 * Utility function for _insertSubTS
	 *
	 * @param	string		$sTSPath: ts path to get
	 * @return	mixed		ts conf
	 */
	function _getTS($sTSPath) {

		return $this->_navConf(
			$sTSPath,
			$this->_removeDots(
				$this->_oParent->conf
			)
		);
	}

	/**
	 * Obsolete method
	 *
	 * @param	array		$aXml: ...
	 * @param	array		$aDynaXml: ...
	 * @return	array		...
	 */
	function _substituteDynaXml($aXml, $aDynaXml) {

		$sXml = t3lib_div::array2xml($aXml);

		reset($aDynaXml);
		while(list(, $aDynaSubst) = each($aDynaXml)) {
			$sXml = str_replace("[" . $aDynaSubst["name"] . "]", $aDynaSubst["value"], $sXml);
		}

		return t3lib_div::xml2array($sXml);
	}

	/**
	 * Removes conf-sections emptied by modifiers, if any
	 *
	 * @param	array		$aConf: array of conf to refine
	 * @return	array		processed conf array
	 */
	function _deleteEmpties($aConf) {

		reset($aConf);
		while(list($sKey, $mValue) = each($aConf)) {
			if(is_array($aConf[$sKey])) {
				if(array_key_exists("empty", $aConf[$sKey])) {
					unset($aConf[$sKey]);
				} else {
					$aConf[$sKey] = $this->_deleteEmpties($aConf[$sKey]);
				}
			}
		}

		reset($aConf);
		return $aConf;
	}

	/**
	 * Utility method for _applyModifiers()
	 *
	 * @param	array		$aSubConf
	 * @return	array		
	 */
	function _applyLocalModifiers($aSubConf) {

		reset($aSubConf);
		if(($aModifiers = $this->_navConf("/modifiers", $aSubConf)) !== FALSE) {

			reset($aModifiers);
			while(list($sModKey, $aModifier) = each($aModifiers)) {

				if($this->_matchConditions($aModifier)) {

					$aSubConf =
						t3lib_div::array_merge_recursive_overrule(
							$aSubConf,
							$aSubConf["modifiers"][$sModKey]["modification"]
						);
				}
			}
		}

		$aSubConf = $this->_deleteEmpties($aSubConf);
		reset($aSubConf);
		return $aSubConf;
	}

	/**
	 * Applies declared modifiers, if any
	 *
	 * @param	array		$aConf: conf to process
	 * @return	array		processed conf
	 */
	function _applyModifiers($aConf) {

		reset($aConf);
		while(list($sKey, $mValue) = each($aConf)) {

			if(is_array($aConf[$sKey])) {
				if($sKey == "modifiers") {
					$aConf[$sKey] = $this->_applyModifiers($aConf[$sKey]);
					$aConf = $this->_applyLocalModifiers($aConf);
					unset($aConf[$sKey]);
				} else {
					$aConf[$sKey] = $this->_applyModifiers($aConf[$sKey]);
				}
			}
		}

		reset($aConf);
		return $aConf;
	}

	/**
	 * Forces datahandler to edit the given uid
	 *
	 * @param	int		$iForcedEntryId: uid to edit
	 * @return	void
	 */
	function _forceEntryId($iForcedEntryId = FALSE) {
		$this->iForcedEntryId = $iForcedEntryId;
	}

	function forceEntryId($iForcedEntryId = FALSE) {
		return $this->_forceEntryId($iForcedEntryId);
	}

	/**
	 * Removes dots in typoscript configuration arrays
	 *
	 * @param	array		$aData: typoscript conf
	 * @param	array		$aTemp: internal use
	 * @param	string		$sParentKey: key in conf currently processed
	 * @return	array		conf without TS dotted notation
	 */
	function _removeDots($aData, $aTemp = FALSE, $sParentKey = FALSE) {

		if($aTemp === FALSE) {
			$aTemp = array();
		}

		while(list($key, $val) = each($aData)) {
			if(is_array($val)) {
				if($sParentKey === "userobj." && $key === "cobj.") {
					$aTemp["cobj"] = $aData["cobj"];
					$aTemp["cobj."] = $aData["cobj."];
				} else {
					$aTemp[substr($key, 0, -1)] = $this->_removeDots($val, FALSE, $key);
				}
			} else {
				$aTemp[$key] = $val;
			}
		}

		return $aTemp;
	}

	/**
	 * Add dots to an array to have a typoscript-like structure
	 *
	 * @param	array		$aData: plain conf
	 * @param	array		$aTemp
	 * @return	array		typoscript dotted conf
	 */
	function _addDots($aData, $aTemp = FALSE) {

		if($aTemp === FALSE) {
			$aTemp = array();
		}

		while(list($key, $val) = each($aData)) {

			if(is_array($val)) {
				$aTemp[$key."."] = $this->_addDots($val);
			}
			else {
				$aTemp[$key] = $val;
			}
		}

		return $aTemp;
	}

	/**
	 * Returns the current version of formidable running
	 *
	 * @return	string		current formidable version number
	 */
	function _getVersion() {
		$_EXTKEY = "ameos_formidable";
		include(PATH_formidable . "ext_emconf.php");
		return $EM_CONF[$_EXTKEY]["version"];
	}

	/**
	 * Initializes the declared datahandler
	 *
	 * @param	int		$iForcedEntryId: optional; uid to edit, if any
	 * @return	void
	 */
	function _initDataHandler($iForcedEntryId = FALSE) {

		//debug($iForcedEntryId, "iciii");
		$aConfDataHandler = $this->_navConf("/control/datahandler/");

		$this->oDataHandler =& $this->_makeDataHandler($aConfDataHandler);

		if($iForcedEntryId === FALSE) {
			$entryId	= $this->oDataHandler->_currentEntryId();
			$forcedId	= FALSE;
		}
		else {
			$entryId	= $iForcedEntryId;
			$forcedId	= TRUE;
		}

		if($entryId !== FALSE){
			$this->oDataHandler->entryId		= $entryId;
			$this->oDataHandler->forcedId		= $forcedId;
		}

		$this->oDataHandler->_initCols();
		/*$this->oDataHandler->_getStoredData();			// initialization

		$this->oDataHandler->_getFormData();			// initialization

		/*$this->oDataHandler->__aFormData = $this->oDataHandler->_processBeforeRender(
			$this->oDataHandler->__aFormData
		);

		$this->oDataHandler->_getFormDataManaged();		// initialization
		*/

		$this->oDataHandler->refreshAllData();

		$this->_debug($this->oDataHandler->__aStoredData, "oDataHandler->__aStoredData initialized with these values");
		$this->_debug($this->oDataHandler->__aFormData, "oDataHandler->__aFormData initialized with these values");
		$this->_debug($this->oDataHandler->__aFormDataManaged, "oDataHandler->__aFormDataManaged initialized with these values");
	}

	/**
	 * Initializes the declared renderer
	 *
	 * @return	void
	 */
	function _initRenderer() {

		$aConfRenderer = $this->_navConf("/control/renderer/");
		$this->oRenderer =& $this->_makeRenderer($aConfRenderer);
	}

	/**
	 * Initializes the declared datasources
	 *
	 * @return	void
	 */
	function _initDataSources() {

		$this->_makeDataSources(
			$this->_navConf("/control/datasources/"),
			"/control/datasources/"
		);
	}

	/**
	 * Initializes the declared datasources
	 *
	 * @param	array		$aConf: conf as given in /control/datasources/*
	 * @param	string		$sXPath: xpath from where the given conf comes
	 * @return	void		...
	 */
	function _makeDataSources($aConf, $sXPath) {

		if(is_array($aConf)) {

			reset($aConf);
			while(list($sElementName, ) = each($aConf)) {

				if($sElementName{0} === "d" && t3lib_div::isFirstPartOfStr($sElementName, "datasource")) {

					$aElement =& $aConf[$sElementName];

					$this->aODataSources[trim($aElement["name"])] = $this->_makeDataSource(
						$aElement,
						$sXPath . $sElementName
					);
				}
			}
		}
	}

	/**
	 * Initializes the declared Renderlets
	 *
	 * @return	void
	 */
	function _initRenderlets() {

		$this->_makeRenderlets(
			$this->_navConf("/elements/"),
			"/elements/",
			$bChilds = FALSE,
			$this	// not used, but required as passing params by ref is not possible with default param value
		);
	}

	/**
	 * Initializes the declared Renderlets
	 *
	 * @param	array		$aConf: array of conf; usually /renderlets/*
	 * @param	string		$sXPath: xpath from where the given conf comes
	 * @param	boolean		$bChilds: TRUE if initializing childs, FALSE if not
	 * @param	boolean		$bOverWrite: if FALSE, two renderlets declared with the same name will trigger a mayday
	 * @return	array		array of references to built renderlets
	 */
	function _makeRenderlets($aConf, $sXPath, $bChilds, &$oChildParent, $bOverWrite = FALSE) {

		$aRdtRefs = array();

		if(is_array($aConf)) {

			reset($aConf);
			while(list($sElementName,) = each($aConf)) {

				if($sElementName{0} === "r" && t3lib_div::isFirstPartOfStr($sElementName, "renderlet")) {

					$aElement =& $aConf[$sElementName];

					if(array_key_exists("name", $aElement) && (trim($aElement["name"]) != "")) {
						$sName = trim($aElement["name"]);
						$bAnonymous = FALSE;
					} else {
						$sName = $this->_getAnonymousName($aElement);
						$this->_aConf["elements"][$sElementName]["name"] = $sName;
						$aElement["name"] = $sName;
						$bAnonymous = TRUE;
					}

					if(!$bAnonymous && !$bOverWrite && array_key_exists($sName, $this->aORenderlets)) {
						$this->mayday("Two (or more) renderlets are using the same name '<b>" . $sName . "</b>' on this form ('<b>" . $this->formid . "</b>') - cannot continue<br /><h2>Inclusions:</h2>" . $this->_viewMixed($this->aTempDebug["aIncHierarchy"]));
					}

					if(($bChilds === FALSE) || $this->_defaultTrue("/process", $aElement)) {
						$this->aORenderlets[$sName] =& $this->_makeRenderlet(
							$aElement,
							$sXPath . $sElementName . "/"
						);

						$this->aORenderlets[$sName]->bSubmitter = FALSE;
						$this->aORenderlets[$sName]->bAnonymous = $bAnonymous;
						$this->aORenderlets[$sName]->bChild = $bChilds;

						if($bChilds === TRUE) {
							$this->aORenderlets[$sName]->oRdtParent =& $oChildParent;
						}

						$aRdtRefs[$sName] =& $this->aORenderlets[$sName];
					}
				}
			}
		}

		return $aRdtRefs;
	}
	/**
	 * Loads the internal _aConf configuration array from the XML file
	 * IMPORTANT NOTE : the root /formidable is deleted, so all pathes shouldn't start with /formidable
	 *
	 * @return	void
	 */
	function _loadXmlConf() {

		$this->_aConf = $this->_getXml($this->_xmlPath);
		$this->_aConf = $this->_aConf["formidable"];	// the root is deleted
	}

	/**
	 * Generates an anonymous name for a renderlet
	 *
	 * @param	array		$aElement: conf of the renderlet
	 * @return	string		anonymous name generated
	 */
	function _getAnonymousName($aElement) {
		return "anonymous_" . $this->_getSafeLock(
			serialize($aElement)
		);
	}

	/**
	 * Obsolete
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function _getMeta($sPath) {
		$aXml = $this->_getXml($sPath);
		return $aXml["formidable"]["meta"];
	}

	/**
	 * Binary-reads a file
	 *
	 * @param	string		$sPath: absolute server path to file
	 * @return	string		file contents
	 */
	function file_readBin($sPath) {
		$sData = "";
		$rFile = fopen($sPath, "rb");
		while(!feof($rFile)) {
			$sData .= fread($rFile, 1024);
		}
		fclose($rFile);

		return $sData;
	}

	/**
	 * Binary-writes a file
	 *
	 * @param	string		$sPath: absolute server path to file
	 * @param	string		$sData: file contents
	 * @param	boolean		$bUTF8: add UTF8-BOM or not ?
	 * @return	void
	 */
	function file_writeBin($sPath, $sData, $bUTF8 = TRUE) {
		$rFile=fopen($sPath, "wb");
		if($bUTF8 === TRUE) {
			fputs($rFile, "\xEF\xBB\xBF" . $sData);
		} else {
			fputs($rFile, $sData);
		}
		fclose($rFile);
	}

	/**
	 * Reads and parse an xml file, and returns an array of XML
	 * Fresh or cached data, depending on $this->conf["cache."]["enabled"]
	 *
	 * @param	string		$sPath: abs server path to xml file
	 * @param	boolean		$isSubXml
	 * @return	array		xml data
	 */
	function _getXml($sPath, $isSubXml = FALSE) {

		if(!file_exists($sPath)) {
			if($isSubXml === FALSE) {
				$this->smartMayday_XmlFile($sPath);
			} else {
				$this->mayday("FORMIDABLE CORE - The given XML file path (<b>'" . $sPath . "'</b>) doesn't exists.");
			}
		} elseif(is_dir($sPath)) {
			$this->mayday("FORMIDABLE CORE - The given XML file path (<b>'" . $sPath . "'</b>) is a directory, and should be a file.");
		} elseif(!is_readable($sPath)) {
			$this->mayday("FORMIDABLE CORE - The given XML file path (<b>'" . $sPath . "'</b>) exists but is not readable.");
		} else {

			$sHash = md5($sPath);

			if(array_key_exists($sHash, $this->_aSubXmlCache)) {
				return $this->_aSubXmlCache[$sHash];
			} else {

				$aConf = array();

				if($this->conf["cache."]["enabled"] == 1) {
					$sProtection = "<?php die('Formidable - Cache protected'); ?><!--FORMIDABLE_CACHE-->";

					//debug(stat($sPath));
					$sHash = md5($sPath . "-" . @filemtime($sPath) . "-" . $this->sApiVersion);
					$sFile = "formidablexmlcache_" . $sHash . ".php";
					$sCacheDir = "ameos_formidable/cache/";
					$sCachePath = PATH_site . "typo3temp/" . $sCacheDir . $sFile;

					if(file_exists($sCachePath)) {
						$aConf = unserialize(
							base64_decode(
								substr($this->file_readBin($sCachePath), strlen($sProtection) + 3)		/* 3 is size of UTF8-header, aka BOM or Byte Order Mark */
							)
						);
						if(is_array($aConf)) {
							reset($aConf);
						}
					}
				}

				if(empty($aConf)) {

					$sXmlData = $this->file_readBin($sPath);
					if(trim($sXmlData) === "") {
						$this->smartMayday_XmlFile($sPath, "FORMIDABLE CORE - The given XML file path (<b>'" . $sPath . "'</b>) exists but is empty.");
					}

					$aMatches = array();
					preg_match("/^<\?xml(.*)\?>/", $sXmlData, $aMatches);

					// Check result
					if(!empty($aMatches)) {
						$sXmlProlog = $aMatches[0];
						$sXmlData = preg_replace("/^<\?xml(.*)\?>/", "", $sXmlData);

						/*ereg('^[[:space:]]*<\?xml[^>]*encoding[[:space:]]*=[[:space:]]*"([^"]*)"',$sXmlProlog, $aParts);

						if($aParts[1]) {
							$sEncoding = trim(strtolower($aParts[1]));
							if($sEncoding === "utf-8" && trim($GLOBALS["TYPO3_CONF_VARS"]['BE']['forceCharset']) === "") {
								// default T3 charset is ISO-8859-1
								// converting from UTF8 to ISO
								$sXmlData = utf8_decode($sXmlData);
							}
						}*/

					} else {
						$sXmlProlog = '<?xml version="1.0" encoding="utf-8" standalone="yes" ?>';
					}
					
					if($isSubXml) {
						$sXmlData = $sXmlProlog . "\n" . "<phparray>" . $sXmlData . "</phparray>";
					} else {
						$sXmlData = $sXmlProlog . "\n" . $sXmlData;
					}


					$aConf = $this->div_xml2array($sXmlData);

					if(is_array($aConf)) {

						if($isSubXml && array_key_exists("phparray", $aConf) && is_array($aConf["phparray"])) {
							$aConf = $aConf["phparray"];
						}

						reset($aConf);

					} else {
						$this->mayday("FORMIDABLE CORE - The given XML file (<b>'" . $sPath . "'</b>) isn't well-formed XML<br>Parser says : <b>" . $aConf . "</b>");
					}

					if($this->conf["cache."]["enabled"] == 1) {

						if(!@is_dir(PATH_site . "typo3temp/" . $sCacheDir)) {
							t3lib_div::mkdir_deep(PATH_site . "typo3temp/", $sCacheDir);
						}

						$this->file_writeBin(
							$sCachePath,
							$sProtection . base64_encode(serialize($aConf)),
							TRUE	// add UTF-8 header
						);
					}
				}

				reset($aConf);
				$this->_aSubXmlCache[$sHash] = $aConf;
				return $aConf;
			}
		}
	}

	/**
	 * Includes and makes instance of the desired object
	 *
	 * @param	string		$sInternalKey: name of the object; something like rdt_text, dh_db, ...
	 * @param	string		$sObjectType: type of the object; something like renderlets, datahandlers, ...
	 * @return	object		Built object or FALSE if failed
	 */
	function _loadObject($sInternalKey, $sObjectType) {

		$aDeclaredObjects =& $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["declaredobjects"][$sObjectType];

		if(array_key_exists($sInternalKey, $aDeclaredObjects)) {

			$aLoadedObjects =& $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"][$sObjectType];

			if(!array_key_exists($sInternalKey, $aLoadedObjects)) {
				$aTemp = array(
					"EXTKEY" => $aDeclaredObjects[$sInternalKey]["key"],
					"TYPE" => $sInternalKey,
					"BASE" => $aDeclaredObjects[$sInternalKey]["base"],
					"OBJECT" => $sObjectType,
				);

				$aTemp["PATH"]			= tx_ameosformidable::_getExtPath($aTemp);
				$aTemp["RELPATH"]		= tx_ameosformidable::_getExtRelPath($aTemp);
				$aTemp["CLASS"]			= "tx_" . str_replace("_", "", $aTemp["EXTKEY"]);
				$aTemp["CLASSPATH"]		= $aTemp["PATH"]  . "api/class." . $aTemp["CLASS"] . ".php";

				if($aTemp["BASE"] === TRUE && file_exists($aTemp["PATH"]  . "ext_localconf.php")) {
					$aTemp["LOCALCONFPATH"]	= $aTemp["PATH"]  . "ext_localconf.php";
				}

				$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"][$sObjectType][$aTemp["TYPE"]] = $aTemp;

				if(file_exists($aTemp["CLASSPATH"])) {
					require_once($aTemp["CLASSPATH"]);
				}

				if(file_exists($aTemp["LOCALCONFPATH"])) {
					require_once($aTemp["LOCALCONFPATH"]);
				}

				$aLoadedObjects[$sInternalKey] = $aTemp;
			}

			if(tx_ameosformidable::__getEnvExecMode() !== "EID") {
				if(isset($this)) {	// allow static calls
					$this->__aRunningObjects[$sObjectType . "::" . $sInternalKey] = array(
						"internalkey" => $sInternalKey,
						"objecttype" => $sObjectType,
					);
				}

				// calls tx_myrdtclass::loaded();
					// params are not passed by ref with call_user_func, so have to pass an array with &
				call_user_func(array($aLoadedObjects[$sInternalKey]["CLASS"], 'loaded'), array("form" => &$this));
			}

			return $aLoadedObjects[$sInternalKey];
		}

		return FALSE;
	}

	/**
	 * Loads the internal collection of available datasources
	 *
	 * @return	void
	 */
	function _loadDeclaredDataSources() {
		$this->_aDataSources = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datasources"];
		reset($this->_aDataSources);
	}

	/**
	 * Loads the internal collection of available datahandlers
	 *
	 * @return	void
	 */
	function _loadDeclaredDataHandlers() {
		$this->_aDataHandlers = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datahandlers"];
		reset($this->_aDataHandlers);
	}

	/**
	 * Loads the internal collection of available renderlets
	 *
	 * @return	void
	 */
	function _loadDeclaredRenderlets() {
		$this->_aRenderlets = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"];
		reset($this->_aRenderlets);
	}

	/**
	 * Loads the internal collection of available renderers
	 *
	 * @return	void
	 */
	function _loadDeclaredRenderers() {
		$this->_aRenderers = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderers"];
		reset($this->_aRenderers);
	}

	/**
	 * Loads the internal collection of available validators
	 *
	 * @return	void
	 */
	function _loadDeclaredValidators() {
		$this->_aValidators = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["validators"];
		reset($this->_aValidators);
	}

	/**
	 * Loads the internal collection of available actionlets
	 *
	 * @return	void
	 */
	function _loadDeclaredActionlets() {
		$this->_aActionlets = $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["actionlets"];
		reset($this->_aActionlets);
	}

	function isTemplateMethod($sToken) {
		$aRes = array();
		$bMatch = (preg_match("@^[\w]+\((.|\n)*\)$@", $sToken, $aRes) !== 0);
		//$this->d($sToken, $aRes);
		return $bMatch;
	}

	function templateDataAsString($mData) {
		if(is_array($mData)) {
			if(array_key_exists("__compiled", $mData)) {
				$mData = $mData["__compiled"];
			} else {
				$mData = "";
			}
		}

		return $mData;
	}

	function executeTemplateMethod($mData, $sCall) {

		$sMethod = substr($sCall, 0, strpos($sCall, "("));
		$sArgs = trim(substr(strstr($sCall, "("), 1, -1));

		$aParams = array();

		if($sArgs !== "") {
			$aArgs = t3lib_div::trimExplode(",", $sArgs);
			reset($aArgs);
			while(list(, $sArg) = each($aArgs)) {
				$sTrimArg = trim($sArg);

				if(($sTrimArg{0} === '"' && $sTrimArg{(strlen($sTrimArg)-1)} === '"')) {
					$sValue = substr($sTrimArg, 1, -1);
					$aParams[] = substr($sTrimArg, 1, -1);
				} else {
					$aParams[] = tx_ameosformidable::resolveForTemplate($sTrimArg);
				}
			}
		}

		switch($sMethod) {
			case "rdt": {
				return $this->aORenderlets[$aParams[0]];
				break;
			}
			case "switch": {

				if($mData === TRUE) {
					return $aParams[0];
				} else {
					return $aParams[1];
				}

				break;
			}
			case "wrapInnerHTML": {
				$sTagName = $this->oHtml->getFirstTagName($mData, TRUE);

				$sTag = $this->oHtml->getFirstTag($mData);
				$sInnerHtml = $this->oHtml->removeFirstAndLastTag($mData);
				$sInnerHtml = str_replace(
					"|",
					$sInnerHtml,
					$aParams[0]
				);

				return $sTag . $sInnerHtml . "</" . $sTagName . ">";
				break;
			}
			case "echo": {
				return $aParams[0];
				break;
			}
			case "equals": {
				if($mData == $aParams[0]) {
					return TRUE;
				}

				return FALSE;
				break;
			}
			case "nl2br": {
				return nl2br(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
/*			case "addClass": {

				$sTag = $this->oHtml->getFirstTag($mData);
				$sTagName = $this->oHtml->getFirstTagName($sTag, TRUE);
				$aAttributes = $this->oHtml->get_tag_attributes($sTag);
				if(array_key_exists("class", $aAttributes)) {
					$aClasses = t3lib_trimExplode(" ", $aAttributes["class"]);
				} else {
					$aClasses = array();
				}

				$aClasses[] = $aParams[0];
				return $aClasses
				//return nl2br(tx_ameosformidable::templateDataAsString($mData));
				break;
			}*/
			case "upper": {
				return strtoupper(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
			case "lower": {
				return strtolower(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
			case "ucfirst": {
				return ucfirst(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
			case "ucwords": {
				return ucwords(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
			case "test": {
				return $this->templateDataAsString($mData) . " !";
				break;
			}
			case "help": {
				
				if(is_array($mData)) {
					$mData = array_keys($mData);
				}

				return tx_ameosformidable::_viewMixed($mData);
				break;
			}
			case "debug": {
				if(is_array($mData) && array_key_exists("help", $mData)) {
					unset($mData["help"]);
				}
				return tx_ameosformidable::_viewMixed($mData);
				break;
			}
			case "displayCond": {
				if(tx_ameosformidable::templateDataAsString($mData) === "") {
					return "display: none;";
				}

				return "";
				break;
			}
			case "concat": {
				return trim(tx_ameosformidable::templateDataAsString($mData) . implode(" ", $aParams));
				break;
			}
			case "wrap": {
				return str_replace(
					"|",
					trim(tx_ameosformidable::templateDataAsString($mData)),
					$aParams[0]
				);
				break;
			}
			/*case "ifNotEmpty": {

				if(tx_ameosformidable::templateDataAsString($mData) === "") {
					return TRUE;
				}

				return $mData;
				break;
			}
			case "ifEmpty": {

				if(tx_ameosformidable::templateDataAsString($mData) === "") {
					return $mData;
				}

				return AMEOSFORMIDABLE_LEXER_BREAKED;
				break;
			}*/
			case "extract" : {

				$aRes = array();
				if(is_array($mData)) {
					reset($aParams);
					while(list(, $sKeyName) = each($aParams)) {
						if(array_key_exists($sKeyName, $mData)) {
							$aRes[$sKeyName] = $mData[$sKeyName];
						}
					}
				}

//				debug($mData);
//				debug($aRes);
				return $aRes;
				break;
			}
			case "implode": {
				if(is_array($mData)) {
					return implode($aParams[0], $mData);
				}

				return "";
				break;
			}
			case "isTrue": {
				if($mData === TRUE) {
					return TRUE;
				}

				return AMEOSFORMIDABLE_LEXER_BREAKED;
				break;
			}
			case "isFalse": {
				if($mData === FALSE) {
					return TRUE;
				}

				return AMEOSFORMIDABLE_LEXER_BREAKED;
				break;
			}
			case "isLoggedIn": {

				if($GLOBALS["TSFE"]->fe_user->user["uid"]){
					return TRUE;
				} else {
					return FALSE;
				}

				break;
			}
			case "substr":
			case "fixed_lgd": {
				return t3lib_div::fixed_lgd(tx_ameosformidable::templateDataAsString($mData),$aParams[0],$aParams[1]);
				break;
			}
			case "fixed_lgd_word": {
				$sStr = tx_ameosformidable::templateDataAsString($mData);
				return substr(
					$sStr,
					0,
					strpos(
						$sStr,
						" ",
						$aParams[0]
					)
				) . $aParams[1];
			}
			case "striptags": {
				return strip_tags(tx_ameosformidable::templateDataAsString($mData));
				break;
			}
			case "formData": {

				if(!empty($aParams)) {
					return $this->oDataHandler->getThisFormData($aParams[0]);
				} else {
					return $this->oDataHandler->getFormData();
				}

				break;
			}
			case "storedData": {
				
				if(!empty($aParams)) {
					return $this->oDataHandler->getStoredData($aParams[0]);
				} else {
					return $this->oDataHandler->getStoredData();
				}

				break;
			}
			case "rteToHtml": {
				return $this->div_rteToHtml(
					tx_ameosformidable::templateDataAsString($mData),
					"",
					""
				);

				break;
			}
			default: {

				if(is_object($mData) && is_string($sMethod) && method_exists($mData, $sMethod)) {
					switch(count($aParams)) {
						case 0: { return $mData->{$sMethod}(); break;}
						case 1: { return $mData->{$sMethod}($aParams[0]); break;}
						case 2: { return $mData->{$sMethod}($aParams[0], $aParams[1]); break;}
						case 3: { return $mData->{$sMethod}($aParams[0], $aParams[1], $aParams[2]); break;}
						case 4: { return $mData->{$sMethod}($aParams[0], $aParams[1], $aParams[2], $aParams[3]); break;}
						case 5: { return $mData->{$sMethod}($aParams[0], $aParams[1], $aParams[2], $aParams[3], $aParams[4]); break;}
					}
				}
			}

		}

		return AMEOSFORMIDABLE_LEXER_FAILED;
	}

	function parseForTemplate($sPath) {

		$sPath = trim($sPath);
		//debug($sPath, "parseForTemplate");

		$iOpened = 0;
		$bInString = FALSE;

		$iCurrent = 0;
		$aCurrent = array(
			0 => array(
				"expr" => "",
				"rec" => FALSE,
				"args" => FALSE
			)
		);

		$sArgs = "";

		$aStr = tx_ameosformidable::str_split($sPath, 1);
		reset($aStr);
		$sLastCar = "";
		while(list(, $sCar) = each($aStr)) {

			if(!$bInString && $sCar === ".") {
				if($iOpened === 0) {
					$iCurrent++;
					$aCurrent[$iCurrent] = array(
						"expr" => "",
						"rec" => FALSE,
						"args" => FALSE
					);
					$sCar = "";
				}
			}

			if($sCar === '"' && $sLastCar !== "\\") {
				$bInString = !$bInString;
			} elseif(!$bInString && $sCar === "(") {
				$iOpened++;
			} elseif(!$bInString && $sCar === ")") {
				$iOpened--;
				$sTrimArg = trim($aCurrent[$iCurrent]["args"]);
				$aCurrent[$iCurrent]["args"] = $sTrimArg;
				if($sTrimArg{0} !== '"' && strpos($sTrimArg, "(") !== FALSE) {
					$aCurrent[$iCurrent]["rec"] = TRUE;
				} else {
					$aCurrent[$iCurrent]["rec"] = FALSE;
				}
			}

			if($iOpened !== 0) {
				if($sCar === "(" && $aCurrent[$iCurrent]["args"] === FALSE) {
					$aCurrent[$iCurrent]["args"] = "";
				} else {
					//debug($sCar, "car:" . $iOpened);
					$aCurrent[$iCurrent]["args"] .= $sCar;
				}
			} else {
				
				if($bInString || ($sCar !== "(" && $sCar !== ")" && $sCar !== " " && $sCar !== "\n" && $sCar !== "\r" && $sCar !== "\t")) {
					if(!$bInString || $sCar !== "\\") {
						$aCurrent[$iCurrent]["expr"] .= $sCar;
					}
				}
			}
			$sLastCar = $sCar;
		}

		//$this->d($aCurrent);
		return $aCurrent;
	}

	function resolveForTemplate($sPath, $aConf = FALSE, $mStackedValue = array()) {

		if($aConf === FALSE && is_object($this)) {
			$aConf = $this->aCurrentTemplateTags;
		}

		$aRes = tx_ameosformidable::parseForTemplate($sPath);
		$aValue = $aConf;

		reset($aRes);
		while(list($i, $aExp) = each($aRes)) {

			if($aValue === AMEOSFORMIDABLE_LEXER_FAILED || $aValue === AMEOSFORMIDABLE_LEXER_BREAKED) {
				// throwing exception to notify that lexer has failed or has breaked
				return $aValue;
			}

			//$this->d("EACH aRES", $aExp);

			$sTrimExpr = trim($aExp["expr"]);

			if($aExp["rec"] === TRUE) {
				if($sTrimExpr{0} == '"' && $sTrimExpr{(strlen($sTrimExpr) - 1)} == '"') {
					$aValue = substr($sTrimExpr, 1, -1);
				} else {

					$aBeforeValue = $aValue;

					$aValue = tx_ameosformidable::resolveForTemplate($aExp["args"], $aConf, $aValue);
					if(!is_array($aValue)) {
						$sExecString = $aExp["expr"] . "(\"" . $aValue . "\")";
					} else {
						$sExecString = $aExp["expr"];
					}

					$aValue = tx_ameosformidable::resolveForTemplate($sExecString, $aBeforeValue);
				}

				$sDebug = $aExp["args"];
			} else {
				if($sTrimExpr{0} == '"' && $sTrimExpr{(strlen($sTrimExpr) - 1)} == '"') {
					$aValue = substr($sTrimExpr, 1, -1);
				} else {

					$sExecString = $aExp["expr"];
					if($aExp["args"] !== FALSE) {
						$sExecString .= "(" . trim($aExp["args"]) . ")";
					}

					if(array_key_exists(($i+1), $aRes)) {
						$aNextExp = $aRes[$i+1];
						$sNextExecString = $aNextExp["expr"];
						if($aNextExp["args"] !== FALSE) {
							$sNextExecString .= "(" . trim($aNextExp["args"]) . ")";
						}
					} else {
						$sNextExecString = FALSE;
					}

					$aValue = tx_ameosformidable::resolveForTemplate_atomic($sExecString, $aValue, $sNextExecString);
				}
				$sDebug = $sExecString;
			}
		}

		return $aValue;
	}

	function resolveForTemplate_atomic($sPath, $aConf = AMEOSFORMIDABLE_LEXER_VOID, $sNextPath = FALSE) {

		if($aConf === AMEOSFORMIDABLE_LEXER_VOID && is_object($this)) {
			$aConf = $this->aCurrentTemplateTags;
		}

		if(is_array($aConf)) {
			reset($aConf);
		}

		$curZone = $aConf;

		if(tx_ameosformidable::isTemplateMethod($sPath)) {
			//debug($sPath, "isTemplateMethod");
			$curZone = tx_ameosformidable::executeTemplateMethod($curZone, $sPath);
			if($curZone === AMEOSFORMIDABLE_LEXER_FAILED || $curZone === AMEOSFORMIDABLE_LEXER_BREAKED) {
				return AMEOSFORMIDABLE_LEXER_FAILED;
			}/* else {
				$this->d("method has returned", $curZone);
			}*/
		} else {
			//debug($sPath, "NOT isTemplateMethod");
			if(is_array($curZone)) {
				if(array_key_exists($sPath, $curZone) && array_key_exists($sPath . ".", $curZone)) {

					// ambiguous case: both "token" and "token." exists in the data array
					/*
						algo:
							if there's a next token asked after this one
								if "nexttoken" exists in "token."
									current zone become "token."
								else
									current zone become "token"
							else
								current zone become "token"
					
					*/

					if($sNextPath !== FALSE) {
						if(array_key_exists($sNextPath, $curZone[$sPath . "."])) {
							$curZone = $curZone[$sPath . "."];
						} else {
							$curZone = $curZone[$sPath];
						}
					} else {
						$curZone = $curZone[$sPath];
					}

				} elseif(array_key_exists($sPath, $curZone)) {
					$curZone = $curZone[$sPath];
				} elseif(array_key_exists($sPath . ".", $curZone)) {
					$curZone = $curZone[$sPath . "."];
				} else {
					return AMEOSFORMIDABLE_LEXER_FAILED;
				}
			} else {
				return $curZone;
			}
		}

		//$this->d($curZone, "curZone");
		return $curZone;
	}

	function _navConf($path, $aConf = -1, $sSep = "/") {

		if($path{0} === $sSep)					{ $path = substr($path, 1);}
		if($path{strlen($path) - 1} === $sSep)	{ $path = substr($path, 0, strlen($path) - 1);}

		$aPath = explode($sSep, $path);
		reset($aPath);

		if($aConf === -1 || !is_array($aConf)) {
			reset($this->_aConf);
			$curZone = $this->_aConf;
		} else {
			reset($aConf);
			$curZone = $aConf;
		}

		$iSize = sizeOf($aPath);
		for($i = 0; $i < $iSize; $i++) {

			if(is_array($curZone) && array_key_exists($aPath[$i], $curZone)) {
				$curZone = $curZone[$aPath[$i]];
				if(is_string($curZone)) {
					if($curZone{0} === "X" && substr($curZone, 0, 6) === "XPATH:") {
						$curZone = $this->xPath($curZone);
					} elseif($curZone{0} === "L" && substr($curZone, 0, 4) === "LLL:") {
						$curZone = $this->_getLLLabel($curZone);
					}
				}
			} else {
				return FALSE;
			}
		}

		return $curZone;
	}

	function navDef($sPath, $mDefault, $aConf = -1) {
		
		if(($aTemp = $this->_navConf($sPath, $aConf)) !== FALSE) {
			return $aTemp;
		}

		return $mDefault;
	}
	/**
	 * Resolves an xpath and returns value pointed by this xpath
	 *
	 * @param	string		$sPath: xpath
	 * @return	mixed
	 */
	function xPath($sPath, $aConf = -1, $bIncludeParent = FALSE) {

		if(is_string($sPath) && $sPath{0} === "X" && substr($sPath, 0, 6) === "XPATH:") {
			
			$sPath = $this->_trimSlashes(
				strtolower(substr($sPath, 6))
			);
			
			if(strpos($sPath, "[")) {
				$aSegments = array();

				if($aConf === -1) {
					$aConf = $this->_aConf;
				}

				$aParts = explode("/", $sPath);
				reset($aParts);
				while(list(, $sPart) = each($aParts)) {
					$aTemp = explode("[", str_replace("]", "", $sPart));
					if(count($aTemp) > 1) {
						// we have to search on a criteria sequence
						$sWhat = $aTemp[0];
						$sTempCrits = $aTemp[1];
						$aTempCrits = t3lib_div::trimExplode(",", $sTempCrits);
						reset($aTempCrits);
						$aCrits = array();
						while(list(, $sTempCrit) = each($aTempCrits)) {
							$aCrit = t3lib_div::trimExplode("=", $sTempCrit);
							$aCrits[$aCrit[0]] = $aCrit[1];
						}

						$aSegments[] = array(
							"what" => $sWhat,
							"crits" => $aCrits
						);
					} else {
						$aSegments[] = array(
							"what" => $sPart,
							"crits" => FALSE,
						);
					}
				}

				$aPossibles = array(
					0 => $aConf
				);


				reset($aConf);
				while(list($iLevel, $aSegment) = each($aSegments)) {
					
					$aNewPossibles = array();
					$aPossKeys = array_keys($aPossibles);
					while(list(, $sPosKey) = each($aPossKeys)) {
						
						$aKeys = array_keys($aPossibles[$sPosKey]);
						reset($aKeys);
						while(list(, $sKey) = each($aKeys)) {
							
							if(substr($sKey, 0, strlen($aSegment["what"])) == $aSegment["what"]) {
								$bMatch = TRUE;
								if($aSegment["crits"] !== FALSE) {
									reset($aSegment["crits"]);								
									while(list($sProp, $sValue) = each($aSegment["crits"])) {
										$bMatch = $bMatch && (array_key_exists(strtolower($sProp), $aPossibles[$sPosKey][$sKey]) && strtolower($aPossibles[$sPosKey][$sKey][$sProp]) == strtolower($sValue));
									}
								}

								if($bMatch) {
									$aNewPossibles[$sKey] = $aPossibles[$sPosKey][$sKey];
								}
							}
						}
					}

					$aPossibles = $aNewPossibles;
				}

				reset($aPossibles);
				return $aPossibles;
			}

			return $this->_navConf($sPath, $aConf);
		}

		return FALSE;
	}



	/**
	 * Alias for _render()
	 *
	 * @return	string
	 */
	function render() {	// alias for _render
		return $this->_render();
	}

	/**
	 * Renders the whole application
	 *
	 * @return	string		Full rendered HTML 
	 */
	function _render() {

		if($this->bInited === FALSE) {
			$this->start_tstamp	= t3lib_div::milliseconds();	// because it has not been initialized yet
			$this->mayday("TRIED TO RENDER FORM BEFORE CALLING INIT() !");
		}

		$this->checkPoint(
			array(
				"before-render",
			)
		);

		$this->bRendered = TRUE;

		$this->_debug($this->oDataHandler->_P(), "RAW POST RETURN");
		$this->_debug($this->oDataHandler->_getFormData(), "_getFormData");
		$this->_debug($this->oDataHandler->_getFormDataManaged(), "_getFormDataManaged");

		$this->oRenderer->renderStyles();

		if($this->oDataHandler->_isSubmitted()) {

			if($this->oDataHandler->_isFullySubmitted()) {

				$this->_debug("", "HANDLING --- FULL --- SUBMIT EVENT");

				// validation of the renderlets
//				$this->_validateElements();
				$this->validateEverything();
				
				$this->_filterUnProcessed();
				$this->_includeLibraries();



				// the datahandler is executed
				$sDH = $this->oDataHandler->_doTheMagic(TRUE);

				// Renderlets are rendered
				$aRendered = array_merge(
					$this->_renderElements(),
					$this->aPreRendered
				);

				if(count($this->_aValidationErrors) > 0) {
					$this->_debug($this->_aValidationErrors, "SOME ELEMENTS ARE NOT VALIDATED");
				} else {
					$this->_debug("", "ALL ELEMENTS ARE VALIDATED");
				}

				// the renderer is executed
				$aHtmlBag = $this->oRenderer->_render(
					$aRendered
				);

				// ACTIONLETS are executed
				if($this->oDataHandler->_allIsValid()) {
					$this->_executeActionlets(
						$aRendered,
						$aHtmlBag["CONTENT"]
					);
				}

			} elseif($this->oDataHandler->_isRefreshSubmitted()) {

				$this->_debug("NO VALIDATION REQUIRED", "HANDLING --- REFRESH --- SUBMIT EVENT");

				$this->_filterUnProcessed();
				$this->_includeLibraries();

				// the datahandler is executed
				$sDH = $this->oDataHandler->_doTheMagic(FALSE);

				// Renderlets are rendered
				$aRendered = array_merge(
					$this->_renderElements(),
					$this->aPreRendered
				);

				// the renderer is executed
				$aHtmlBag = $this->oRenderer->_render(
					$aRendered
				);

			} elseif($this->oDataHandler->_isTestSubmitted()) {

				$this->_debug("VALIDATION REQUIRED ( ONLY )", "HANDLING --- TEST --- SUBMIT EVENT");

				// validation of the renderlets
//				$this->_validateElements();
				$this->validateEverything();

				$this->_filterUnProcessed();
				$this->_includeLibraries();

				// the datahandler is executed
				$sDH = $this->oDataHandler->_doTheMagic(FALSE);

				// Renderlets are rendered
				$aRendered = array_merge(
					$this->_renderElements(),
					$this->aPreRendered
				);

				// the renderer is executed
				$aHtmlBag = $this->oRenderer->_render(
					$aRendered
				);

			} elseif($this->oDataHandler->_isDraftSubmitted()) {

				$this->_debug("NO VALIDATION REQUIRED", "HANDLING --- DRAFT --- SUBMIT EVENT");

				// validation of the renderlets
//				$this->_validateElementsDraft();
				$this->validateEverythingDraft();

				$this->_filterUnProcessed();
				$this->_includeLibraries();

				// the datahandler is executed
				$sDH = $this->oDataHandler->_doTheMagic(TRUE);

				// Renderlets are rendered
				$aRendered = array_merge(
					$this->_renderElements(),
					$this->aPreRendered
				);

				// the renderer is executed
				$aHtmlBag = $this->oRenderer->_render(
					$aRendered
				);

			} elseif($this->oDataHandler->_isClearSubmitted() || $this->oDataHandler->_isSearchSubmitted()) {
				
				$this->_debug("NO VALIDATION REQUIRED", "HANDLING --- CLEAR OR SEARCH --- SUBMIT EVENT");

				$this->_filterUnProcessed();
				$this->_includeLibraries();

				// the datahandler is executed
				$sDH = $this->oDataHandler->_doTheMagic(FALSE);

				// Renderlets are rendered
				$aRendered = array_merge(
					$this->_renderElements(),
					$this->aPreRendered
				);

				// the renderer is executed
				$aHtmlBag = $this->oRenderer->_render(
					$aRendered
				);
			}
		} else {

			$this->_debug("NO VALIDATION REQUIRED", "NO SUBMIT EVENT TO HANDLE");

			$this->_filterUnProcessed();
			$this->_includeLibraries();

			// the datahandler is executed
			$sDH = $this->oDataHandler->_doTheMagic(FALSE);

			// Renderlets are rendered
			$aRendered = array_merge(
				$this->_renderElements(),
				$this->aPreRendered
			);

			// the renderer is executed
			$aHtmlBag = $this->oRenderer->_render($aRendered);
		}

		if($this->_defaultTrue("/meta/exportstyles")) {
			$aStyles = $this->getAllHtmlTags("style", $aHtmlBag["CONTENT"]);
			if(!empty($aStyles)) {

				$aHtmlBag["CONTENT"] = str_replace($aStyles, "<!-- Style tag exported to external css file -->\n", $aHtmlBag["CONTENT"]);

				reset($aStyles);
				while(list(, $sStyle) = each($aStyles)) {

					$sStyle = $this->oHtml->removeFirstAndLastTag($sStyle);

					reset($this->aORenderlets);
					while(list($sName, ) = each($this->aORenderlets)) {
						$sStyle = str_replace("#" . $sName, "#" . $this->aORenderlets[$sName]->_getElementCssId(), $sStyle);
					}

					$this->additionalHeaderData(
						$this->inline2TempFile(
							$sStyle,
							'css',
							"Exported style-tags of '" . $this->formid . "'"
						)
					);
				}
			}
		}

		$debug = "";

		$this->_debug($aHtmlBag, "FORMIDABLE CORE - RETURN");

		if($this->bDebug) {
			$debug = $this->debug();
		}

		if($this->useJs()) {
			reset($this->aORenderlets);
			while(list($sName, ) = each($this->aORenderlets)) {

				if(!$this->aORenderlets[$sName]->bCustomIncludeScript) {
					$this->aORenderlets[$sName]->includeScripts();
				}

				$this->aORenderlets[$sName]->includeLibs();
			}

			$this->_restoreViewState();

			$this->attachAccessibilityInit();

			reset($this->aOnloadEvents["ajax"]);

			while(list($sEventId, $aEvent) = each($this->aOnloadEvents["ajax"])) {
				//debug($aEvent, $sEventId);
				$this->attachInitTask(
					$this->oRenderer->_getAjaxEvent(
						$this->aORenderlets[$aEvent["name"]],
						$aEvent["event"],
						"onload"
					),
					"AJAX Event onload for " . $this->formid . "." . $aEvent["name"],
					$sEventId
				);
			}

			reset($this->aOnloadEvents["client"]);

			while(list(, $aEvent) = each($this->aOnloadEvents["client"])) {
				$this->attachInitTask(
					$this->oRenderer->_getClientEvent(
						$aEvent["name"],
						$aEvent["event"],
						$aEvent["eventdata"]
					),
					"CLIENT Event onload for " . $this->formid . "." . $aEvent["name"]
				);
			}
			
			if($this->aAddPostVars !== FALSE) {

				reset($this->aAddPostVars);

				while(list($sKey, ) = each($this->aAddPostVars)) {
					if(array_key_exists("action", $this->aAddPostVars[$sKey]) && $this->aAddPostVars[$sKey]["action"] == "execOnNextPage") {
						
						$aTask = $this->aAddPostVars[$sKey]["params"];

						$this->attachInitTask(
							$this->oRenderer->_getClientEvent(
								$aTask["object"],
								array(),
								$aTask
							)
						);
					}
				}
			}

			$this->attachRdtEvents();


			$sJs = "Event.onDOMReady(function() {\n" . implode("\n", $this->aInitTasks) . "});";
			$sJs .= implode("\n", $this->aInitTasksOutsideLoad);
			$this->additionalHeaderData(
				$this->inline2TempFile(
					$sJs,
					'js',
					"Form '" . $this->formid . "' initialization"
				)
			);

			$this->oJs->_loadScriptaculous();
		}

		$this->checkPoint(
			array(
				"after-validation",
				"after-render",
				"end"
			)
		);

		//debug($this->aAjaxEvents);

		$this->bStoreFormInSession = ($this->bStoreFormInSession || $this->_defaultFalse("/meta/keepinsession"));

		if($this->bStoreFormInSession === TRUE) {
			$this->_storeFormInSession();
		} else {
			$this->_clearFormInSession();
		}

		$this->end_tstamp = t3lib_div::milliseconds();

		if(!empty($sDH)) {
			return $aHtmlBag["FORMBEGIN"] . $sDH . $aHtmlBag["HIDDEN"] . $aHtmlBag["FORMEND"] . $debug;
		} else {
			return $aHtmlBag["FORMBEGIN"] . $aHtmlBag["CONTENT"] . $aHtmlBag["HIDDEN"] . $aHtmlBag["FORMEND"] . $debug;
		}
	}

	function attachRdtEvents() {
		//debug($this->aRdtEvents);
		
		$this->attachInitTask(
			implode("\n", $this->aRdtEvents),
			"RDT Events"
		);
	}

	function attachAccessibilityInit() {
		reset($this->aORenderlets);
		while(list($sKey, ) = each($this->aORenderlets)) {
			if($this->aORenderlets[$sKey]->hideIfJs() === TRUE) {
				$this->attachInitTask(
					"Formidable.f('" . $this->formid . "').o('" . $this->aORenderlets[$sKey]->_getElementHtmlId() . "').displayNone();",
					"Access"
				);
			}
		}

		$this->aInitTasks = array_merge(array_values($this->aInitTasksUnobtrusive), array_values($this->aInitTasks));
		// array_merge of array_values to avoid overruling
	}

	/**
	 * Obsolete function
	 *
	 * @return	[type]		...
	 */
	function _restoreViewState() {
		$aData = $this->oDataHandler->_P();

		if(trim($aData["AMEOSFORMIDABLE_VIEWSTATE"]) != "") {
			
			$sJsonData = $this->array2json($this->oDataHandler->_getFormData());
			$sScript =<<<JAVASCRIPT

	Formidable.f("{$this->formid}").executeViewState(
		{$aData["AMEOSFORMIDABLE_VIEWSTATE"]}
	);

JAVASCRIPT;

			$this->attachInitTask(
				$sScript,
				"Restoring ViewState"
			);
		}
	}

	/**
	 * Declares a JS task to execute at page init time
	 *
	 * @param	string		$sScript: JS code
	 * @param	string		$sDesc: optional; description of the code, place as a comment in the HTML
	 * @param	string		$sKey: optional; key of the js code in the header array
	 * @param	boolean		$bOutsideLoad: optional; load it at onload time, or after
	 * @return	void
	 */
	function attachInitTask($sScript, $sDesc = "", $sKey = FALSE, $bOutsideLoad = FALSE) {

		if($this->__getEnvExecMode() === "EID") {
			$this->attachInitTask_ajax(
				$sScript,
				$sDesc,
				$sKey,
				$bOutsideLoad
			);
		} else {
			$this->attachInitTask_plain(
				$sScript,
				$sDesc,
				$sKey,
				$bOutsideLoad
			);
		}
	}

	function attachInitTaskUnobtrusive($sScript) {
		$this->aInitTasksUnobtrusive[] = $sScript;
	}

	function attachInitTask_plain($sScript, $sDesc = "", $sKey = FALSE, $bOutsideLoad = FALSE) {
		
		if($sDesc != "") {
			$sDesc = "\n\n/* FORMIDABLE: " . trim(str_replace(array("/*", "*/", "//"), "", $sDesc)) . " */";
		}

		$sJs = $sDesc . "\n" . trim($sScript) . "\n";

		if($bOutsideLoad) {
			if($sKey === FALSE) {
				$this->aInitTasksOutsideLoad[] = $sJs;
			} else {
				$this->aInitTasksOutsideLoad[$sKey] = $sJs;
			}
		} else {
			if($sKey === FALSE) {
				$this->aInitTasks[] = $sJs;
			} else {
				$this->aInitTasks[$sKey] = $sJs;
			}
		}
	}

	function attachInitTask_ajax($sScript, $sDesc = "", $sKey = FALSE, $bOutsideLoad = FALSE) {
		
		if($sDesc != "") {
			$sDesc = "\n\n/* FORMIDABLE: " . trim(str_replace(array("/*", "*/", "//"), "", $sDesc)) . " */";
		}

		$sJs = $sDesc . "\n" . trim($sScript) . "\n";

		if($sKey === FALSE) {
			$this->aInitTasksAjax[] = $sJs;
		} else {
			$this->aInitTasksAjax[$sKey] = $sJs;
		}
	}

	/**
	 * Renders all the Renderlets elements as defined in conf
	 *
	 * @param	boolean		$bRenderChilds: render childs ? or not
	 * @return	array		Array of rendered elements, structured as $elementname => $renderedHTML
	 */
	function _renderElements($bRenderChilds = FALSE) {

		$aHtml = array();

		$aKeys = array_keys($this->aORenderlets);
		$iKeys = sizeOf($aKeys);

		for($k = 0; $k < $iKeys; $k++) {

			$sName = $aKeys[$k];

			if(!$this->aORenderlets[$sName]->isChild() || $bRenderChilds) {
				if($this->oDataHandler->aObjectType["TYPE"] != "LISTER" || $this->aORenderlets[$sName]->_searchable()) {
					if(($mHtml = $this->_renderElement($this->aORenderlets[$sName])) !== FALSE) {
						$aHtml[$sName] = $mHtml;
					}
				}
			}
		}

		reset($aHtml);
		return $aHtml;
	}

	/**
	 * Renders the given Renderlet
	 *
	 * @param	array		$aElement: details about the Renderlet to render, extracted from conf
	 * @return	string		The Rendered HTML
	 */
	function _renderElement(&$oRdt) {

		$mHtml = "";

		if($oRdt->bSubmitter === FALSE || ($oRdt->bSubmitter === TRUE && $this->oRenderer->bValidation)) {

			if(!$oRdt->i18n_hideBecauseNotTranslated()) {
				$mHtml = $this->oRenderer->processHtmlBag(
					$oRdt->render(),
					$oRdt	// changed: avoid call-time pass-by-reference
				);

				return $mHtml;
			}
		}

		return FALSE;
	}






	/**
	 * Returns system informations about an object-type
	 *
	 * @param	string		$type: something like TEXT, IMAGE, ...
	 * @param	array		$aCollectionInfos: the collection of objects where to get infos
	 * @return	mixed		array of info or FALSE if failed
	 */
	function _getInfosForType($type, $aCollectionInfos) {

		reset($aCollectionInfos);
		while(list(, $aInfos) = each($aCollectionInfos)) {

			if($aInfos["TYPE"] == $type) {
				reset($aInfos);
				return $aInfos;
			}
		}

		return FALSE;
	}

	/**
	 * Returns system-informations about the given datasource type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosDataSourceForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datasources"][$sType];
	}

	/**
	 * Returns system-informations about the given datahandler type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosDataHandlerForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["datahandlers"][$sType];
	}

	/**
	 * Returns system-informations about the given renderer type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosRendererForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderers"][$sType];
	}

	/**
	 * Returns system-informations about the given renderlet type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosRenderletForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"][$sType];
	}

	/**
	 * Returns system-informations about the given validator type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosValidatorForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["validators"][$sType];
	}

	/**
	 * Returns system-informations about the given actionlet type
	 *
	 * @param	string		$type: given type
	 * @return	array		system-informations
	 */
	function _getInfosActionletForType($sType) {
		return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["actionlets"][$sType];
	}

	/**
	 * Makes and initializes a datasource object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @param	string		$sXPath: xpath where this conf is declared
	 * @return	object
	 */
	function _makeDataSource($aElement, $sXPath) {
		return $this->_makeObject($aElement, "datasources", $sXPath);
	}

	/**
	 * Makes and initializes a datahandler object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @return	object
	 */
	function _makeDataHandler($aElement) {
		return $this->_makeObject($aElement, "datahandlers", "/control/datahandler/");
	}

	/**
	 * Makes and initializes a renderer object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @return	object
	 */
	function _makeRenderer($aElement) {
		return $this->_makeObject($aElement, "renderers", "/control/renderer/");
	}

	/**
	 * Makes and initializes a renderlet object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @param	string		$sXPath: xpath where this conf is declared
	 * @return	object
	 */
	function _makeRenderlet($aElement, $sXPath) {
		return $this->_makeObject($aElement, "renderlets", $sXPath);
	}

	/**
	 * Makes and initializes a validator object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @return	object
	 */
	function _makeValidator($aElement) {
		return $this->_makeObject($aElement, "validators", "");
	}

	/**
	 * Makes and initializes an actionlet object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @param	string		$sXPath: xpath where this conf is declared
	 * @return	object
	 */
	function _makeActionlet($aElement) {
		return $this->_makeObject($aElement, "actionlets", "/control/actionlets/");
	}

	/**
	 * Makes and initializes an object
	 *
	 * @param	array		$aElement: conf for this object instance
	 * @param	array		$sNature: renderers, datahandlers, ...
	 * @param	string		$sXPath: xpath where this conf is declared
	 * @return	object
	 */
	function &_makeObject($aElement, $sNature, $sXPath) {

		$aObj = $this->_loadObject(
			$aElement["type"],
			$sNature
		);

		if(is_array($aObj)) {

			$oObj = t3lib_div::makeInstance($aObj["CLASS"]);
			$oObj->_init($this, $aElement, $aObj, $sXPath);

			return $oObj;
		} else {
			$this->mayday("TYPE " . $aElement["type"] . " is not associated to any " . $sNature);
		}
	}

	function validateEverything() {
		$this->_validateElements();
		$this->_validateGlobalValidators();
	}

	function validateEverythingDraft() {
		$this->_validateElementsDraft();
		$this->_validateGlobalValidatorsDraft();
	}

	function _validateGlobalValidators() {
	}

	function _validateGlobalValidatorsDraft() {
	}

	/**
	 * Validates data returned by all the Renderlets elements as defined in conf
	 *
	 * @return	void		Writes into $this->_aValidationErrors[] using tx_ameosformidable::_declareValidationError()
	 */
	function _validateElements() {

		if($this->oDataHandler->_isSubmitted()) {

			$aRdtKeys = array_keys($this->aORenderlets);

			reset($aRdtKeys);
			while(list(, $sName) = each($aRdtKeys)) {
				$this->_validateElement($this->aORenderlets[$sName]->aElement);
			}
		}
	}

	/**
	 * Validates data returned by all the Renderlets, draft-mode
	 *
	 * @return	void
	 */
	function _validateElementsDraft() {

		if($this->oDataHandler->_isSubmitted()) {

			$aORenderlets = $this->aORenderlets;
			reset($aORenderlets);

			while(list($iName, $oRenderlet) = each($aORenderlets)) {
				if($oRenderlet->_hasToValidateForDraft()) {
					$this->_validateElement($oRenderlet->aElement);
				}
			}
		}
	}

	/**
	 * Validates the given Renderlet element
	 *
	 * @param	array		$aElement: details about the Renderlet element to validate, extracted from XML conf / used in formidable_mainvalidator::_doTheMagic()
	 * @return	void		Writes into $this->_aValidationErrors[] using tx_ameosformidable::_declareValidationError()
	 */
	function _validateElement($aElement) {

		if($aElement) {

			$aConfValidators = $this->_navConf("/validators/", $aElement);

			if(is_array($aConfValidators) && !empty($aConfValidators)) {

				while(list(, $aConfValidator) = each($aConfValidators)) {

					// the conf section exists
					// call validator

					$oValidator = $this->_makeValidator($aConfValidator);

					if($this->oDataHandler->_isValid($aElement["name"])) {

						if($oValidator->_matchConditions($aConfValidator)) {

							$bHasToValidate = TRUE;

							$aValidMap = $this->_navConf("/control/factorize/switchvalidation");
							if($this->isUserObj($aValidMap)) {
								$aValidMap = $this->_callUserObj($aValidMap);
							}

							if(is_array($aValidMap) && array_key_exists($aElement["name"], $aValidMap)) {
								$bHasToValidate = $aValidMap[$aElement["name"]];
							}

							if($bHasToValidate === TRUE) {
								$oValidator->_doTheMagic(
									$this->_getInfosValidatorForType($aConfValidator["type"]),
									$aConfValidator,
									$aElement
								);
							}
						}
					} else {
						break;
					}
				}
			}

			$oRdt =& $this->aORenderlets[$aElement["name"]];
			$oRdt->declareCustomValidationErrors();
		}
	}

	/**
	 * Return
	 *
	 * @param	[type]		$aConditioner: ...
	 * @return	[type]		...
	 */
	function _matchConditions($aConditioner) {

		$bRet = TRUE;

		if(($aConditions = $this->_navConf("/conditions/", $aConditioner)) !== FALSE) {

			if(($sLogic = $this->_navConf("/logic", $aConditions)) === FALSE) {
				$sLogic = "AND";
			} else {
				$sLogic = strtoupper($sLogic);
			}

			while(list($sCondKey, ) = each($aConditions)) {

				if($sCondKey{0} === "c" && $sCondKey{1} === "o" && t3lib_div::isFirstPartOfStr($sCondKey, "condition")) {
					$aCondition = $this->_navConf($sCondKey, $aConditions);
					switch($sLogic) {
						case "OR": {
							$bRet = $bRet || $this->_matchCondition($aCondition);
							break;
						}
						case "AND":
						default: {
							$bRet = $bRet && $this->_matchCondition($aCondition);
							break;
						}
					}
				}
			}
		}

		return $bRet;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aCondition: ...
	 * @return	[type]		...
	 */
	function _matchCondition($aCondition) {

		reset($aCondition);

		if($this->isUserObj($aCondition)) {
			return $this->_callUserObj(
				$aCondition
			);
		} else {

			list($sType, $aInfos) = each($aCondition);

			if($this->isUserObj($aInfos)) {
				$aInfos = $this->_callUserObj(
					$aInfos
				);
			}

			switch(strtoupper($sType)) {

				case "ISTRUE" : {
					return $this->_isTrueVal($aInfos);
					break;
				}
				case "ISFALSE" : {
					return $this->_isFalseVal($aInfos);
					break;
				}
				case "USERID" :
				case "USERIDS" : {

					$aUserIds = t3lib_div::trimExplode(",", $aInfos);

					if(is_array($aUserIds)) {
						return in_array(
							@intval($GLOBALS["TSFE"]->fe_user->user[$GLOBALS["TSFE"]->fe_user->userid_column]),
							$aUserIds
						);
					}

					break;
				}
				case "ISAUTHENTIFIED" : {

					$bAuth = (intval($GLOBALS["TSFE"]->fe_user->user[$GLOBALS["TSFE"]->fe_user->userid_column]) > 0);
					return ($this->_isTrueVal($aCondition["ISAUTHENTIFIED"])) ? $bAuth : !$bAuth;
					break;
				}
				case "USERNAME" :
				case "USERNAMES" : {

					$aUserNames = t3lib_div::trimExplode(",", $aInfos);

					if(is_array($aUserNames)) {
						return @in_array(
							$GLOBALS["TSFE"]->fe_user->user[$GLOBALS["TSFE"]->fe_user->username_column],
							$aUserNames
						);
					}

					break;
				}
				case "USERGROUP" :
				case "USERGROUPS" : {

					$aUserGroups = t3lib_div::trimExplode(",", $aInfos);
					$aCurrentUserGroups = t3lib_div::trimExplode(
						",",
						$GLOBALS["TSFE"]->fe_user->user["usergroup"]
					);

					if(count(array_intersect($aUserGroups, $aCurrentUserGroups)) > 0) {
						return TRUE;
					}
					else {
						return FALSE;
					}

					break;
				}
				case "FORMID" : {

					if($aInfos == $this->formid) {
						return TRUE;
					}

					return FALSE;
					break;
				}
				case "ISCREATION" : {
					return !($this->oDataHandler->_edition());
					break;
				}
				case "ISEDITION" : {
					return ($this->oDataHandler->_edition());
					break;
				}
			}
		}

		return TRUE;
	}

	/**
	 * Declares validation error
	 * Used by Validators Objects
	 *
	 * @param	string		$elementname: not used for the moment
	 * @param	string		$key: not used for the moment
	 * @param	string		$message: the error message to display
	 * @return	void		Writes into $this->_aValidationErrors[]
	 */
	function _declareValidationError($sElementName, $sKey, $sMessage) {

		if(!array_key_exists($sElementName, $this->_aValidationErrors)) {
			$this->_aValidationErrors[$sElementName] = $sMessage;	// only one message per renderlet per refresh ( errors displayed one by one )

			$this->_aValidationErrorsInfos[$sElementName] = array(
				"elementname" => $sElementName,
				"message" => $sMessage,
				"namespace" => array_shift(explode(":", $sKey)),
				"type" => array_pop(explode(":", $sKey)),
			);

			$this->_aValidationErrorsTypes[$sKey] = array(
				"namespace" => array_shift(explode(":", $sKey)),
				"type" => array_pop(explode(":", $sKey)),
			);
		}
	}

	// $sKey like "STANDARD:required", "DB:unique", ...
	function _hasErrorType($sKey) {
		// consider unstable as of rev 101 if /process has unset renderlet, and this renderlet was the only one to throw that type of error
			// TODO: unset also type if it's the case
		return array_key_exists($sKey, $this->_aValidationErrorsTypes);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sExtKey: ...
	 * @param	[type]		$sServiceKey: ...
	 * @param	[type]		$bVirtualizeFE: ...
	 * @param	[type]		$bInitBEuser: ...
	 * @return	[type]		...
	 */
	function declareAjaxService($sExtKey, $sServiceKey, $bVirtualizeFE = TRUE, $bInitBEuser = FALSE) {
		$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["ajax_services"][$sExtKey][$sServiceKey]["conf"] = array(
			"virtualizeFE"	=> $bVirtualizeFE,
			"initBEuser"	=> $bInitBEuser,
		);
	}

	/**
	 * Execute each actionlet declared for this FORM
	 *
	 * @param	array		$aRendered:	array containing the HTML of the rendered renderlets
	 * @param	string		$sForm: the whole FORM html string
	 * @return	void
	 * @see tx_ameosformidable::_render()
	 */
	function _executeActionlets($aRendered, $sForm) {

		$aActionlets = $this->_navConf("/control/actionlets/");

		if(is_array($aActionlets))
		{
			while(list(, $aActionlet) = each($aActionlets))
			{
				if($aActionlet)
				{ $this->_executeActionlet($aActionlet, $aRendered, $sForm);}
			}
		}
	}

	/**
	 * Executes the specific process for this actionlet
	 *
	 * @param	array		$aActionlet: details about the Renderlet element to validate, extracted from XML conf / used in formidable_mainvalidator::_doTheMagic()
	 * @param	array		$aRendered: array containing the HTML of the rendered renderlets
	 * @param	string		$sForm: the whole FORM html string
	 * @return	void
	 * @see tx_ameosformidable::_executeActionlets()
	 */
	function _executeActionlet($aActionlet, $aRendered, $sForm) {

		$oActionlet = $this->_makeActionlet($aActionlet);
		$oActionlet->_doTheMagic($aRendered, $sForm);
	}









	/*********************************
	 *
	 * Debugging functions
	 *
	 *********************************/


	/**
	 * Displays a full debug of :
	 * - the XML conf
	 * - the collection of declared DataHandlers
	 * - the collection of declared Renderers
	 * - the collection of declared Renderlets
	 * - the collection of declared Validators
	 *
	 * Can be called by the parent Extension, or by FORMidable itselves, if the XML conf sets /formidable/meta/debug/ to TRUE
	 *
	 * @param	[type]		$bExpand: ...
	 * @return	void
	 * @see	tx_ameosformidable::mayday(), tx_ameosformidable::_render()
	 */
	function debug($bExpand = FALSE) {

//		$this->oJs->_includeThisFormDebugFuncs();

		$aHtml = array();

		$aHtml[] = "<a name = '" . $this->formid . "formidable_debugtop' />";

		if($bExpand === FALSE) {
			$aHtml[] = "<a href = 'javascript:void(Formidable.f(\"" . $this->formid . "\").toggleDebug())'><img src='" . t3lib_div::getIndpEnv("TYPO3_SITE_URL") . t3lib_extmgm::siteRelPath("ameos_formidable") . "/res/images/debug.gif' border='0' alt='Toggle FORMidable::debug()' title='Toggle FORMidable::debug()'></a>";
			$aHtml[] = "<div id = '" . $this->formid . "_debugzone' style = 'font-family: Verdana; display: none; background-color: #bed1f4; padding-left: 10px; padding-top: 3px; padding-bottom: 10px;font-size: 9px;'>";
		} else {
			$aHtml[] = "<img src='" . t3lib_div::getIndpEnv("TYPO3_SITE_URL") . t3lib_extmgm::siteRelPath("ameos_formidable") . "/res/images/debug.gif' border='0' alt='Toggle FORMidable::debug()' title='Toggle FORMidable::debug()'>";
			$aHtml[] = "<div id = '" . $this->formid . "_debugzone' style = 'font-family: Verdana; display: block; background-color: #bed1f4; padding-left: 10px; padding-top: 3px; padding-bottom: 10px;font-size: 9px;'>";
		}

		$aHtml[] = "<h4>FORMidable debug()</h4>";

		$aHtml[] = "<h5>t3lib_div::_POST()</h5>";
		$aHtml[] = t3lib_div::view_array(t3lib_div::_POST());

		$aHtml[] = "<h5>t3lib_div::_GET()</h5>";
		$aHtml[] = t3lib_div::view_array(t3lib_div::_GET());

		/*$aHtml[] = "<ul>";
		$aHtml[] = "<li><a href = 'http://typo3.org/documentation/document-library/ameos_formidable/' target = '_blank'>FORMidable user documentation</a></li>";

		$aHtml[] = "</ul>";*/
		$aHtml[] = "<br>";
		$aHtml[] = "<ul>";

		if(!is_null($this->_xmlPath)) {
			// conf passed by php array ( // typoscript )
			$aHtml[] = "<li><a href = '#" . $this->formid . "formidable_xmlpath' target = '_self'>XML Path</a></li>";
		}

		$aHtml[] = "<li><a href = '#" . $this->formid . "formidable_configuration' target = '_self'>FORM configuration</a></li>";
		$aHtml[] = "<li><a href = '#" . $this->formid . "formidable_callstack' target = '_self'>Call stack</a></li>";
//		$aHtml[] = "<li><a href = '#" . $this->formid . "formidable_objects' target = '_self'>Available DataHandlers, Renderers, Renderlets, Validators and Actionlets</a></li>";
		$aHtml[] = "</ul>";

		if(!is_null($this->_xmlPath)) {

			$aHtml[] = "<a name = '" . $this->formid . "formidable_xmlpath' />";
			$aHtml[] = "<h5>XML Path</h5>";
			$aHtml[] = $this->_xmlPath;

			$aHtml[] = "<p align = 'right'><a href = '#" . $this->formid . "formidable_debugtop' target = '_self'>^top^</a></p>";


			$aHtml[] = "<a name = '" . $this->formid . "formidable_xmlfile' />";
		}


		$aHtml[] = "<a name = '" . $this->formid . "formidable_configuration' />";
		$aHtml[] = "<h5>FORM configuration</h5>";
		$aHtml[] = "<div WIDTH = '100%' style = 'HEIGHT: 400px; overflow: scroll'>" . t3lib_div::view_array($this->_aConf) . "</div>";
		$aHtml[] = "<p align = 'right'><a href = '#" . $this->formid . "formidable_debugtop' target = '_self'>^top^</a></p>";

		$aHtml[] = "<a name = '" . $this->formid . "formidable_callstack' />";
		$aHtml[] = "<h5>Call stack</h5>";
		$aHtml[] = implode("<hr>", $this->aDebug);
		$aHtml[] = "<p align = 'right'><a href = '#" . $this->formid . "formidable_debugtop' target = '_self'>^top^</a></p>";

		$aHtml[] = "</div>";

		return implode("\n", $aHtml);
	}

	/**
	 * Internal debug function
	 * Calls the TYPO3 debug function if the XML conf sets /formidable/meta/debug/ to TRUE
	 *
	 * @param	mixed		$variable: the variable to dump
	 * @param	string		$name: title of this debug section
	 * @param	string		$line: PHP code line calling this function ( __LINE__ )
	 * @param	string		$file: PHP script calling this function ( __FILE__ )
	 * @param	integer		$recursiveDepth: number of levels to debug, if recursive variable
	 * @param	string		$debugLevel: the sensibility of this warning
	 * @return	void
	 */
	function _debug($variable, $name, $bAnalyze = TRUE) {

		if($this->bDebug) {

			$aTrace		= debug_backtrace();
			$aLocation	= array_shift($aTrace);
			$aTrace1	= array_shift($aTrace);
			$aTrace2	= array_shift($aTrace);
			$aTrace3	= array_shift($aTrace);
			$aTrace4	= array_shift($aTrace);

			$numcall = sizeof($this->aDebug) + 1;

			$aDebug = array();
			$aDebug[] = "<p align = 'right'><a href = '#" . $this->formid . "formidable_debugtop' target = '_self'>^top^</a></p>";
			$aDebug[] = "<a name = '" . $this->formid . "formidable_call" . $numcall . "' />";
			$aDebug[] = "<a href = '#" . $this->formid . "formidable_call" . ($numcall - 1) . "'>&lt;&lt; prev</a> / <a href = '#" . $this->formid . "formidable_call" . ($numcall + 1) . "'>next &gt;&gt;</a><br>";
			$aDebug[] = "<strong>#" . $numcall ." - " . $name . "</strong>";
			$aDebug[] = "<br/>";
			$aDebug[] = "<span style='font-family: verdana;font-size: 9px; font-style: italic;'><b>(Total exec. time: </b>" . round(t3lib_div::milliseconds() - $this->start_tstamp, 4) / 1000 ." sec)</span>";
			$aDebug[] = "<br/>";


			$aDebug[] = "<a href='javascript:void(Formidable.f(\"" . $this->formid . "\").toggleBacktrace(" . $numcall . "))'>Toggle details</a><br>";
			$aDebug[] = "<div id='" . $this->formid . "_formidable_call" . $numcall . "_backtrace' style='display: none; background-color: #FFFFCC' >";
//			$aDebug[] = "<hr/>";

			if($this->iDebug < 2) {
				$aDebug[] = "<span style='font-family: verdana;font-size: 9px; font-style: italic;'><b>Call 0: </b>" . str_replace(PATH_site, "/", $aLocation["file"]) . ":" . $aLocation["line"]  . " | <b>" . $aTrace1["class"] . $aTrace1["type"] . $aTrace1["function"] . "</b></span><br>" . $this->_viewMixed($aTrace1["args"]);
				$aDebug[] = "<hr/>";
				$aDebug[] = "<span style='font-family: verdana;font-size: 9px; font-style: italic;'><b>Call -1: </b>" . str_replace(PATH_site, "/", $aTrace1["file"]) . ":" . $aTrace1["line"]  . " | <b>" . $aTrace2["class"] . $aTrace2["type"] . $aTrace2["function"] . "</b></span><br>" . $this->_viewMixed($aTrace2["args"]);
				$aDebug[] = "<hr/>";
				$aDebug[] = "<span style='font-family: verdana;font-size: 9px; font-style: italic;'><b>Call -2: </b>" . str_replace(PATH_site, "/", $aTrace2["file"]) . ":" . $aTrace2["line"]  . " | <b>" . $aTrace3["class"] . $aTrace3["type"] . $aTrace3["function"] . "</b></span><br>" . $this->_viewMixed($aTrace3["args"]);
				$aDebug[] = "<hr/>";
				$aDebug[] = "<span style='font-family: verdana;font-size: 9px; font-style: italic;'><b>Call -3: </b>" . str_replace(PATH_site, "/", $aTrace3["file"]) . ":" . $aTrace3["line"]  . " | <b>" . $aTrace4["class"] . $aTrace4["type"] . $aTrace4["function"] . "</b></span><br>" . $this->_viewMixed($aTrace4["args"]);
				$aDebug[] = "<hr/>";
			}

//			$aDebug[] = "<p style='font-family: verdana;font-size: 9px; font-style: italic;'>" . t3lib_div::debug_trail() . "</p>";

			//if($this->iDebug < 2)
			{
				if(is_string($variable)) {
					$aDebug[] = $variable;
				} else {
					if($bAnalyze) {
						$aDebug[] = $this->_viewMixed($variable);
					} else {
						$aDebug[] = t3lib_div::view_array($variable);
					}
				}
			}

			$aDebug[] = "</div>";

			$aDebug[] = "<br/>";

			$this->aDebug[] = implode("", $aDebug);
		}
	}

	/**
	 * Stops Formidable and PHP execution : die() if some critical error appeared
	 *
	 * @param	string		$msg: the error message
	 * @return	void
	 */
	function mayday($msg) {
		$aTrace		= debug_backtrace();
		$aLocation	= array_shift($aTrace);
		$aTrace1	= array_shift($aTrace);
		$aTrace2	= array_shift($aTrace);
		$aTrace3	= array_shift($aTrace);
		$aTrace4	= array_shift($aTrace);

		$aDebug = array();
		$aDebug[] = "<span class='notice'><b>XML: </b> " . $this->_xmlPath . "</span>";
		$aDebug[] = "<br/>";
		$aDebug[] = "<span class='notice'><b>Formidable: </b>v" . $this->sApiVersion . "</span>";
		$aDebug[] = "<br/>";
		$aDebug[] = "<span class='notice'><b>Total exec. time: </b>" . round(t3lib_div::milliseconds() - $this->start_tstamp, 4) / 1000 ." sec</span>";
		$aDebug[] = "<br/>";


		$aDebug[] = "<h2 id='backtracetitle'>Call stack</h2>";
		$aDebug[] = "<div class='backtrace'>";
		$aDebug[] = "<span class='notice'><b>Call 0: </b>" . str_replace(PATH_site, "/", $aLocation["file"]) . ":" . $aLocation["line"]  . " | <b>" . $aTrace1["class"] . $aTrace1["type"] . $aTrace1["function"] . "</b></span><br/>With parameters: " . (!empty($aTrace1["args"]) ? $this->_viewMixed($aTrace1["args"]) : " no parameters");
		$aDebug[] = "<hr/>";
		$aDebug[] = "<span class='notice'><b>Call -1: </b>" . str_replace(PATH_site, "/", $aTrace1["file"]) . ":" . $aTrace1["line"]  . " | <b>" . $aTrace2["class"] . $aTrace2["type"] . $aTrace2["function"] . "</b></span><br />With parameters: " . (!empty($aTrace2["args"]) ? $this->_viewMixed($aTrace2["args"]) : " no parameters");
		$aDebug[] = "<hr/>";
		$aDebug[] = "<span class='notice'><b>Call -2: </b>" . str_replace(PATH_site, "/", $aTrace2["file"]) . ":" . $aTrace2["line"]  . " | <b>" . $aTrace3["class"] . $aTrace3["type"] . $aTrace3["function"] . "</b></span><br />With parameters: " . (!empty($aTrace3["args"]) ? $this->_viewMixed($aTrace3["args"]) : " no parameters");
		$aDebug[] = "<hr/>";
		$aDebug[] = "<span class='notice'><b>Call -3: </b>" . str_replace(PATH_site, "/", $aTrace3["file"]) . ":" . $aTrace3["line"]  . " | <b>" . $aTrace4["class"] . $aTrace4["type"] . $aTrace4["function"] . "</b></span><br />With parameters: " . (!empty($aTrace4["args"]) ? $this->_viewMixed($aTrace4["args"]) : " no parameters");
		$aDebug[] = "<hr/>";

		if(is_callable(array("t3lib_div", "debug_trail"))) {
			$aDebug[] = "<span class='notice'>" . t3lib_div::debug_trail() . "</span>";
			$aDebug[] = "<hr/>";
		}

		$aDebug[] = "</div>";

		$aDebug[] = "<br/>";

		$sContent =	"<h1 id='title'>Formidable::Mayday</h1>";
		$sContent .= "<div id='errormessage'>" . $msg . "</div>";
		$sContent .= "<hr />";
		$sContent .= implode("", $aDebug);
		//$sContent .= $this->debug(TRUE);

		$sPage =<<<MAYDAYPAGE
<!DOCTYPE html
	PUBLIC "-//W3C//DTD XHTML 1.1//EN"
	"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
	<head>
		<title>Formidable::Mayday</title>
		<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
		<meta name="robots" content="noindex, nofollow" />
		<style type="text/css">

			#title {
				color: red;
				font-family: Verdana;
			}

			#errormessage {
				border: 2px solid red;
				padding: 10px;
				color: white;
				background-color: red;
				font-family: Verdana;
				font-size: 12px;
			}

			.notice {
				font-family: Verdana;
				font-size: 9px;
				font-style: italic;
			}

			#backtracetitle {
			}

			.backtrace {
				background-color: #FFFFCC;
			}

			HR {
				border: 1px solid silver;
			}
		</style>
	</head>
	<body>
		{$sContent}
	</body>
</html>

MAYDAYPAGE;

/*		if($this->bDebug) {
			die($sPage);
		} elseif($this->bDebugIP) {
			$sPage = "<h4 style='color: red'>This full detail error-message is displayed because your IP(" . t3lib_div::getIndpEnv('REMOTE_ADDR') . ") matches the TYPO3 devIPMask</h4>" . $sPage;
			die($sPage);
		} elseif($GLOBALS["TSFE"]->TYPO3_CONF_VARS['FE']['pageNotFound_handling']) {
			$GLOBALS["TSFE"]->pageNotFoundAndExit('FORMIDABLE Mayday: ' . $msg);
		}*/

		die($sPage);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mVar: ...
	 * @return	[type]		...
	 */
	function debug4ajax($mVar) {
		echo "/*";
		print_r($mVar);
		echo "*/";
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mMixed: ...
	 * @param	[type]		$bRecursive: ...
	 * @return	[type]		...
	 */
	function d() {
		
		$aVars = func_get_args();
		
		if(func_num_args() === 1) {
			$aVars = func_get_arg(0);
		}

		echo "<div>" . tx_ameosformidable::_viewMixed(
			$aVars,
			TRUE,
			0
		) . "</div>";
		flush();
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mMixed: ...
	 * @param	[type]		$bRecursive: ...
	 * @param	[type]		$iLevel: ...
	 * @return	[type]		...
	 */
	function _viewMixed($mMixed, $bRecursive = TRUE, $iLevel=0) {

		$sStyle = "font-family: Verdana; font-size: 9px;";
		$sStyleBlack = $sStyle . "color: black;";
		$sStyleRed = $sStyle . "color: red;";
		$sStyleGreen = $sStyle . "color: green;";

		$aBgColors = array(
			"FFFFFF", "F8F8F8", "EEEEEE", "E7E7E7", "DDDDDD", "D7D7D7", "CCCCCC", "C6C6C6", "BBBBBB", "B6B6B6", "AAAAAA", "A5A5A5", "999999", "949494", "888888", "848484", "777777", "737373"
		);

		if(is_array($mMixed)) {

			$result="<table border=1 style='border: 1px solid silver' cellpadding=1 cellspacing=0 bgcolor='#" . $aBgColors[$iLevel] . "'>";

			if(!count($mMixed)) {
				$result.= "<tr><td><span style='" . $sStyleBlack . "'><b>".htmlspecialchars("EMPTY!")."</b></span></td></tr>";
			} else {
				while(list($key, $val)=each($mMixed)) {

					$result.= "<tr><td valign='top'><span style='" . $sStyleBlack . "'>".htmlspecialchars((string)$key)."</span></td><td>";

					if(is_array($val))	{
						$result.=tx_ameosformidable::_viewMixed($val, $bRecursive, $iLevel + 1);
					} else {
						$result.= "<span style='" . $sStyleRed . "'>".tx_ameosformidable::_viewMixed($val, $bRecursive, $iLevel + 1)."<br /></span>";
					}

					$result.= "</td></tr>";
				}
			}

			$result.= "</table>";

		} elseif(is_resource($mMixed)) {
			$result = "<span style='" . $sStyleGreen . "'>RESOURCE: </span>" . $mMixed;
		} elseif(is_object($mMixed)) {
			if($bRecursive) {
				$result = "<span style='" . $sStyleGreen . "'>OBJECT (" . get_class($mMixed) .") : </span>" . tx_ameosformidable::_viewMixed(get_object_vars($mMixed), FALSE, $iLevel + 1);
			} else {
				$result = "<span style='" . $sStyleGreen . "'>OBJECT (" . get_class($mMixed) .") : !RECURSION STOPPED!</span>";// . t3lib_div::view_array(get_object_vars($mMixed), FALSE);
			}
		} elseif(is_bool($mMixed)) {
			$result = "<span style='" . $sStyleGreen . "'>BOOLEAN: </span>" . ($mMixed ? "TRUE" : "FALSE");
		} elseif(is_string($mMixed)) {
			if(empty($mMixed)) {
				$result = "<span style='" . $sStyleGreen . "'>STRING(0)</span>";
			} else {
				$result = "<span style='" . $sStyleGreen . "'>STRING(" . strlen($mMixed) . "): </span>" . nl2br(htmlspecialchars((string)$mMixed));
			}
		} elseif(is_null($mMixed)) {
			$result = "<span style='" . $sStyleGreen . "'>!NULL!</span>";
		} elseif(is_integer($mMixed)) {
			$result = "<span style='" . $sStyleGreen . "'>INTEGER: </span>" . $mMixed;
		} else {
			$result = "<span style='" . $sStyleGreen . "'>MIXED: </span>" . nl2br(htmlspecialchars(strVal($mMixed)));
		}

		return $result;
	}

	/*********************************
	 *
	 * Utilitary functions
	 *
	 *********************************/

	function getLLLabel($mLabel) {
		return $this->_getLLLabel($mLabel);
	}
	/**
	 * Returns the translated string for the given LLL path
	 *
	 * @param	string		$label: LLL path
	 * @return	string		The translated string
	 */
	function _getLLLabel($mLabel) {

		if($this->isUserObj($mLabel)) {
			$mLabel = $this->_callUserObj($mLabel);
		}

		if($this->sDefaultLLLPrefix !== FALSE) {
			if(t3lib_div::isFirstPartOfStr($mLabel, "LLL:") && !t3lib_div::isFirstPartOfStr($mLabel, "LLL:EXT:")) {
				$mLabel = str_replace("LLL:", "LLL:" . $this->sDefaultLLLPrefix . ":", $mLabel);
			}
		}

		if(TYPO3_MODE == "FE") {
			// front end
			return $GLOBALS["TSFE"]->sL($mLabel);
		}
		else {
			// back end
			return $GLOBALS["LANG"]->sL($mLabel);
		}
	}

	/**
	 * Callback function for preg_callback_replace
	 *
	 * Returns the translated string for the given {LLL} path
	 *
	 * @param	string		$label: {LLL} path
	 * @return	string		The translated string
	 */
	function _getLLLabelTag($aLabel) {
		return $this->_getLLLabel(
			str_replace(array("{", "}"), "", array_pop($aLabel))
		);
	}

	/**
	 * Parses a template
	 *
	 * @param	string		$templatePath: the path to the template file
	 * @param	string		$templateMarker: the marker subpart
	 * @param	array		$aTags: array containing the values to render
	 * @param	[type]		$aExclude: ...
	 * @param	[type]		$bClearNotUsed: ...
	 * @param	[type]		$aLabels: ...
	 * @return	string		HTML string with substituted values
	 */
	function _parseTemplate(
		$templatePath,
		$templateMarker,
		$aTags = array(),
		$aExclude = array(),
		$bClearNotUsed = TRUE,
		$aLabels = array()
	) {

		// $tempUrl : the path of the template for use with t3lib_div::getUrl()
		// $tempMarker :  the template subpart marker
		// $aTags : the marker array for substitution
		// $aExclude : tag names that should not be substituted

		$templatePath = tx_ameosformidable::toServerPath($templatePath);

		return tx_ameosformidable::_parseTemplateCode(
			t3lib_parsehtml::getSubpart(
				t3lib_div::getUrl($templatePath),
				$templateMarker
			),
			$aTags,
			$aExclude,
			$bClearNotUsed,
			$aLabels
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _getParentExtSitePath() {

		if(TYPO3_MODE === "FE") {
			if(is_subclass_of($this->_oParent, "tslib_pibase")) {
				$sExtKey = $this->_oParent->extKey;
			} else {
				$sExtKey = "ameos_formidable";
			}
		} else {
			$sExtKey = $GLOBALS["_EXTKEY"];
		}

		return t3lib_div::getIndpEnv("TYPO3_SITE_URL") . t3lib_extMgm::siteRelPath($sExtKey);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sHtml: ...
	 * @return	[type]		...
	 */
	function _substLLLInHtml($sHtml) {

		if($sHtml{0} === "L" && t3lib_div::isFirstPartOfStr($sHtml, "LLL:")) {
			return $this->_getLLLabel($sHtml);
		}

		return @preg_replace_callback(
			"/{LLL:[a-zA-Z0-9_:\/.\-]*}/",
			array($this, "_getLLLabelTag"),
			$sHtml
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$sHtml: ...
	 * @param	[type]		$aTags: ...
	 * @param	[type]		$aExclude: ...
	 * @param	[type]		$bClearNotUsed: ...
	 * @param	[type]		$aLabels: ...
	 * @return	[type]		...
	 */
	function _parseTemplateCode(
		$sHtml,
		$aTags,
		$aExclude = array(),
		$bClearNotUsed = TRUE,
		$aLabels = array()
	) {

//		debug("parseTemplateCode", "ici");

		if(is_object($this)) {
			$this->aCurrentTemplateTags = $aTags;
		}

		if(is_object($this->_oParent) && !empty($this->_oParent->extKey)) {
			$aTags["PARENTPATH"] = $this->_getParentExtSitePath();
		}

		if(count($aExclude) > 0) {

			$sExcludePfx = md5(microtime(TRUE));
			$sExcludePfx2 = md5(microtime(TRUE)+1);

			reset($aExclude);
			while(list(, $tag) = each($aExclude)) {

				$sHtml = str_replace("{" . $tag . "}", $sExcludePfx . $tag . $sExcludePfx, $sHtml);
				$sHtml = str_replace("{" . $tag . ".label}", $sExcludePfx2 . $tag . $sExcludePfx2, $sHtml);
			}
		}

//		debug($aTags);

		reset($aTags);
		while(list($sName, $mVal) = each($aTags)) {
			if(($sRdtSubPart = t3lib_parsehtml::getSubpart($sHtml, "###" . $sName . "###")) !== "") {
				$sHtml = t3lib_parsehtml::substituteSubpart(
					$sHtml,
					"###" . $sName . "###",
					$mVal["__compiled"],
					FALSE,
					FALSE
				);
			}
		}

		$aCatches = array();
		//preg_match_all("|{[a-zA-Z0-9_:/.\-\(\)]*}|", $sHtml, $aCatches);
		preg_match_all("|{[^\{\}]*}|", $sHtml, $aCatches);
		//debug($aCatches);

		$aSubParts = array();
		//if(preg_match_all("|\<\!\-\-.*\#\#\#(.*\bperimeter\b)\#\#\#.*\bbegin\b.*\-\-\>|", $sHtml, $aSubParts) !== 0) {
		if(preg_match_all("|\<\!\-\-.*\#\#\#(.+\ \bperimeter\b)\#\#\#.*\-\-\>|", $sHtml, $aSubParts) !== 0) {
			$aPerimeters = array_unique($aSubParts[1]);
			//debug($aPerimeters);
			reset($aPerimeters);
			while(list(, $sSubPart) = each($aPerimeters)) {

				/*$sSubPartHtml = t3lib_parsehtml::getSubPart(
					$sHtml,
					"###" . $sSubPart . "###"
				);

				$aSubPartsConditions = array();
				if(preg_match_all("|\<\!\-\-.*\#\#\#(\bperimeter:conditions\b)\#\#\#.*\-\-\>|", $sSubPartHtml, $aSubPartsConditions) !== 0) {

					$sCondition = t3lib_parsehtml::getSubPart(
						$sSubPartHtml,
						"###perimeter:conditions###"
					);

					debug($sCondition, "condition");
				}*/

				$bDelete = FALSE;
				
				$aParts = t3lib_div::trimExplode(" ", $sSubPart);
				$mValue = $this->resolveForTemplate($aParts[0], $aTags);
				
				if(is_array($mValue)) {
					if(array_key_exists("__compiled", $mValue)) {
						if(trim($mValue["__compiled"]) === "") {
							$bDelete = TRUE;
						}
					} elseif(empty($mValue)) {
						$bDelete = TRUE;
					}
				} else {
					if(is_string($mValue) && (trim($mValue) === "")) {
						$bDelete = TRUE;
					} elseif(is_bool($mValue) && $mValue===FALSE) {
						$bDelete = TRUE;
					}
				}

				if($bDelete === TRUE) {
					//$content,$marker,$subpartContent,$recursive=1,$keepMarker=0
					$sHtml = t3lib_parsehtml::substituteSubpart(
						$sHtml,
						"###" . $sSubPart . "###",
						"",
						FALSE,
						FALSE
					);
				}
			}
		}

		if(is_array($aCatches) && !empty($aCatches)) {
			$aCatches = $aCatches[0];
		} else {
			$aCatches = array();
		}

		reset($aCatches);
		while(list($sKey,) = each($aCatches)) {
			$sTag = $aCatches[$sKey];
			$sCatch = str_replace(array("{", "}"), "", $sTag);

			if(($sCatch{0} === "L" && $sCatch{1} === "L" && $sCatch{2} === "L") && ($sCatch{3} === ":")) {
				if(isset($this)) {
					$sHtml = str_replace($sTag, $this->_getLLLabel($sCatch), $sHtml);
				}
			} else {

				if(($mVal = tx_ameosformidable::resolveForTemplate($sCatch, $aTags)) !== AMEOSFORMIDABLE_LEXER_FAILED && $mVal !== AMEOSFORMIDABLE_LEXER_BREAKED) {
					if(is_array($mVal)) {
						if(array_key_exists("__compiled", $mVal)) {
							$sHtml = str_replace($sTag, $mVal["__compiled"], $sHtml);
						} else {
							/* nothing */
							//debug($sCatch, "not found");
						}
					} else {
						$sHtml = str_replace($sTag, $mVal, $sHtml);
					}
				} else {
					/* nothing */
					//debug($sCatch, "not found");
				}
			}
		}

		/*
		reset($aLabels);
		while(list($sName, $mVal) = each($aLabels)) {
			$sTag = "{" . $sName . ".label}";
			$sHtml = str_replace($sTag, $mVal, $sHtml);
		}*/

		if($bClearNotUsed) {
			//$sHtml = ereg_replace("\{[a-zA-Z0-9_:/.\-\(\)]*}", "", $sHtml);
			$sHtml = ereg_replace("\{[^\{\}]*}", "", $sHtml);
		}

		if(count($aExclude) > 0) {

			reset($aExclude);
			while(list(, $tag) = each($aExclude)) {
				$sHtml = str_replace($sExcludePfx . $tag . $sExcludePfx, "{" . $tag . "}", $sHtml);
				$sHtml = str_replace($sExcludePfx2 . $tag . $sExcludePfx2, "{" . $tag . ".label}", $sHtml);
			}
		}
		
		if(is_object($this)) {
			$this->aCurrentTemplateTags = array();
		}

		return $sHtml;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$main_prefix: ...
	 * @param	[type]		$content: ...
	 * @return	[type]		...
	 */
	function prefixResourcePath($main_prefix, $content) {
		$this->makeHtmlParser();

		// fooling the htmlparser to avoid replacement of {tags} in template
		$content = str_replace("{", "http://{", $content);
		$content = $this->oHtml->prefixResourcePath(
			$main_prefix,
			$content
		);
		return str_replace("http://{", "{", $content);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sTag: ...
	 * @param	[type]		$sHtml: ...
	 * @return	[type]		...
	 */
	function getAllHtmlTags($sTag, $sHtml) {
		$this->makeHtmlParser();
		$aParts = $this->oHtml->splitIntoBlock(
			$sTag,
			$sHtml
		);

		$iCount = count($aParts);
		for($k = 0; $k < $iCount; $k+=2) {
			unset($aParts[$k]);
		}

		reset($aParts);
		return array_reverse(array_reverse($aParts));	// reordering keys
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$iErrno: ...
	 * @param	[type]		$sMessage: ...
	 * @param	[type]		$sFile: ...
	 * @param	[type]		$iLine: ...
	 * @param	[type]		$oObj: ...
	 * @return	[type]		...
	 */
	function __catchEvalException($iErrno, $sMessage, $sFile, $iLine, $oObj) {

		$aErrors = array (
			E_ERROR		=> "Error",
//			E_WARNING	=> "Warning",
			E_PARSE		=> "Parse error",
		);

		if(array_key_exists($iErrno, $aErrors)) {
			ob_start();
			highlight_string($this->__sEvalTemp["code"]);
			$sPhp = ob_get_contents();
			ob_end_clean();
			$sXml = $this->_viewMixed($this->__sEvalTemp["xml"]);

			$this->mayday("<b>" . $aErrors[$iErrno] . "</b>: " . $sMessage . " in <b>" . $sFile . "</b> on line <b>" . $iLine . "</b><br /><hr />" . $sXml . "<hr/>" . $sPhp);
		}

		return TRUE;
	}

	function pushUserObjParam($aParam) {
		array_push($this->aUserObjParamsStack, $aParam);
	}

	function pullUserObjParam() {
		array_pop($this->aUserObjParamsStack);
	}
	
	function pushForcedUserObjParam($aParam) {
		//$this->aForcedUserObjParamsStack[$sName] = $aParam;
		array_push($this->aForcedUserObjParamsStack, $aParam);
		return (count($this->aForcedUserObjParamsStack) - 1);
	}

	function pullForcedUserObjParam($iIndex = FALSE) {

		if($iIndex === FALSE) {
			if(!empty($this->aForcedUserObjParamsStack)) {
				array_pop($this->aForcedUserObjParamsStack);
			}
		} else {
			if(array_key_exists($iIndex, $this->aForcedUserObjParamsStack)) {
				unset($this->aForcedUserObjParamsStack[$sName]);
			}
		}
	}

	function getForcedUserObjParams() {
		$aParams = array();
		if(!empty($this->aForcedUserObjParamsStack)) {
			$aParams = $this->aForcedUserObjParamsStack[count($this->aForcedUserObjParamsStack) - 1];
		}

		return $aParams;
	}

	function getUserObjParams() {

		$aParams = array();

		if(!empty($this->aUserObjParamsStack)) {
			$aParams = $this->aUserObjParamsStack[count($this->aUserObjParamsStack) - 1];
		}

		if(!empty($this->aForcedUserObjParamsStack)) {
			$aForcedParams = $this->getForcedUserObjParams();
			$aParams = t3lib_div::array_merge_recursive_overrule($aParams, $aForcedParams);
		}

		return $aParams;
	}

	function getListData($sKey = FALSE) {
		return $this->oDataHandler->_getListData($sKey);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aUserobj: ...
	 * @param	[type]		$aParams: ...
	 * @param	[type]		$aItems: ...
	 * @param	[type]		$oRef: ...
	 * @param	[type]		$bAddDots: ...
	 * @return	[type]		...
	 */
	function _callUserObj($aUserobj, $aParams = array(), $aItems = FALSE, $oRef = FALSE, $bAddDots = TRUE) {

		if(is_array($this->_navConf("/userobj/", $aUserobj))) {

			$aUserObjParams = $this->_navConf("/userobj/params/", $aUserobj);

			if($aUserObjParams !== FALSE && is_array($aUserObjParams)) {

				while(list($index, $aParam) = each($aUserObjParams)) {

					$name = $aParam["name"];
					$value = $aParam["value"];

					$aParams[$name] = $value;
				}
				reset($aParams);
			}

			if(($sPhp = $this->_navConf("/userobj/php", $aUserobj)) !== FALSE) {

				$sClassName = uniqid("tempcl");
				$sMethodName = uniqid("tempmet");

				$this->__sEvalTemp = array("code" => $sPhp, "xml" => $aUserobj);
//				debug($sPhp, "ici");
				$GLOBALS['formidable_tempuserobj'] =& $this;
				$sPhp = str_replace("\$this", "\$GLOBALS['formidable_tempuserobj']", $sPhp);

				$sClass =	"class " . $sClassName . " {"
					.	"	function " . $sMethodName . "(\$_this, \$aParams) { \$_this=&\$GLOBALS['formidable_tempuserobj'];"
					.	"		" . $sPhp
					.	"	}"
					.	"}" ;

				set_error_handler(array(&$this, "__catchEvalException"));
				eval($sClass);
				$oObj = new $sClassName();

				$this->pushUserObjParam($aParams);

				$sRes = call_user_func(
					array(
						&$oObj,
						$sMethodName
					),
					$this,
					$aParams
				);

				$this->pullUserObjParam();

				unset($this->__sEvalTemp);
				restore_error_handler();

				return $sRes;

			} elseif(($this->_navConf("/userobj/cobj", $aUserobj)) !== FALSE) {
				if($this->__getEnvExecMode() === "BE") {
					return $this->cObj->cObjGetSingle(
						$aUserobj["userobj"]["cobj"],
						$aUserobj["userobj"]["cobj."]
					);
				} else {
					return $GLOBALS["TSFE"]->cObj->cObjGetSingle(
						$aUserobj["userobj"]["cobj"],
						$aUserobj["userobj"]["cobj."]
					);
				}
			} elseif(($sTs = $this->_navConf("/userobj/ts", $aUserobj)) !== FALSE) {

				$sTs = '
						temp.ameos_formidable >
						temp.ameos_formidable {
							' . $sTs . '
						}
				';

				require_once(PATH_t3lib."class.t3lib_tsparser.php");

				$oParser = t3lib_div::makeInstance("t3lib_tsparser");
				$oParser->tt_track = 0;	// Do not log time-performance information
				$oParser->setup = $GLOBALS["TSFE"]->tmpl->setup;

				if(array_key_exists("params.", $oParser->setup)) { unset($oParser->setup["params."]);}

				if($bAddDots) {
					$oParser->setup["params."] = $this->_addDots($aParams);
				} else {
					$oParser->setup["params."] = $aParams;
				}

				if($oRef !== FALSE && is_object($oRef) && is_array($oRef->aElement) && !array_key_exists("obj.", $oParser->setup["params."])) {
					$oParser->setup["params."]["obj."] = $this->_addDots($oRef->aElement);
				} else {
					$oParser->setup["params."]["obj."] = array();
				}

				if(($aUserObjParams = $this->_navConf("/userobj/params", $aUserobj)) !== FALSE) {
					if(is_array($aUserObjParams)) {
						
						if($this->isUserObj($aUserObjParams)) {
							$aUserObjParams = $this->_callUserObj($aUserObjParams);
							if(!is_array($aUserObjParams)) {
								$aUserObjParams = array();
							}
						}

						$oParser->setup["params."] = t3lib_div::array_merge_recursive_overrule(
							$oParser->setup["params."],
							$aUserObjParams
						);
					}
				}


				$oParser->parse($sTs);
				$this->aLastTs = $oParser->setup["temp."]["ameos_formidable."];

				$sOldCWD = getcwd();		// changing current working directory for use of GIFBUILDER in BE
				chdir(PATH_site);

				$aRes = $this->cObj->cObjGet(
					$oParser->setup["temp."]["ameos_formidable."]
				);
				chdir($sOldCWD);

				return $aRes;
			} else {

				$extension = $this->_navConf("/userobj/extension/", $aUserobj);
				$method = $this->_navConf("/userobj/method/", $aUserobj);



				if(strcasecmp($extension, "this") == 0)
				{ $oExtension =& $this->_oParent;}
				else
				{ $oExtension = t3lib_div::makeInstance($extension);}

				if(is_object($oExtension)) {

					if(method_exists($oExtension, $method)) {

						if($aItems !== FALSE) {
							// renderlet userobj
							$newData = $oExtension->{$method}($aItems, $aParams, $this);
						} else {
							$newData = $oExtension->{$method}($aParams, $this);
						}

						$this->_debug($newData, "RESULT OF " . $extension . "->" . $method . "()");

						return $newData;
					} else {

						$sObject = ($extension == "this") ? "\$this (<b>" . get_class($this->_oParent) . "</b>)" : $extension;
						$this->mayday($this->_navConf("/type/", $aElement) . " <b>" . $this->_navConf("/name/", $aElement) . "</b> : callback method <b>" . $method . "</b> of the Object <b>" . $sObject . "</b> doesn't exist");
					}
				}
			}
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$iStart: ...
	 * @param	[type]		$aData: ...
	 * @param	[type]		$bCycle: ...
	 * @param	[type]		$iDirection: ...
	 * @param	[type]		$bKey: ...
	 * @return	[type]		...
	 */
	function __getNeighbourInArray($iStart, $aData, $bCycle, $iDirection, $bKey = FALSE) {

		if(!empty($aData)) {

			$aKeys = array_keys($aData);
			$iPos = array_search(
				$iStart,
				$aKeys
			);

			$iNeighbourPos = intval($iPos+($iDirection));

			if(array_key_exists($iNeighbourPos, $aKeys)) {

				if($bKey !== FALSE) {
					return $aKeys[$iNeighbourPos];
				} else {
					return $aData[$aKeys[$iNeighbourPos]];
				}

			} elseif($bCycle) {
				if($iDirection > 0) {

					if($bKey !== FALSE) {
						return $aKeys[0];
					} else {
						return $aData[$aKeys[0]];
					}

				} else {

					if($bKey !== FALSE) {
						return $aKeys[(count($aKeys) - 1)];
					} else {
						return $aData[$aKeys[(count($aKeys) - 1)]];
					}
				}
			}
		}

		return FALSE;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$iStart: ...
	 * @param	[type]		$aData: ...
	 * @param	[type]		$bCycle: ...
	 * @param	[type]		$bKey: ...
	 * @return	[type]		...
	 */
	function _getNextInArray($iStart, $aData, $bCycle = FALSE, $bKey = FALSE) {
		return tx_ameosformidable::__getNeighbourInArray(
			$iStart,
			$aData,
			$bCycle,
			+1,
			$bKey
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$iStart: ...
	 * @param	[type]		$aData: ...
	 * @param	[type]		$bCycle: ...
	 * @param	[type]		$bKey: ...
	 * @return	[type]		...
	 */
	function _getPrevInArray($iStart, $aData, $bCycle = FALSE, $bKey = FALSE) {
		return tx_ameosformidable::__getNeighbourInArray(
			$iStart,
			$aData,
			$bCycle,
			-1,
			$bKey
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @param	[type]		$aConf: ...
	 * @return	[type]		...
	 */
	function _isTrue($sPath, $aConf = -1) {

		return $this->_isTrueVal(
			$this->_navConf(
				$sPath,
				$aConf
			)
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @param	[type]		$aConf: ...
	 * @return	[type]		...
	 */
	function _isFalse($sPath, $aConf = -1) {

		$mValue = $this->_navConf(
			$sPath,
			$aConf
		);

		if($mValue !== FALSE) {
			return $this->_isFalseVal($mValue);
		} else {
			return FALSE;	// if not found in conf, the searched value is not FALSE, so _isFalse() returns FALSE !!!!
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mVal: ...
	 * @return	[type]		...
	 */
	function _isTrueVal($mVal) {

		if($this->isUserObj($mVal)) {
			$mVal = $this->_callUserObj(
				$mVal
			);
		}

		return (($mVal === TRUE) || ($mVal == "1") || (strtoupper($mVal) == "TRUE"));
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mVal: ...
	 * @return	[type]		...
	 */
	function _isFalseVal($mVal) {

		if($this->isUserObj($mVal)) {
			$mVal = $this->_callUserObj(
				$mVal
			);
		}

		return (($mVal == FALSE) || (strtoupper($mVal) == "FALSE"));
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @param	[type]		$aConf: ...
	 * @return	[type]		...
	 */
	function _defaultTrue($sPath, $aConf = -1) {

		if($this->_navConf($sPath, $aConf) !== FALSE) {
			return $this->_isTrue($sPath, $aConf);
		} else {
			return TRUE;	// TRUE as a default
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @param	[type]		$aConf: ...
	 * @return	[type]		...
	 */
	function _defaultFalse($sPath, $aConf = -1) {

		if($this->_navConf($sPath, $aConf) !== FALSE) {
			return $this->_isTrue($sPath, $aConf);
		} else {
			return FALSE;	// FALSE as a default
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mInfos: ...
	 * @return	[type]		...
	 */
	function _getExtRelPath($mInfos) {

		if(!is_array($mInfos)) {
			// should be object type

			if(isset($this)) {
				$aInfos = $this->_getInfosRenderletForType($mInfos);
			} else {
				$aInfos = tx_ameosformidable::_getInfosForType(
					$mInfos,
					$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"]
				);
			}
		} else {
			$aInfos = $mInfos;
		}

		if($aInfos["BASE"] === TRUE) {
			return t3lib_extMgm::siteRelPath("ameos_formidable") . "api/base/" . $aInfos["EXTKEY"] . "/";
		} else {
			return t3lib_extMgm::siteRelPath($aInfos["EXTKEY"]);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mInfos: ...
	 * @return	[type]		...
	 */
	function _getExtPath($mInfos) {

		if(!is_array($mInfos)) {
			// should be object type

			if(isset($this)) {
				$aInfos = $this->_getInfosRenderletForType($mInfos);
			} else {
				$aInfos = tx_ameosformidable::_getInfosForType(
					$mInfos,
					$GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["renderlets"]
				);
			}
		} else {
			$aInfos = $mInfos;
		}

		if($aInfos["BASE"] === TRUE) {
			return PATH_formidable . "api/base/" . $aInfos["EXTKEY"] . "/";
		} else {
			return t3lib_extmgm::extPath($aInfos["EXTKEY"]);
		}

		return $extsrcpath;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aTags: ...
	 * @param	[type]		$aUserobjParams: ...
	 * @return	[type]		...
	 */
	function _getCustomTags($aTags, $aUserobjParams = array()) {

		$aCustomTags = array(
			"values" => array(),
			"labels" => array()
		);

		if(is_array($aTags)) {
			reset($aTags);
			while(list(, $aTag) = each($aTags))
			{
				$label = array_key_exists("label", $aTag) ? $aTag["label"] : "";
				$name = $aTag["name"];
				$value = $aTag["value"];

				if($this->isUserObj($aTag["value"])) {
					$value = $this->_callUserObj(
						$aTag["value"],
						$aUserobjParams
					);
				}

				if($value !== FALSE) {
					$aCustomTags["values"][$name] = $value;
					$aCustomTags["labels"][$name] = $label;
				}
			}
		}

		reset($aCustomTags);
		return $aCustomTags;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sName: ...
	 * @param	[type]		$mValue: ...
	 * @return	[type]		...
	 */
	function injectData($sName, $mValue) {
		$this->mayday("injectData is disabled");
		$this->_aInjectedData[$sName] = $mValue;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sName: ...
	 * @return	[type]		...
	 */
	function unsetInjectedData($sName) {

		if(array_key_exists($sName, $this->_aInjectedData)) {
			unset($this->_aInjectedData[$sName]);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function renderList() {

		if(!$this->bRendered) {
			$this->mayday("ATTEMPT TO CALL renderlist() BEFORE CALL TO render()");
		}

		if($this->oDataHandler->aObjectType["TYPE"] == "LISTER") {
			return $this->oDataHandler->sHtmlList;
		}

		return "";
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sStr: ...
	 * @return	[type]		...
	 */
	function _getSafeLock($sStr = FALSE) {

		if($sStr === FALSE) {
			$sStr = $this->conf["misc."]["safelockseed"];
		}

		return t3lib_div::shortMD5(
			$GLOBALS['TYPO3_CONF_VARS']['SYS']['encryptionKey'] . "||" . $sStr
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$rRes: ...
	 * @return	[type]		...
	 */
	function _watchOutDB($rRes, $sSql = FALSE) {

		if(!is_resource($rRes) && $GLOBALS["TYPO3_DB"]->sql_error()) {

			$sMsg = "SQL QUERY IS NOT VALID";
			$sMsg .= "<br/>";
			$sMsg .= "<b>" . $GLOBALS["TYPO3_DB"]->sql_error() . "</b>";
			$sMsg .= "<br />";
			
			if($sSql !== FALSE) {
				$sMsg .= $sSql;
			} else {
				$sMsg .= $GLOBALS["TYPO3_DB"]->debug_lastBuiltQuery;
			}

			$this->mayday($sMsg);
		}

		return $rRes;
	}

	// alias for __sendMail
	function sendMail($sAdresse, $sMessage, $sSubject, $sFromAd, $sFromName, $sReplyAd, $sReplyName, $aAttachPaths = array()) {
		if(is_object($this)) {
			return $this->__sendMail($sAdresse, $sMessage, $sSubject, $sFromAd, $sFromName, $sReplyAd, $sReplyName, $aAttachPaths);
		} else {
			return tx_ameosformidable::__sendMail($sAdresse, $sMessage, $sSubject, $sFromAd, $sFromName, $sReplyAd, $sReplyName, $aAttachPaths);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sAdresse: ...
	 * @param	[type]		$sMessage: ...
	 * @param	[type]		$sSubject: ...
	 * @param	[type]		$sFromAd: ...
	 * @param	[type]		$sFromName: ...
	 * @param	[type]		$sReplyAd: ...
	 * @param	[type]		$sReplyName: ...
	 * @param	[type]		$aAttachPaths: ...
	 * @return	[type]		...
	 */
	function __sendMail($sAdresse, $sMessage, $sSubject, $sFromAd, $sFromName, $sReplyAd, $sReplyName, $aAttachPaths = array()) {

		require_once(PATH_t3lib . "class.t3lib_htmlmail.php");

		if(is_object($this)) {
			if(($sDebugSendMail = $this->_navConf("/meta/debugsendmail")) !== FALSE) {
				$sAdresseOld = $sAdresse;
				$sAdresse = $sDebugSendMail;
				

				$sMessage .= "<hr />Formidable /meta/debugSendMail: This mail would be sent to " . $sAdresseOld;
			}
		}

		$oMail = t3lib_div::makeInstance("t3lib_htmlmail");
		$oMail->start();
		$oMail->useBase64();

		$oMail->subject			= $sSubject;
		$oMail->from_email		= $sFromAd;
		$oMail->from_name		= $sFromName;
		$oMail->replyto_email	= $sReplyAd;
		$oMail->replyto_name	= $sReplyName;
		$oMail->organisation	= '';
		$oMail->priority		= 3;

		// HTML
		$oMail->theParts['html']['content'] = $sMessage;	// Fetches the content of the page
		$oMail->theParts['html']['path'] = '';
		$oMail->extractMediaLinks();
		$oMail->extractHyperLinks();
		$oMail->fetchHTMLMedia();
		$oMail->substMediaNamesInHTML(0);	// 0 = relative
		$oMail->substHREFsInHTML();
		$oMail->setHTML($oMail->encodeMsg($oMail->theParts['html']['content']));




		// SET Attachements

		if(is_array($aAttachPaths) && !empty($aAttachPaths)) {

			require_once(PATH_t3lib . "class.t3lib_basicfilefunc.php");
			$oFile = t3lib_div::makeInstance("t3lib_basicFileFunctions");

			reset($aAttachPaths);
			while(list(, $sPath) = each($aAttachPaths)) {

				$sFilePath = t3lib_div::fixWindowsFilePath(
					$oFile->rmDoubleSlash(
						$sPath
					)
				);

				if(file_exists($sFilePath) && is_file($sFilePath) && is_readable($sFilePath)) {
					$oMail->addAttachment($sFilePath);
					$oMail->theParts["attach"][count($oMail->theParts["attach"]) - 1]["content_type"] = "application/force-download";
					$oMail->theParts["attach"][count($oMail->theParts["attach"]) - 1]["filename"] = basename($sFilePath);
				}
			}
		}

		// SET Headers and Content
		$oMail->setHeaders();
		$oMail->setContent();

		$oMail->setRecipient($sAdresse);
		$oMail->sendTheMail();
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sVarName: ...
	 * @param	[type]		$aData: ...
	 * @param	[type]		$bMultiLines: ...
	 * @return	[type]		...
	 */
	function _arrayToJs($sVarName, $aData, $bMultiLines=FALSE) {

		// deprecated; use array2json instead
		$aJs = array();
		$aJs[] = "var " . $sVarName . " = new Array();";

		$aJsSafe = array(
			rawurlencode("-"),
			rawurlencode("_"),
			rawurlencode("."),
			rawurlencode("!"),
			rawurlencode("~"),
			rawurlencode("*"),
		);

		$aJsSafeReplace = array(
			"-",
			"_",
			".",
			"!",
			"~",
			"*",
		);

		reset($aData);
		while(list($sKey, $mVal) = each($aData)) {
			$aJs[] = $sVarName . "[\"" . $sKey . "\"]=unescape(\"" . str_replace(array("%96", "%92"), array("", "'"), rawurlencode($mVal)) . "\");";
		}

		if($bMultiLines) {
			return "\n" . implode("\n", $aJs) . "\n";
		} else {
			return implode("", $aJs);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aData: ...
	 * @return	[type]		...
	 */
	function _arrayToRdtItems($aData, $sCaptionMap = FALSE, $sValueMap = FALSE) {
		
		$aItems = array();
		reset($aData);

		if($sCaptionMap !== FALSE && $sValueMap !== FALSE) {
			while(list($sKey, ) = each($aData)) {
				$aItems[] = array(
					"value" => $aData[$sKey][$sValueMap],
					"caption" => $aData[$sKey][$sCaptionMap],
				);
			}
		} else {
			while(list($sValue, $sCaption) = each($aData)) {
				$aItems[] = array(
					"value" => $sValue,
					"caption" => $sCaption
				);
			}
		}

		reset($aItems);
		return $aItems;
	}

	function _rdtItemsToArray($aData) {
		$aArray = array();
		
		reset($aData);
		while(list($iKey, ) = each($aData)) {
			$aArray[$aData[$iKey]["value"]] = $aData[$iKey]["caption"];
		}

		reset($aArray);
		return $aArray;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sCaptionKey: ...
	 * @param	[type]		$sValueKey: ...
	 * @param	[type]		$aData: ...
	 * @return	[type]		...
	 */
	function _arrayRowsToRdtItems($sCaptionKey, $sValueKey, $aData) {

		$aItems = array();

		reset($aData);
		while(list(, $aRow) = each($aData)) {
			$aItems[] = array(
				"value" => $aRow[$sValueKey],
				"caption" => $aRow[$sCaptionKey]
			);
		}

		reset($aItems);
		return $aItems;
	}

	function _tcaToRdtItems($aItems) {
		reset($aItems);
		return $this->_arrayToRdtItems($aItems, "0", "1");
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$iTemplateUid: ...
	 * @param	[type]		$iPageId: ...
	 * @return	[type]		...
	 */
	function _parseTsInBE($iTemplateUid, $iPageId) {

		require_once (PATH_t3lib."class.t3lib_page.php");
		require_once (PATH_t3lib."class.t3lib_tstemplate.php");
		require_once (PATH_t3lib."class.t3lib_tsparser_ext.php");


		global $tmpl;

		$tmpl = t3lib_div::makeInstance("t3lib_tsparser_ext");	// Defined global here!
		$tmpl->tt_track = 0;	// Do not log time-performance information
		$tmpl->init();

		// Gets the rootLine
		$sys_page = t3lib_div::makeInstance("t3lib_pageSelect");

		$tmpl->runThroughTemplates(
			$sys_page->getRootLine(
				$iPageId
			),
			$iTemplateUid
		);

		// This generates the constants/config + hierarchy info for the template.

//		$tplRow = $tmpl->ext_getFirstTemplate($pageId,$template_uid);	// Get the row of the first VISIBLE template of the page. whereclause like the frontend.

		$tmpl->generateConfig();

		$aConfig = $tmpl->setup["config."];
		reset($aConfig);
		return $aConfig;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$a1: ...
	 * @param	[type]		$a2: ...
	 * @return	[type]		...
	 */
	function array_add($a1, $a2) {

		if(is_array($a1)) {
			$aTemp = array(); $aTemp2 = array();
			reset($a1); reset($a2);

			while(list($key, $val) = each($a1)) {

				if($key != "type" && array_key_exists($key, $a2)) {

					$counter = 0;
					while(array_key_exists($key . "-" . $counter, $a2)) {
						$counter++;
					}

					$a2[$key . "-" . $counter] = $val;
				} else {
					$a2[$key] = $val;
				}
			}
		}
		reset($a2);
		return $a2;
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _perf() {
		return $this->end_tstamp - $this->start_tstamp;
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _makeJsonObj() {
		if(is_null($this->oJson)) {
			$this->oJson = t3lib_div::makeInstance("formidable_json");
			$this->oJson->use = SERVICES_JSON_LOOSE_TYPE;	// *decodes* objects as associative arrays
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aArray: ...
	 * @return	[type]		...
	 */
	function array2json($aArray) {
		$this->_makeJsonObj();
		return $this->oJson->encode($aArray);

	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sJson: ...
	 * @return	[type]		...
	 */
	function json2array($sJson) {
		$this->_makeJsonObj();
		return $this->oJson->decode($sJson);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$aArray: ...
	 * @param	[type]		$bFirst: ...
	 * @return	[type]		...
	 */
	function array2tree($aArray, $bFirst = TRUE) {

		$aNodes = array();
		while(list($sKey, $mVal) = each($aArray)) {
			if(is_array($mVal)) {
				$aNodes[] = array(
					"label" => $sKey,
					"nodes" => $this->array2tree($mVal, FALSE),
				);
			} else {

				$sLabel = (trim($sKey) !== "") ? trim($sKey) . ": " : "";

				$aNodes[] = array(
					"label" => $sLabel . trim($mVal)	// avoiding null values
				);
			}
		}

		if($bFirst && count(array_keys($aArray)) > 1) {
			return array(
				array(
					"label" => "Root",
					"nodes" => $aNodes,
				),
			);
		}

		return $aNodes;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sStr: ...
	 * @return	[type]		...
	 */
	function _strToHtmlChar($sStr) {

		$sOut = "";
		$iLen = strlen($sStr);

		for ($a=0; $a<$iLen; $a++) {
			$sOut .= '&#'.ord(substr($sStr, $a, 1)).';';
		}

		return $sOut;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sData: ...
	 * @param	[type]		$sKey: ...
	 * @param	[type]		$bFirstPos: ...
	 * @return	[type]		...
	 */
	function additionalHeaderData($sData, $sKey = FALSE, $bFirstPos = FALSE) {

		if(TYPO3_MODE === "FE") {
			$aHeaders =& $GLOBALS["TSFE"]->additionalHeaderData;
		} else {
			$aHeaders =& $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["context"]["be_headerdata"];
		}

		if($sKey === FALSE) {
			if($bFirstPos === TRUE) {
				$aHeaders = array(rand() => $sData) + $aHeaders;
			} else {
				$aHeaders[] = $sData;
			}
		} else {

			if($sKey == "ameosformidable_tx_rdttinymce") {
				if(!in_array($sData, $aHeaders)) {
					array_unshift($aHeaders, $sData);
				}
			} else {
				if($bFirstPos === TRUE) {
					$aHeaders = array($sKey => $sData) + $aHeaders;
				} else {
					$aHeaders[$sKey] = $sData;
				}
			}
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function getAdditionalHeaderData() {
		if(TYPO3_MODE === "FE") {
			return $GLOBALS["TSFE"]->additionalHeaderData;
		} else {
			return $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["context"]["be_headerdata"];
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$str: ...
	 * @param	[type]		$ext: ...
	 * @param	[type]		$sDesc: ...
	 * @return	[type]		...
	 */
	function inline2TempFile($str, $ext, $sDesc="")	{
		if($sDesc != "") {
			$sDesc = "\n\n<!-- FORMIDABLE: " . str_replace(array("<!--", "-->"), "", $sDesc) . " -->";
		}

		// Create filename / tags:
		$script = '';
		switch($ext) {
			case 'js': {

				$script = 'typo3temp/javascript_'.substr(md5($str),0,10).'.js';
				$output = $sDesc . "\n" . '<script type="text/javascript" src="'.htmlspecialchars(t3lib_div::getIndpEnv("TYPO3_SITE_URL") . $script).'"></script>' . "\n\n";
				break;
			}
			case 'css': {
				$script = 'typo3temp/stylesheet_'.substr(md5($str),0,10).'.css';
				$output = $sDesc . "\n" . '<link rel="stylesheet" type="text/css" href="'.htmlspecialchars(t3lib_div::getIndpEnv("TYPO3_SITE_URL") . $script).'" />' . "\n\n";
				break;
			}
		}

		// Write file:
		if($script){
			if(!@is_file(PATH_site.$script)) {
				t3lib_div::writeFile(PATH_site.$script, $str);
			}
		}

		return $output;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sKey: ...
	 * @return	[type]		...
	 */
	function issetAdditionalHeaderData($sKey) {
		if(TYPO3_MODE === "FE") {
			return isset($GLOBALS['TSFE']->additionalHeaderData[$sKey]);
		} else {
			return isset($this->_oParent->doc->inDocStylesArray[$sKey]);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function toWebPath($sPath) {
		return tx_ameosformidable::_removeEndingSlash(t3lib_div::getIndpEnv("TYPO3_SITE_URL")) . tx_ameosformidable::toRelPath($sPath);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function toRelPath($sPath) {

		if(substr($sPath, 0, 4) === "EXT:") {
			$sPath = t3lib_div::getFileAbsFileName($sPath);
		}

		$sPath = str_replace(t3lib_div::getIndpEnv("TYPO3_DOCUMENT_ROOT"), "", $sPath);

		if($sPath{0} != "/") {
			$sPath = "/" . $sPath;
		}

		/*$sPath = $this->_removeEndingSlash(t3lib_div::getIndpEnv("TYPO3_SITE_URL")) . $sPath;*/

		return $sPath;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function toServerPath($sPath) {

		$sPath = tx_ameosformidable::toRelPath($sPath);

		if(file_exists($sPath) && is_dir($sPath) && ($sPath{(strlen($sPath) - 1)} !== "/")) {
			$sPath .= "/";
		}

		return tx_ameosformidable::_removeEndingSlash(t3lib_div::getIndpEnv("TYPO3_DOCUMENT_ROOT")) . "/" .  tx_ameosformidable::_removeStartingSlash($sPath);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function _removeStartingSlash($sPath) {
		return ($sPath{0} === "/") ? substr($sPath, 1) : $sPath;
	}
	
	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function _removeEndingSlash($sPath) {
		return preg_replace('|/$|','', $sPath);
	}

	function _trimSlashes($sPath) {
		return $this->_removeStartingSlash(
			$this->_removeEndingSlash(
				$sPath
			)
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sPath: ...
	 * @return	[type]		...
	 */
	function isAbsServerPath($sPath) {
		$sServerRoot = t3lib_div::getIndpEnv("TYPO3_DOCUMENT_ROOT");
		return (substr($sPath, 0, strlen($sServerRoot)) === $sServerRoot);
	}

	function isAbsWebPath($sPath) {
		return (substr(strtolower($sPath), 0, 7) === "http://");
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function __getEnvExecMode() {

		if(TYPO3_MODE == "BE") {
			return "BE";
		} elseif(TYPO3_MODE == "FE") {
			if(is_null(t3lib_div::_GP('eID'))) {
				return "FE";
			} else {
				return "EID";
			}
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function __virtualizeFE($aConfig = FALSE) {

		if (!defined('PATH_tslib')) {
			if (@is_dir(PATH_site.TYPO3_mainDir.'sysext/cms/tslib/')) {
				define('PATH_tslib', PATH_site.TYPO3_mainDir.'sysext/cms/tslib/');
			} elseif (@is_dir(PATH_site.'tslib/')) {
				define('PATH_tslib', PATH_site.'tslib/');
			}
		}

		require_once(PATH_tslib.'class.tslib_content.php');
		require_once(PATH_t3lib.'class.t3lib_timetrack.php');
		require_once(PATH_tslib.'class.tslib_fe.php');
		require_once(PATH_t3lib.'class.t3lib_page.php');
		require_once(PATH_t3lib.'class.t3lib_userauth.php');
		require_once(PATH_tslib.'class.tslib_feuserauth.php');
		require_once(PATH_t3lib.'class.t3lib_tstemplate.php');
		require_once(PATH_t3lib.'class.t3lib_cs.php');

		$GLOBALS["TT"] = new t3lib_timeTrack;
		$GLOBALS["CLIENT"] = t3lib_div::clientInfo();

		// ***********************************
		// Create $TSFE object (TSFE = TypoScript Front End)
		// Connecting to database
		// ***********************************

		$temp_TSFEclassName = t3lib_div::makeInstanceClassName('tslib_fe');
		$GLOBALS["TSFE"] = new $temp_TSFEclassName(
			$GLOBALS["TYPO3_CONF_VARS"],
			t3lib_div::_GP('id'),
			t3lib_div::_GP('type'),
			t3lib_div::_GP('no_cache'),
			t3lib_div::_GP('cHash'),
			t3lib_div::_GP('jumpurl'),
			t3lib_div::_GP('MP'),
			t3lib_div::_GP('RDCT')
		);

		//$GLOBALS['TSFE']->absRefPrefix = "/";
		$GLOBALS["TSFE"]->connectToDB();
		$GLOBALS["TSFE"]->initFEuser();
		$GLOBALS["TSFE"]->determineId();
		$GLOBALS["TSFE"]->getCompressedTCarray();
		$GLOBALS["TSFE"]->initTemplate();

		if($aConfig === FALSE) {
			$GLOBALS["TSFE"]->getConfigArray();
		} else {
			$GLOBALS["TSFE"]->config = $aConfig;
		}

		$GLOBALS["TSFE"]->convPOSTCharset();
		$GLOBALS["TSFE"]->settingLanguage();
		$GLOBALS["TSFE"]->settingLocale();

		$GLOBALS["TSFE"]->cObj = t3lib_div::makeInstance('tslib_cObj');
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _storeFormInSession() {

		//$sHashKey = $this->_getSessionDataHashKey();

		$this->cleanBeforeSession();

		$GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"][$this->formid] = array(
			"object" => serialize($this),
			"xmlpath" => $this->_xmlPath,
			"runningobjects" => $this->__aRunningObjects,
			"sys_language_uid" => intval($GLOBALS["TSFE"]->sys_language_uid),
			"sys_language_content" => intval($GLOBALS["TSFE"]->sys_language_content),
			"tsfe_config" => $GLOBALS["TSFE"]->config,
			"pageid" => $GLOBALS["TSFE"]->id,
			"lang" => $GLOBALS["TSFE"]->lang,
			"spamProtectEmailAddresses" => $GLOBALS["TSFE"]->spamProtectEmailAddresses,
			"spamProtectEmailAddresses_atSubst" => $GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses_atSubst'],
			"spamProtectEmailAddresses_lastDotSubst" => $GLOBALS['TSFE']->config['config']['spamProtectEmailAddresses_lastDotSubst'],
			"parent" => FALSE,
		);

		if($this->bStoreParentInSession === TRUE) {

			$sClass = get_class($this->_oParent);
			$aParentConf = $GLOBALS["TSFE"]->tmpl->setup["plugin."][$sClass . "."];

			$GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"][$this->formid]["parent"] = array(
				"classpath" => $this->_removeEndingSlash(t3lib_div::getIndpEnv("TYPO3_DOCUMENT_ROOT")) . "/" . $this->_removeStartingSlash($aParentConf["includeLibs"]),
			);
		}
	}

	function cleanBeforeSession() {

		$this->oDataHandler->cleanBeforeSession();
		$this->oRenderer->cleanBeforeSession();
		
		reset($this->aORenderlets);
		while(list($sKey, ) = each($this->aORenderlets)) {
			$this->aORenderlets[$sKey]->cleanBeforeSession();
		}

		reset($this->aODataSources);
		while(list($sKey, ) = each($this->aODataSources)) {
			$this->aODataSources[$sKey]->cleanBeforeSession();
		}

		unset($this->oSandBox->oForm);
		unset($this->_oParent);
		unset($this->oParent);
		unset($this->oJs->oForm);

		$this->oSandBox = serialize($this->oSandBox);
		$this->aDebug = array();
		$this->_aSubXmlCache = array();
		$this->aInitTasksUnobtrusive = array();
		$this->aInitTasks = array();
		$this->aInitTasksOutsideLoad = array();
		$this->aInitTasksAjax = array();
		$this->aOnloadEvents = array();
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _clearFormInSession() {
		if(array_key_exists($this->formid, $GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"])) {
			unset($GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"][$this->formid]);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function _getSessionDataHashKey() {
		return $this->_getSafeLock(
			$GLOBALS["TSFE"]->id . "||" . $this->formid	// (unique but stable accross refreshes)
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$oRequest: ...
	 * @return	[type]		...
	 */
	function handleAjaxRequest(&$oRequest) {

		if($oRequest->aRequest["servicekey"] == "ajaxevent") {

			$this->oMajixEvent =& $oRequest;

			$sEventId = $oRequest->aRequest["eventid"];
			return $this->_callUserObj(
				$this->aAjaxEvents[$sEventId]["event"],
				$oRequest->aRequest["params"]
			);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sFormId: ...
	 * @return	[type]		...
	 */
	function &getFromContext($sFormId) {
		$sExecMode = $this->__getEnvExecMode();
		if($sExecMode === "EID") {
			// ajax context
			// getting form in session
			if(array_key_exists($sFormId, $GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"])) {
				return tx_ameosformidable::unHibernate(
					$GLOBALS["_SESSION"]["ameos_formidable"]["hibernate"][$sFormId]
				);
			} else {
				return FALSE;
			}
		} else {

		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$aHibernation: ...
	 * @return	[type]		...
	 */
	function &unHibernate(&$aHibernation) {
		tx_ameosformidable::loadRunningObjects($aHibernation);
		tx_ameosformidable::loadParent($aHibernation);

		$oForm = unserialize($aHibernation["object"]);
		$oForm->_includeSandBox();	// rebuilding class
		$oForm->oSandBox = unserialize($oForm->oSandBox);
		$oForm->oSandBox->oForm =& $oForm;

		$oForm->oDataHandler->oForm =& $oForm;
		$oForm->oRenderer->oForm =& $oForm;

		return $oForm;
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$aHibernation: ...
	 * @return	[type]		...
	 */
	function loadRunningObjects(&$aHibernation) {
		$aRObjects =& $aHibernation["runningobjects"];
		reset($aRObjects);
		while(list(, $aObject) = each($aRObjects)) {
			tx_ameosformidable::_loadObject(
				$aObject["internalkey"],
				$aObject["objecttype"]
			);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$aHibernation: ...
	 * @return	[type]		...
	 */
	function loadParent(&$aHibernation) {
		if($aHibernation["parent"] !== FALSE) {
			$sClassPath = $aHibernation["parent"]["classpath"];
			require_once($sClassPath);
		}
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sMethod: ...
	 * @param	[type]		$mData: ...
	 * @param	[type]		$sFormId: ...
	 * @param	[type]		$sElementId: ...
	 * @return	[type]		...
	 */
	function majixStatic($sMethod, $mData, $sFormId, $sElementId) {

		$aExecuter = $this->buildMajixExecuter(
			$sMethod,
			$mData,
			$sElementId
		);

		$aExecuter["formid"] = $sFormId;

		return $aExecuter;
	}

	function addMajixOnload($aMajixTasks) {
		$this->aOnloadEvents["client"][] = array(
			"name" => "Event added by addMajixOnload()",
			"eventdata" => $aMajixTasks
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sMethod: ...
	 * @param	[type]		$mData: ...
	 * @param	[type]		$sElementId: ...
	 * @return	[type]		...
	 */
	function buildMajixExecuter($sMethod, $mData, $sElementId) {
		return array(
			"method" => $sMethod,
			"data" => $mData,
			"object" => $sElementId,
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function majixSubmit() {

		return $this->buildMajixExecuter(
			"submitFull",
			null,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function majixSubmitRefresh() {
		return $this->majixRefresh();
	}

	function majixSubmitSearch() {
		return $this->buildMajixExecuter(
			"submitSearch",
			null,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @return	[type]		...
	 */
	function majixRefresh() {

		return $this->buildMajixExecuter(
			"submitRefresh",
			null,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sName: ...
	 * @return	[type]		...
	 */
	function majixScrollTo($sName) {

		return $this->buildMajixExecuter(
			"scrollTo",
			$sName,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sUrl: ...
	 * @return	[type]		...
	 */
	function majixSendToPage($sUrl) {
		return $this->buildMajixExecuter(
			"sendToPage",
			$sUrl,
			$this->formid
		);
	}

	function majixOpenPopup($mParams) {

		$aParams = array();

		if(is_string($mParams)) {
			$aParams["url"] = $mParams;
		} else {
			$aParams = $mParams;
		}

		return $this->buildMajixExecuter(
			"openPopup",
			$aParams,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sMessage: ...
	 * @return	[type]		...
	 */
	function majixDebug($sMessage) {
		return $this->buildMajixExecuter(
			"debug",
			$this->_viewMixed($sMessage),
			$this->formid
		);
	}

	function majixRequestNewI18n($sTableName, $iRecordUid, $iLangUid) {
		return $this->buildMajixExecuter(
			"requestNewI18n",
			array(
				"tablename" => $sTableName,
				"recorduid" => $iRecordUid,
				"languid" => $iLangUid,
				"hash" => $this->_getSafeLock("requestNewI18n" . ":" . $sTableName . ":" . $iRecordUid . ":" . $iLangUid),
			),
			$this->formid
		);
	}

	function majixRequestEdition($iRecordUid, $sTableName = FALSE) {

		if($sTableName === FALSE) {
			$sTableName = $this->oDataHandler->tablename();
		}

		if($sTableName !== FALSE) {
			return $this->buildMajixExecuter(
				"requestEdition",
				array(
					"tablename" => $sTableName,
					"recorduid" => $iRecordUid,
					"hash" => $this->_getSafeLock("requestEdition" . ":" . $sTableName . ":" . $iRecordUid),
				),
				$this->formid
			);
		}
	}

	function majixExecOnNextPage($aTask) {
		return $this->buildMajixExecuter(
			"execOnNextPage",
			$aTask,
			$this->formid
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sUrl: ...
	 * @return	[type]		...
	 */
	function xhtmlUrl($sUrl) {
		return str_replace("&", "&amp;", $sUrl);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sUrl: ...
	 * @return	[type]		...
	 */
	function sendToPage($sUrl) {
		if(is_numeric($sUrl)) {
			$sUrl = $this->toWebPath(
				$this->cObj->typolink_URL(array(
					"parameter" => $sUrl
				))
			);
		}

		header("Location: " . $sUrl);
		die();
	}

	function forceDownload($sAbsPath, $sFileName = FALSE) {

		if($sFileName === FALSE) {
			$sFileName = basename($sAbsPath);
		}

		header("Expires: Mon, 01 Jul 1997 00:00:00 GMT"); // some day in the past
		header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
		header("Content-type: application/x-download");
		header("Content-Disposition: attachment; filename=" . $sFileName);
		header("Content-Transfer-Encoding: binary");
		fpassthru(fopen($sAbsPath, "r"));
		die();
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$mMixed: ...
	 * @return	[type]		...
	 */
	function isUserObj($mMixed) {
		return is_array($mMixed) && array_key_exists("userobj", $mMixed);
	}

	function clearSearchForm($sSearchRdtName, $sFormId = FALSE) {
		if($sFormId === FALSE) {
			$sFormId = $this->formid;
		}

		if($sFormId === $this->formid) {
			if(array_key_exists($sSearchRdtName, $this->aORenderlets)) {
				$this->aORenderlets[$sSearchRdtName]->clearFilters();
				return;
			}
		}

		// else
		$GLOBALS["_SESSION"]["ameos_formidable"]["applicationdata"]["rdt_lister"][$sFormId][$sSearchRdtName]["criterias"] = array();
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$$oModule: ...
	 * @return	[type]		...
	 */
	function backendHeaders(&$oModule) {
		$oModule->content = str_replace(
			array(
				"<!--###POSTJSMARKER###-->",
				$oModule->doc->form,
			),
			array(
				"<!-- FORMIDABLE JS FWK begin-->\n" . implode("\n", $GLOBALS["TYPO3_CONF_VARS"]["EXTCONF"]["ameos_formidable"]["context"]["be_headerdata"]) . "\n<!-- FORMIDABLE JS FWK end-->\n<!--###POSTJSMARKER###-->",
				"<!-- DEFAULT FORM NEUTRALIZED BY FORMIDABLE " . $oModule->doc->form . "-->",
			),
			$oModule->content
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sStr: ...
	 * @return	[type]		...
	 */
	function convertAccents($sStr) {
		return html_entity_decode(
			preg_replace(
				'/&([a-zA-Z])(uml|acute|grave|circ|tilde|slash|ring|elig|cedil);/',
				'$1',
				htmlentities($sStr, ENT_COMPAT, "UTF-8")
			)
		);
	}

	/**
	 * [Describe function...]
	 *
	 * @param	[type]		$sStr: ...
	 * @return	[type]		...
	 */
	function removeNonAlnum($sStr) {
		// removes everything but a-z, A-Z, 0-9
		return ereg_replace(
			"[^<>[:alnum:]]",
			"",
			$sStr
		);
	}

	function generatePassword($iLength = 6) {
		
		$aLetters = array(
			"cons" => "aeiouy",
			"voy" => "bcdfghjklmnpqrstvwxz",
		);
		
		$sPassword = "";
		$sType = "cons";

		for($k=0; $k < $iLength; $k++) {
			$sType = ($sType === "cons") ? "voy" : "cons";
			$iNbLetters = strlen($aLetters[$sType]);
			$sPassword .= $aLetters[$sType]{rand(0, ($iNbLetters - 1))};
		}

		return $sPassword;
	}

	function div_autoLogin($iUserId) {

		if($this->__getEnvExecMode() === "FE") {
			$rSql = $GLOBALS["TYPO3_DB"]->exec_SELECTquery(
				"*",
				"fe_users",
				"uid='" . $iUserId . "'"
			);

			if(($aUser = $GLOBALS["TYPO3_DB"]->sql_fetch_assoc($rSql)) !== FALSE) {
				$GLOBALS["TSFE"]->fe_user->createUserSession($aUser);
				$GLOBALS["TSFE"]->fe_user->loginSessionStarted = TRUE;
				$GLOBALS["TSFE"]->fe_user->user = $GLOBALS["TSFE"]->fe_user->fetchUserSession();

				return TRUE;
			}
		}

		return FALSE;
	}

	function wrapImplode($sWrap, $aData, $sGlue = "") {
		$aRes = array();
		reset($aData);
		while(list($iKey,) = each($aData)) {
			if(is_string($aData[$iKey])) {
				$aRes[] = str_replace("|", $aData[$iKey], $sWrap);
			}
		}

		return implode($sGlue, $aRes);
	}

	function str_split($text, $split = 1){
		//place each character of the string into an array
		$array = array();
		for ($i=0; $i < strlen($text); $i++){
			$key = "";
			for ($j = 0; $j < $split; $j++){
				$key .= $text[$i+$j]; 
			}
			$i = $i + $j - 1;
			array_push($array, $key);
		}
		return $array;
	}

	function div_rteToHtml($sRteHtml, $sTable, $sColumn) {
		
		require_once(PATH_t3lib . "class.t3lib_rteapi.php");

		$pageTSConfig = $GLOBALS['TSFE']->getPagesTSconfig();

		$aConfig = $pageTSConfig['RTE.']['default.']['FE.'];
		$aSpecConf['rte_transform']['parameters'] = array(
			"flag" => "rte_enabled",
			"mode" => "ts"
		);
		
		$aDataArray = array(
			$sColumn => $sRteHtml,
		);

		
		return t3lib_rteapi::transformContent(
			'rte',
			$sRteHtml,
			$sTable,
			$sColumn,
			$aDataArray,
			$aSpecConf,
			$aConfig,
			'',
			0
		);
	}

	function devlog($sMessage, $iPad, $bCallStack = FALSE) {

/*		$sCallStack = "";
		if($bCallStack === TRUE) {
			$sCallStack = t3lib_div::debug_trail() . "\n";
		}

		error_log($sCallStack . str_repeat("\t", $iPad) . $sMessage . "\n", 3, $this->toServerPath("EXT:ameos_formidable/dev.log.txt"));*/
	}

	/**
	 * Method div_xml2array taken from the Developer API (api_macmade).
	 *
	 * (c) 2004 macmade.net
	 * All rights reserved
	 *
	 * The goal of this API is to provide to the Typo3 developers community
	 * some useful functions, to help in the process of extension development.
	 *
	 * It includes functions, for frontend, backend, databases and miscellaneous
	 * development.
	 *
	 * It's not here to replace any of the existing Typo3 core class or
	 * function. It just try to complete them by providing a quick way to
	 * develop extensions.
	 *
	 * Please take a look at the manual for a complete description of this API.
	 *
	 * @author		Jean-David Gadina (macmade@gadlab.net)
	 * @version		2.3
	 */

	/**
	 * Convert XML data to an array.
	 *
	 * This function is used to convert an XML data to a multi-dimensionnal array,
	 * representing the structure of the data.
	 *
	 * This function is based on the Typo3 array2xml function, in t3lib_div. It basically
	 * does the same, but has a few more options, like the inclusion of the xml tags arguments
	 * in the output array. This function also has support for same multiple tag names
	 * inside the same XML element, which is not the case with the core Typo3 function. In that
	 * specific case, the array keys are suffixed with '-N', where N is a numeric value.
	 *
	 * SPECIAL NOTE: This function can be called without the API class instantiated.
	 *
	 * @param	$data		The XML data to process
	 * @param	$keepAttribs		If set, also includes the tag attributes in the array (with key 'xml-attribs')
	 * @param	$caseFolding		XML parser option: case management
	 * @param	$skipWhite		XML parser option: white space management
	 * @param	$prefix		A tag prefix to remove
	 * @param	$numeric		Keep only the numeric value for a tag prefixed with this argument (default is 'n')
	 * @param	$index		Set the tag name to an alternate value found in the tag arguments (default is 'index')
	 * @param	$type		Force the tag value to a special type, found in the tag arguments (default is 'type')
	 * @param	$base64		Decode the tag value from base64 if the specified tag argument is present (default is 'base64')
	 * @param	$php5defCharset		The default charset to use with PHP5
	 * @return	An		array with the XML structure, or an XML error message if the data is not valid
	 */
	function div_xml2array($data,$keepAttribs=1,$caseFolding=0,$skipWhite=0,$prefix=FALSE,$numeric='n',$index='index',$type='type',$base64='base64',$php5defCharset='UTF-8') {

		// Function ID
		$funcId = 'div_xml2array';

		// Storage
		$xml = array();
		$xmlValues = array();
		$xmlIndex = array();
		$stack = array(array());

		// Counter
		$stackCount = 0;

		// New XML parser
		$parser = xml_parser_create();

		// Case management option
		xml_parser_set_option($parser,XML_OPTION_CASE_FOLDING,$caseFolding);

		// White space management option
		xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE,$skipWhite);

		// Support for PHP5 charset detection
		if ((double) phpversion() >= 5) {

			// Find the encoding parameter in the XML declaration
			ereg('^[[:space:]]*<\?xml[^>]*encoding[[:space:]]*=[[:space:]]*"([^"]*)"',substr($data,0,200),$result);

			// Check result
			if ($result[1]) {

				// Charset found in the XML declaration
				$charset = $result[1];

			} else if ($TYPO3_CONF_VARS['BE']['forceCharset']) {

				// Force charset to Typo3 configuration if defined
				$charset = $TYPO3_CONF_VARS['BE']['forceCharset'];

			} else {

				// Default charset
				$charset = $php5defCharset;
			}

			// Charset management option
			xml_parser_set_option($parser,XML_OPTION_TARGET_ENCODING,$charset);
		}

		// Parse XML structure
		xml_parse_into_struct($parser,$data,$xmlValues,$xmlIndex);

		// Error in XML
		if (xml_get_error_code($parser)) {

			// Error
			$error = 'XML error: ' . xml_error_string(xml_get_error_code($parser)) . ' at line ' . xml_get_current_line_number($parser);

			// Free XML parser
			xml_parser_free($parser);

			// Return error
			return $error;

		} else {

			// Free XML parser
			xml_parser_free($parser);

			// Counter for multiple same keys
			$sameKeyCount = array();

			// Process each value
			while(list($key,$val) = each($xmlValues)) {

				// lower-case on tagName
				$val['tag'] = strtolower($val['tag']);
				// lower-case on attribute name
				if(array_key_exists("attributes", $val)) {
					$val["attributes"] = array_change_key_case($val["attributes"], CASE_LOWER);
				}

				// Get the tag name (without prefix if specified)
				$tagName = ($prefix && t3lib_div::isFirstPartOfStr($val['tag'], $prefix)) ? substr($val['tag'],strlen($prefix)) : $val['tag'];

				$aTagName = explode(
					":",
					$tagName
				);

				if(sizeof($aTagName) > 1) {

					// debug($aTagName[1]);

					$type = $aTagName[1];
					$tagName = $aTagName[0];

					$val['attributes']['type'] = strtoupper($type);
				}


				// Support for numeric tags (<nXXX>)
				$numTag = (substr($tagName,0,1) == $numeric) ? substr($tagName,1) : FALSE;

				// Check if tag is a real numeric value
				if ($numTag && !strcmp(intval($numTag),$numTag)) {

					// Store only numeric value
					$tagName = intval($numTag);
				}

				// Support for alternative value
				if (strlen($val['attributes'][$index])) {

					// Store alternate value
					$tagName = $val['attributes'][$index];
				}

				// Check if array key already exists
				if (array_key_exists($tagName,$xml)) {

					// Check if the current level has already a key counter
					if (!isset($sameKeyCount[$val['level']])) {

						// Create array
						$sameKeyCount[$val['level']] = 0;
					}

					// Increase key counter
					$sameKeyCount[$val['level']]++;

					// Change tag name to avoid overwriting existing values
					$tagName = $tagName . '-' . $sameKeyCount[$val['level']];
				}

				//debug($tagName);

				



				// Check tag type
				switch($val['type']) {

					// Open tag
					case 'open':

						// Storage
						$xml[$tagName] = array();



						// Memorize content
						$stack[$stackCount++] = $xml;

						// Reset main storage
						$xml = array();

						// Support for tag attributes
						if ($keepAttribs && $val['attributes']) {
//							echo t3lib_div::view_array($val['attributes'], "attributes");
							$xml = $val['attributes'];
						}


					break;

					// Close tag
					case 'close':

						// Memorize array
						$tempXML = $xml;

						// Decrease the stack counter
						$xml = $stack[--$stackCount];

						// Go to the end of the array
						end($xml);

						// Add temp array
						if(!empty($tempXML)) {
							$xml[key($xml)] = $tempXML;
						}

						// Unset temp array
						unset($tempXML);

						// Unset key counters for the child level
						unset($sameKeyCount[$val['level'] + 1]);

					break;

					// Complete tag
					case 'complete':

						// Check for base64
						if ($val['attributes']['base64']) {

							// Decode value
							$xml[$tagName] = base64_decode($val['value']);

						} else {

							// Add value (force string)
							if(
								array_key_exists("value", $val) != ""
								&&
								$tagName != "0"
							) {
								$xml[$tagName] = (string) $val['value'];
							} else {
								$xml[$tagName] = "";
							}

							// Support for value types
							switch((string)$val['attributes'][$type]) {

								// Integer
								case 'integer':

									// Force variable type
									$xml[$tagName] = (integer) $xml[$tagName];

								break;

								// Double
								case 'double':

									$xml[$tagName] = (double) $xml[$tagName];

								break;

								// Boolean
								case 'boolean':

									// Force type
									$xml[$tagName] = (bool) $xml[$tagName];

								break;

								// Array
								case 'array':

									// Create an empty array
									$xml[$tagName] = array();

								break;
							}
						}

						// Memorize tag value
//						$tempTagValue = $xml[$tagName];

						// New array with value
//						$xml[$tagName] = $tempTagValue;

						// Support for tag attributes
						if ($keepAttribs && $val['attributes']) {

							// Store attributes
							if(is_array($xml[$tagName])) {
								$xml[$tagName] = array_merge($xml[$tagName], $val['attributes']);
							} else {
								$xml[$tagName] = array_merge(
									$val['attributes'],
									array(
										"__value" => $val["value"]
									)
								);
							}

							// Unset memorized value
							unset($tempTagValue);
						}

					break;
				}
			}

			// Return the array of the XML root element
			return $xml;
		}
	}

	function smartMayday_XmlFile($sPath, $sMessage = FALSE) {

		$sVersion = $this->_getVersion();
		$sXml =<<<XMLFILE
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<formidable version="{$sVersion}"
	xmlns:datahandler="http://www.ameos.com/formidable/{$sVersion}/datahandler"
	xmlns:datasource="http://www.ameos.com/formidable/{$sVersion}/datasource"
	xmlns:renderer="http://www.ameos.com/formidable/{$sVersion}/renderer"
	xmlns:renderlet="http://www.ameos.com/formidable/{$sVersion}/renderlet"
	xmlns:validator="http://www.ameos.com/formidable/{$sVersion}/validator"
	xmlns:actionlet="http://www.ameos.com/formidable/{$sVersion}/actionlet">
	
	<meta>
		<name>Standard XML file</name>
		<form formid="myform"/>
		<debug>false</debug>
		<displaylabels>true</displaylabels>
	</meta>
	
	<control>
		<datahandler:STANDARD/>
		<renderer:STANDARD/>
	</control>

	<elements>
		<renderlet:TEXT name="mytxt" label="Some text field"/>
	</elements>

</formidable>
XMLFILE;

		if($sMessage === FALSE) {
			$sMessage = "FORMIDABLE CORE - The given XML file path (<b>" . $sPath . "</b>) does not exist";
		}

		$sXml = htmlspecialchars($sXml);
		$sMayday =<<<ERRORMESSAGE

	<div>{$sMessage}</div>
	<hr />
	<div>This basic XML might be usefull: </div>
	<br />
	<div style='color: black; background-color: #e6e6fa; border: 2px dashed #4682b4; font-family: Courier; padding-left: 20px;'>
		<br />
<pre>{$sXml}</pre>
		<br /><br />
	</div>

ERRORMESSAGE;
		
		$this->mayday($sMayday);
	}
}

	if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/class.tx_ameosformidable.php"])	{
		include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/ameos_formidable/api/class.tx_ameosformidable.php"]);
	}
?>